/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
/** 
    \mainpage O2scl User's Guide

    \htmlonly
    <div align="right"><a href="dl_page.html"><b>Download
    O<span style='position: relative; top: 0.3em; 
    font-size: 0.8em'>2</span>scl</b></a></div>
    \endhtmlonly

    \o2 is a C++ class library for object-oriented numerical
    programming. It includes
    - Classes based on numerical routines from GSL and CERNLIB
    - Vector and matrix classes which are fully compatible with
    \c gsl_vector and \c gsl_matrix, yet offer indexing with
    \c operator[] and other object-oriented features
    - The CERNLIB-based classes are rewritten in C++ and are often 
    faster than their GSL counterparts
    - Classes which require function inputs are designed to accept
    (public or private) member functions, even if they are virtual.
    - Classes use templated vector types, which allow the use of 
    object-oriented vectors or C-style arrays.
    - Highly compatible - Recent versions have been tested on 
    Linux (32- and 64-bit systems, with Intel and AMD chips), 
    Windows XP with Cygwin, and MacOSX.
    - Free! \o2 is provided under Version 3 of the GNU 
    Public License
    - Two mini-libraries
    <ul>
    <li>Thermodynamics of ideal and nearly-ideal particles with
    quantum statistics</li>
    <li>Equations of state for finite density relevant for 
    neutron stars</li>
    </ul>

    This is a beta version. The library should install and 
    test successfully, and most of the classes are ready
    for production use. Some of the interfaces may change
    slightly in future versions. There are a few classes
    which are more experimental, and this is clearly stated
    at the top of the documentation for these classes. 

    See licensing information at \ref license_section. 

    \htmlonly
    <hr />
    <a href="dl_page.html">Download 
    O<span style='position: relative; top: 0.3em; font-size: 
    0.8em'>2</span>scl</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    <a href="../latex/refman.pdf">PDF documentation</a>
    \endhtmlonly

    \hline
    \section toc_section Quick Reference to User's Guide
    
    - \ref install_section
    - \ref usage_section
    - \ref examples_section
    - \ref related_section
    - \ref cplxnum_section
    - \ref vecmat_section
    - \ref permute_section
    - \ref linalg_section
    - \ref intp_section
    - \ref const_section
    - \ref funct_section
    - \ref table_section
    - \ref string_section
    - \ref diff_section
    - \ref inte_section
    - \ref poly_section
    - \ref solve_section
    - \ref min_section
    - \ref conmin_section
    - \ref mcarlo_section
    - \ref anneal_section
    - \ref fit_section
    - \ref ode_section
    - \ref rng_section
    - \ref tintp_section
    - \ref gslcheb_section
    - \ref unitconv_section
    - \ref other_section
    - \ref lset_section
    - \ref io_section
    - \ref source_section
    - \ref design_section
    - \ref license_section
    - \ref ack_section
    - \ref ref_section
    - \ref todo 
    - \ref bug

    \htmlonly
    <UL><LI>Other mini-libraries
    <UL>
    <LI><B><a href="../part/html/index.html">Particles</a></b>
    <LI><B><a href="../eos/html/index.html">Equations of State</a></b>
    </UL>
    </LI>
    <br />
    <li>Add-on library: There is also a related library, 
    <a href='../../o2scl_ext/html/index.html'>O<span style='position: relative; top: 0.3em; font-size: 0.8em'>2</span>scl_ext</a>,
    which has a separate source distribution.
    </li>
    </UL>
    \endhtmlonly

    \hline
    \section install_section Installation

    The rules for installation are generally the same as that for
    other GNU libraries. The file \c INSTALL has some details on this
    procedure. Generally, you should be able to run
    <tt>./configure</tt> and then type \c make and \c make \c
    install. More information on the \c configure command can also be
    obtained from <tt>./configure --help</tt>. \o2 requires the GSL
    libraries. If the <tt>configure</tt> script cannot find them, you
    may have to specify their location in the <tt>CPPFLAGS</tt> and
    <tt>LDFLAGS</tt> environment variables (<tt>./configure
    --help</tt> shows some information on this). The documentation is
    automatically installed by \c make \c install.

    After \c make \c install, you may test the library with \c make \c
    o2scl-test. At the moment, testing \o2 requires that you have
    <tt>grep, awk, tail, cat,</tt> and <tt>wc</tt> on your machine to
    summarize the test results, but this will hopefully be improved in
    later version.

    This library requires GSL and is designed to work with GSL
    versions 1.12 or greater. Some classes may work with older
    versions of GSL, but this cannot be guaranteed.

    Range-checking for vectors and matrices is performed similar to
    the GSL approach, and is turned on by default. You can 
    disable range-checking by defining -DO2SCL_NO_RANGE_CHECK
    \code
    CPPFLAGS="-DO2SCL_NO_RANGE_CHECK" ./configure
    \endcode

    The separate libraries \o2e and \o2p are installed by default. To
    disable the installation of these libraries and their associated
    documentation, run <tt>./configure</tt> with the flags \c
    --disable-eoslib or \c --disable-partlib.  Note that \o2e depends
    on \o2p so using \c --disable-partlib without \c --disable-eoslib
    will not work.

    There are several warning flags that are useful when configuring
    and compiling with \o2. See the GSL documentation for an 
    excellent discussion, and also see the generic installation
    documentation in the file <tt>INSTALL</tt> in the \o2 top-level 
    directory. For running \c configure, for example, if you do
    not have privileges to write to <tt>/usr/local</tt>,
    \code
    CPPFLAGS="-O3" -I/home/asteiner/install/include" \
    LDFLAGS="-L/home/asteiner/install/lib" ./configure -C \
    --prefix=/home/asteiner/install
    \endcode
    In this example, specifying
    <tt>-I/home/asteiner/install/include</tt> and
    <tt>-L/home/asteiner/install/lib</tt> above ensures that the GSL
    libraries can be found (this is where they are installed on my
    machine). The <tt>--prefix=/home/asteiner/install</tt> argument to
    <tt>./configure</tt> ensures that \o2 is installed there as well.

    The documentation is generated with \doxygen. In principle, the 
    documentation can be regenerated by the end-user, but this is 
    not supported and requires several external applications
    not included in the distribution.

    \b Un-installation: While there is no explicit "uninstall"
    procedure, there are only a couple places to check. Installation
    creates directories named <tt>o2scl</tt> in the include, doc and shared
    files directory (which default to \c /usr/local/include, \c
    /usr/local/doc, and \c /usr/local/share) which can be
    removed. Finally, all of the libraries are named with the prefix
    \c libo2scl and are created by default in /usr/local/lib. As
    configured with the settings above, the files are in \c
    /home/asteiner/install/include/o2scl, \c /home/asteiner/install/lib,
    \c /home/asteiner/install/share/o2scl, and \c
    /home/asteiner/install/doc/o2scl.

    \hline
    \section usage_section General Usage

    \subsection namespace_subsect Namespaces
    
    Most of the classes reside in the namespace \c o2scl (this
    namespace has been removed from the documentation for
    clarity). Numerical constants (many of them based on the GSL
    constants) are placed in separate namespaces (gsl_cgs, gsl_cgsm,
    gsl_mks, gsl_mksa, gsl_num, o2scl_const, and o2scl_fm).  There are
    also two namespaces which hold integration coefficients,
    o2scl_inte_qag_coeffs and o2scl_inte_qng_coeffs.

    \subsection docconv_subsect Documentation conventions

    In the following documentation, function parameters are denoted by
    \c parameter, except when used in mathematical formulas as in \f$
    \mathrm{variable} \f$ .
    
    \subsection nomen_subsect Nomenclature

    Classes directly derived from the GNU Scientific Library are
    preceeded by the prefix \c gsl_ and classes derived from CERNLIB
    are preceeded by the prefix \c cern_. Some of those classes 
    derived from GSL and CERNLIB operate slightly differently 
    from the original versions. The differences are detailed
    in the corresponding class documentation.
    
    \subsection errorhand_subsect Basic error handling

    Error handling is similar to GSL. When an error occurs, functions
    and/or classes call a GSL-like error handler and (when
    appropriate) return a non-zero value. When functions succeed they
    return 0 (\ref gsl_success). The error handler, \ref err_hnd is a
    global pointer to an object of type \ref err_base. There is a
    global default error handler of type \ref err_class. The list of
    error codes is given in the documentation for the file \ref
    err_hnd.h. The default error handler can be replaced by simply
    assigning the address of a descendant of \ref err_base to \ref
    err_hnd. Most of the time, if a function returns an integer value,
    and the returned value is nonzero, then the error handler was
    called (at least once). Functions can return success even when the
    error handler was called during execution, and this happens
    particularly in template classes where templated methods are
    beyond the direct control of the function using them. Also, \o2
    functions never reset the error handler. 

    The default behavior for all errors is to store the error
    information, print the information to <tt>cout</tt> and call
    <tt>exit(error_number)</tt>. You can modify the behavior of the
    default error handler with the err_class::set_mode() function, 
    Mode '2' is the default, mode '1' just prints the information, 
    and mode '0' ignores the error. 

    Although \o2 does not yet have this functionality by default, it
    is striaghtforward to define an error handler which throws a C++
    exception. Also, object destructors do not generally call the
    error handler. Internally, \o2 does not use <tt>try</tt> blocks,
    but these can easily be effectively employed by an \o2 user. 

    Errors can be set by the user through the macros \ref O2SCL_ERR,
    which sets an error, and \ref O2SCL_ERR_RET, which sets an error
    and returns the error number.

    Functionality similar to assert() is provided with the macro
    \ref O2SCL_ASSERT, which exits if its argument is non-zero, and 
    \ref O2SCL_BOOL_ASSERT which exits if its argument is false.

    Note that the GSL function <tt>gsl_set_error_handler()</tt> is
    called in the abstract base class err_base to set the GSL error
    handler any time a new instance of \ref err_base is created. If
    the user creates a new handler, that new handler will be used by
    GSL functions as well.
    
    \subsection errorhand2_subsect What is an error?

    \o2 assumes that errors are events which should
    happen infrequently. Error handling strategies are often
    time-consuming and they are not a replacement for normal code
    flow. However, even with this in mind, one can still distinguish a
    large spectrum of posibillities from "fatal" errors, those likely
    to corrupt the stack and/or cause a dreaded "segmentation fault"
    and "non-fatal" errors, those errors which might cause incorrect
    results, but might be somehow recoverable. One of the purposes of
    error handling is to decide if and how these different types of
    errors should be handled differently. Most errors in \o2 result
    in a call to the error handler by default. Some of the classes
    which attempt to reach numerical convergence have an option (e.g.
    mroot::err_nonconv) to turn this default behavior off for
    convergence errors (which are not necessarily "fatal" errors). To
    set these "convergence" errors, the macros \ref O2SCL_CONV and
    \ref O2SCL_CONV_RET can be used.

    Another related issue is that \o2 often calls functions which are
    supplied by the user, these user-designed functions may create
    errors, and the library needs to decide how to deal with them,
    even though it knows little about what is actually happening
    inside these user-defined functions. Most of the time, \o2 assumes
    that if a function returns a nonzero value, then an error has
    occured and the present calculation should abort.

    \subsection objscope_subsect Objects and scope
    
    \o2 objects frequently take inputs which are of the form of a
    reference to a smaller object. This is particularly convenient
    because it allows a lot of flexibility, while providing a certain
    degree of safety. However, the user retains the responsibility of
    ensuring that input objects do not go out of scope before they are
    utilized by objects which require them. (This is actually no
    different than the requirements on the user imposed by GSL, for
    example.)

    A simple example of this is provided by the simulated annealing
    class \ref gsl_anneal. Simulated annealing works by providing a
    temperature annealing schedule, which slowly decreases the
    temperature. However, if the temperature schedule object goes out
    of scope before the annealing is done, a segmentation fault will
    result. For example, the following code will fail:
    \include scope_doc.cpp

    This will fail because the temperature schedule object goes out of
    scope (it's a local variable in the function and its destructor is
    called before the <tt>set_schedule()</tt> function exits) before
    the <tt>mmin()</tt> function is called.

    \hline
    \section examples_section Compiling examples
    
    The example programs are in the \c examples directory. After
    installation, they can be compiled and executed by running \c make
    \c o2scl-examples in that directory. This will also test the
    output of the examples to make sure it is correct and summarize
    these tests in \c examples-summary.txt. The output for each
    example is placed in the corresponding file with a <tt>.scr</tt>
    extension.

    Alternatively, you can make the executable for each example in the
    \c examples directory individually using, e.g. <tt>make
    ex_mroot</tt>.

    See \ref source_section for the documented source code
    of the individual examples

    Also, the testing code for each class is sometimes useful for
    providing examples of their usage.  The testing source code for
    each source file is named with an \c _ts.cpp prefix in the same
    directory as the class source.
      
    \hline
    \section related_section Related projects

    Several noteworthy related projects:

    - GSL - The GNU Scientific Library \n The first truly free,
    ANSI-compliant, fully tested, and well-documented scientific
    computing library. The GSL is located at
    http://www.gnu.org/software/gsl . Manual is available at
    http://www.gnu.org/software/gsl/manual/html_node/ . Many GSL
    routines are included and reworked in \o2 (the corresponding
    classes begin with the \c gsl_ prefix) and \o2 was specifically
    designed to be used with GSL. GSL is a must-have for most serious
    numerical work in C or C++ and is required for installation of
    \o2.

    - CERNLIB - The gold standard in FORTRAN computing libraries \n
    Several CERNLIB routines are rewritten completely in C++ and
    included in \o2 (they begin with the \c cern_ prefix). CERNLIB is
    located at http://cernlib.web.cern.ch/cernlib/mathlib.html

    - LAPACK and ATLAS - The gold standard for linear algebra \n
    Available at http://www.netlib.org/lapack and
    http://www.netlib.org/atlas . See also
    http://www.netlib.org/clapack .

    - QUADPACK - FORTRAN adaptive integration library \n This is the
    library on which the GSL integration routines are based (prefix \c
    gsl_inte_). It is available at http://www.netlib.org/quadpack .

    - TNT - Template numerical toolkit (http://math.nist.gov) \n TNT
    provides vector and matrix types and basic arithmetic operations.
    Most of the classes in \o2 which use vector and matrix types are
    templated to allow compatbility with TNT. (Though there are a few
    small differences.) This software is in the public domain.
    
    - Blitz++ - http://www.oonumerics.org/blitz \n Another linear
    algebra library designed in C++ which includes vector and matrix
    types and basic arithmetic. As the name implies, Blitz++ has the
    capability to be particularly fast. Distributed with a Perl-like
    artistic license "or the GPL".

    - Boost - http://www.boost.org \n Free (license is compatible with
    GPL) peer-reviewed portable C++ source libraries that work well
    with the C++ Standard Library. Boost also contains uBlas,
    for linear algebra computing. For examples of how to use
    \o2 with uBlas vector and matrices, see \ref ex_ublas_sect.
    
    - FLENS - http://flens.sourceforge.net \n A C++ library with
    object-oriented linear algebra types and an interface to BLAS
    and LAPACK. 

    - MESA - Modules for Experiments in Stellar Astrophysics
    (http://mesa.sourceforge.net) \n An excellent FORTRAN library with
    accurate low-density equations of state, interpolation, opacities
    and other routines useful in stellar physics. Work is currently
    under way to rewrite some of the MESA routines in C/C++ for \o2.
    Licensed with LGPL (not GPL). 

    - OOL - Open Optimization Library (http://ool.sourceforge.net) \n
    Constrained minimization library designed with GSL in mind. The
    \o2 constrained minimization classes are derived from this 
    library. See \ref conmin_section .

    - Root - CERN's new C++ analysis package (http://root.cern.ch) \n
    A gargantuan library for data analysis, focused mostly on
    high-energy physics. Their histograms, graphics, file I/O and
    support for large data sets is particuarly good.

    \hline
    \section cplxnum_section Complex Numbers
    
    Several arithmetic operations for \c gsl_complex are
    defined in cx_arith.h, but no constructor has been written. The
    object \c gsl_complex is still a \c struct, not a \c class. For
    example,
    \include cx_arith_doc.cpp

    In case the user needs to convert between \c gsl_complex and \c
    std::complex<double>, two conversion functions gsl_to_complex()
    and complex_to_gsl() are also provided in \ref cx_arith.h.

    \hline
    \section vecmat_section Arrays, Vectors, Matrices and Tensors

    \subsection vmintro_subsect Introduction

    The \o2 library uses a standard nomenclature to distinguish
    a couple different concepts. The word "array" is always used to
    refer to C-style arrays, i.e. \c double[]. If there are two
    dimensions in the array, it is a "two-dimensional array", i.e. \c
    double[][] . The word "vector" is reserved generic objects with
    array-like semantics, i.e. any type of object or class which can
    be treated similar to an array in that it has an function \c
    operator[] which gives array-like indexing. Thus arrays are
    vectors, but not all vectors are arrays. There are a couple
    specific vector types defined in \o2 like \ref ovector and \ref
    uvector . The STL vector <tt>std::vector<double></tt> is also a
    vector type in this language. The word "matrix" is reserved for
    the a generic object which has matrix-like semantics and can be
    accessed using either \c operator() or two successive applications
    of \c operator[] (or sometimes both). The \o2 objects \ref omatrix
    and \ref umatrix are matrix objects, as is a C-style
    two-dimensional array, \c double[][] . The header files are named
    in this spirit also: functions and classes which operate on 
    generic vector types are in \ref vector.h and functions and
    classes which work only with C-style arrays are in \ref array.h .
    The word "tensor" is used for a generic object which has 
    rank \c n and then has \c n associated indices. A vector is just
    a \tensor of rank 1 and a matrix is just a \tensor of rank 2. 

    Most of the classes in \o2 which use vectors and/or matrices are
    designed so that they can be used with any appropriately-defined
    vector or matrix types. This is a major part of the design goals
    for \o2 and most of the classes are compatible with matrix and
    vector objects from GSL, TNT, MV++, uBlas, and Blitz++.

    The first index of a matrix type is defined always to be the index
    associated with "rows" and the second is associated with
    "columns". The \o2 matrix output functions respect this notation
    as well, so that all of the elements of the first row is sent to
    the screen, then all of the elements in the second row, and so on.
    With this in mind, one can make the distinction between
    "row-major" and "column-major" matrix storage. C-style 
    two-dimensional arrays are "row-major" in that the elements
    of the first row occupy the first locations in memory as 
    opposed "column-major" where the first column occupies the 
    first locations in memory. It is important to note that the 
    majority of the classes in \o2 do not care about the details
    of the underlying memory structure, so long as two
    successive applications of <tt>operator[]</tt> (or
    in some cases <tt>operator(,)</tt> ) works properly. The storage
    format used by \ref omatrix and \ref umatrix is row-major, and there
    are no column-major matrix classes in \o2 (yet).

    \subsection index_subsect Matrix indexing with [][] vs. (,)

    While vector indexing with \c operator[] is very compatible with
    almost any vector type, matrix indexing is a bit more difficult.
    There are two options: assume matrix objects provide an \c
    operator[] method which can be applied twice, i.e.
    <tt>m[i][j]</tt>, or assume that matrix elements should be
    referred to with <tt>m(i,j)</tt>. Most of the \o2 classes use the
    former approach so that they are also compatible with
    two-dimensional arrays. However, there are sometimes good reasons
    to want to use <tt>operator()</tt> for matrix-intensive operations
    from linear algebra. For this reason, some of the functions given
    in the <tt>linalg</tt> directory are specified in two forms: the
    first default form which assumes <tt>[][]</tt>, and the second
    form with the same name, but in a namespace which has a suffix
    <tt>_paren</tt> and assumes matrix types use <tt>(,)</tt>.

    \subsection rowcol_subsect Rows and columns vs. x and y

    Sometimes its useful to think about the rows and columns in a
    matrix as referring to a elements of a grid, and the matrix
    indices refer to points in a grid in \f$ (x,y) \f$. It might seem
    intuitive to think of a matrix as \c A[ix][iy] where \c ix and \c
    iy are the \f$ x \f$ and \f$ y \f$ indices because the ordering of
    the indices is alphabetical. However, it is useful to note that
    because functions like \ref matrix_out print the first "row" first
    rather than the first column, a matrix constructed as \c A[ix][iy]
    will be printed out with x on the "vertical axis" and y on the
    "horizontal axis", so it is sometimes useful to store data
    in the form \c A[iy][ix] (for example, in the two dimensional 
    interpolation class, twod_intp). In any case, all classes 
    which take matrix data as input will document how the matrix
    ought to be arranged.

    \subsection genvec_subsection Generic vector functions
    
    There are a couple functions which operate on generic vectors of
    any type in \ref vector.h . They perform sorting, summing,
    rotating, copying, and computations of minima and maxima. For
    more statistically-oriented operations, see also \ref vec_stats.h .
    
    \subsection nativetypes_subsect Native matrix and vector types

    The rest of this section in the User's guide is dedicated to
    describing the \o2 implementations of vector, matrix, 
    and \tensor types.

    Vectors and matrices are designed using the templates \ref
    ovector_tlate and \ref omatrix_tlate, which are compatible with \c
    gsl_vector and \c gsl_matrix. Vectors and matrices with unit
    stride are provided in \ref uvector_tlate and \ref umatrix_tlate.
    The most commonly used double-precision versions of these template
    classes are \ref ovector, \ref omatrix, \ref uvector and \ref
    umatrix, and their associated "views" (analogous to GSL vector and
    matrix views) which are named with a \c _view suffix. The classes
    \ref ovector_tlate and \ref omatrix_tlate offer the syntactic
    simplicity of array-like indexing, which is easy to apply to
    vectors and matrices created with GSL.

    The following sections primarily discuss the operation objects of
    type \ref ovector. The generalizations to objects of type \ref
    uvector, \ref omatrix, and the complex vector and matrix objects
    \ref ovector_cx, \ref omatrix_cx, \ref uvector_cx, and \ref
    umatrix_cx are straightforward.

    \subsection view_subsect Vector and matrix views

    Vector and matrix views are provided as parents of the vector and
    matrix classes which do not have methods for memory allocation. As
    in GSL, it is simple to "view" normal C-style arrays or pointer
    arrays (see \ref ovector_array), parts of vectors (\ref
    ovector_subvector), and rows and columns of matrices (\ref
    omatrix_row and \ref omatrix_col). Several operations are defined,
    including addition, subtraction, and dot products.

    \subsection typedef_subsect Vector and matrix typedefs

    Several typedefs are used to give smaller names to often used
    templates. Vectors and matrices of double-precision numbers all
    begin with the prefixes \ref ovector and \ref omatrix. Integer
    versions begin with the prefixes \ref ovector_int and \ref
    omatrix_int. Complex versions have an additional \c "_cx",
    e.g. \ref ovector_cx. See \ref ovector_tlate.h, \ref
    ovector_cx_tlate.h, \ref omatrix_tlate.h, and \ref
    omatrix_cx_tlate.h.

    \subsection unit_subsect Unit-stride vectors

    The \ref uvector_tlate objects are naturally somewhat faster
    albeit less flexible than their finite-stride counterparts.
    Conversion to GSL vectors and matrices is not trivial for \ref
    uvector_tlate objects, but demands copying the entire
    vector. Vector views, operators, and the nomenclature is similar
    to that of \ref ovector. 

    \subsection memalloc_subsect Memory allocation

    Memory for vectors can be allocated using ovector::allocate() and
    ovector::free(). Allocation can also be performed by the
    constructor, and the destructor automatically calls <tt>free()</tt> if
    necessary. In contrast to \c gsl_vector_alloc(),
    ovector::allocate() will call ovector::free(), if necessary, to
    free previously allocated space. Allocating memory does not clear
    the recently allocated memory to zero.  You can use \ref
    ovector::set_all() with a zero argument to clear a vector
    (and similarly for a matrix).

    If the memory allocation fails, either in the constructor or in
    allocate(), then the error handler will be called, partially
    allocated memory will be freed, and the size will be reset to
    zero. 

    Although memory allocation in \o2 vectors is very similar to 
    that in GSL, the user must not mix allocation and deallocation
    between GSL and \o2.
    
    \subsection getset_subsect Get and set
    
    Vectors and matrices can be modified using ovector::get() and
    ovector::set() methods analogous to \c gsl_vector_get() and \c
    gsl_vector_set(), or they can be modified through
    ovector::operator[] (or ovector::operator() ), e.g.
    \comment
    For some reason the references to ovector::operator have been
    failing in the Doxygen compilation.
    \endcomment
    \code
    ovector a(4);
    a.set(0,2.0);
    a.set(1,3.0);
    a[2]=4.0;
    a[3]=2.0*a[1];
    \endcode
    
    If you want to set all of the values in an \ref ovector or an \ref
    omatrix at the same time, then use ovector::set_all() or
    omatrix::set_all().

    \subsection rangecheck_subsect Range checking
    
    Range checking is performed depending on whether or not \c
    O2SCL_NO_RANGE_CHECK is defined. It can be defined in the
    arguments to \c ./configure upon installation to turn off range
    checking. Note that this is completely separate from the GSL range
    checking mechanism, so range checking may be on in \o2 even if
    it has been turned off in GSL. Range checking is used primarily
    in the vector, matrix, and \tensor \c get() and \c set() methods.

    To see if range checking was turned on during installation
    (without calling the error handler), use
    lib_settings_class::range_check().

    Note that range checking in \o2 code is most often present in
    header files, rather than in source code. This means that range
    checking can be turned on or off in user-defined functions
    separately from whether or not it was used in the library classes
    and functions.

    \subsection copy_subsect Shallow and deep copy

    Copying \o2 vectors using constructors or the \c = operator
    is performed according to what kind of object is
    on the left-hand side (LHS) of the equals sign. If the LHS is a
    view, then a shallow copy is performed, and if the LHS is a \ref
    ovector, then a deep copy is performed. If an attempt is made to
    perform a deep copy onto a vector that has already been allocated,
    then that previously allocated memory is automatically freed. 
    The user must be careful to ensure that information is not lost
    this way, even though no memory leak will occur.
    
    For generic deep vector and matrix copying, you can use the
    template functions \ref vector_copy(), \ref matrix_copy(). These
    would allow you, for example, to copy an \ref ovector to a \c
    std::vector<double> object. These functions do not do any
    memory allocation so that must be handled beforehand by the user.

    \subsection vecarith_subsect Vector and matrix arithmetic
    
    Several operators are available as member functions of the
    corresponding template:
    
    Vector_view unary operators:
    - vector_view += vector_view
    - vector_view -= vector_view
    - vector_view += scalar
    - vector_view -= scalar
    - vector_view *= scalar
    - scalar = norm(vector_view)

    Matrix_view unary operators:
    - matrix_view += matrix_view
    - matrix_view -= matrix_view
    - matrix_view += scalar
    - matrix_view -= scalar
    - matrix_view *= scalar

    Binary operators like addition, subtraction, and matrix
    multiplication are also defined for \ref ovector, \ref uvector,
    and related objects. The generic template for a binary operator,
    e.g.
    \code
    template<class vec_t> vec_t &operator+(vec_t &v1, vec_t &v2);
    \endcode
    is difficult because the compiler has no way of distinguishing vector 
    and non-vector classes. At the moment, this is solved by 
    creating a define macro for the binary operators. In addition
    to the predefined operators for native classes, the user may 
    also define binary operators for other classes using the same
    macros. For example, 
    \code
    O2SCL_OP_VEC_VEC_ADD(o2scl::ovector,std::vector<double>,
    std::vector<double>)
    \endcode
    would provide an addition operator for \ref ovector and vectors
    from the Standard Template Library. A full list of the 
    operators which are defined by default and the associated macros
    are detailed in the documentation for \ref vec_arith.h. 

    The GSL BLAS routines can also be used directly with \ref ovector
    and \ref omatrix objects.

    Note that some of these arithmetic operations succeed even with
    non-matching vector and matrix sizes. For example, adding a 3x3
    matrix to a 4x4 matrix will result in a 3x3 matrix and the 7 outer
    elements of the 4x4 matrix are ignored.

    \subsection gslconv_subsect Converting to and from GSL forms

    Because of the way \ref ovector is constructed, you may use
    type conversion to convert to and from objects of type \c gsl_vector.
    \code
    ovector a(2);
    a[0]=1.0;
    a[1]=2.0;
    gsl_vector *g=(gsl_vector *)(&a);
    cout << gsl_vector_get(g,0) << " " << gsl_vector_get(g,1) << endl;
    \endcode
    Or,
    \code
    gsl_vector *g=gsl_vector_alloc(2);
    gsl_vector_set(0,1.0);
    gsl_vector_set(1,2.0);
    ovector &a=(ovector &)(*g);
    cout << a[0] << " " << a[1] << endl;
    \endcode
    This sort of type-casting is discouraged among unrelated classes,
    but is permissible here because ovector_tlate is a descendant of
    gsl_vector.  In particular, this will not generate "type-punning"
    warnings in later gcc versions.  If this bothers your
    sensibilities, however, then you can use the following approach:
    \code
    ovector a(2);
    gsl_vector *g=a.get_gsl_vector();
    \endcode
    The ease of converting between these two kind of objects makes it
    easy to use gsl functions on objects of type \ref ovector, i.e.
    \code
    ovector a(2);
    a[0]=2.0;
    a[1]=1.0;
    gsl_vector_sort((gsl_vector *)(&a));
    cout << a[0] << " " << a[1] << endl;
    \endcode

    \subsection stlconv_subsect Converting from STL form

    To "view" a \c std::vector<double>, you can use \ref ovector_array
    \code
    std::vector<double> d;
    d.push_back(1.0);
    d.push_back(3.0);
    ovector_array aa(d.size,&(d[0]));
    cout << aa[0] << " " << aa[1] << endl;
    \endcode

    However, you should note that if the memory for the \c std::vector
    is reallocated (for example because of a call to \c push_back()),
    then a previously created \ref ovector_view will be incorrect.

    \subsection pushpop_subsect push_back() and pop() methods
    
    These two functions give a behavior similar to the corresponding
    methods for \c std::vector<>. This will work in \o2 classes,
    but may not be compatible with all of the GSL functions. This 
    will break if the address of a \ref ovector_tlate is given
    to a GSL function which accesses the \c block->size parameter
    instead of the \c size parameter of a \c gsl_vector. Please 
    contact the author of \o2 if you find a GSL function with this
    behavior. 
    
    \subsection view_subsect Views

    Views are slightly different than in GSL in that they are now
    implemented as parent classes. Effectively, this means that vector
    views are just the same as normal vectors, except that they have
    no memory allocation functions (because the memory is presumably
    owned by a different object or scope). The code
    \code
    double x[2]={1.0,2.0};
    gsl_vector_view_array v(2,x);
    gsl_vector *g=&(v.vector);
    gsl_vector_set(g,0,3.0);
    cout << gsl_vector_get(g,0) << " " << gsl_vector_get(g,1) << endl;
    \endcode
    can be replaced by
    \code 
    double x[2]={1.0,2.0};
    ovector_array a(2,x);
    a[0]=3.0;
    cout << a << endl;
    \endcode

    \subsection vecparm_subsect Passing ovector parameters
    
    It is often best to pass an \ref ovector as a const reference
    to an \ref ovector_view, i.e.
    \code
    void function(const ovector_view &a);
    \endcode
    If the function may change the values in the \ref ovector,
    then just leave out \c const
    \code
    void function(ovector_view &a);
    \endcode
    This way, you ensure that the function is not allowed to modify
    the memory for the vector argument.

    If you intend for a function (rather than the user) to handle 
    the memory allocation, then some care is necessary. The
    following code
    \include ovector_doc1.cpp
    is confusing because the user may have already allocated memory
    for \c a. To avoid this, you may want to ensure that the user
    sends an empty vector. For example,
    \include ovector_doc2.cpp
    In lieu of this, it is often preferable to use a local variable for
    the storage and offer a \c get() function, 
    \include ovector_doc3.cpp
    The \o2 classes run into this situation quite frequently, but
    the vector type is specified through a template
    \include ovector_doc4.cpp

    \subsection eqoper_subsect Vectors and operator=()

    An "operator=(value)" method for setting all vector elements to
    the same value is not included because it leads to confusion
    between, \c ovector_tlate::operator=(const data_t &val) and
    ovector_tlate::ovector_tlate(size_t val) For example, after
    implementing \c operator=() and executing the following
    \code
    ovector_int o1=2;
    ovector_int o2;
    o2=2;
    \endcode
    \c o1 will be a vector of size two, and \c o2 will be an
    empty vector! 

    To set all of the vector elements to the same value, use
    ovector_tlate::set_all().  Because of the existence of
    constructors like ovector_tlate::ovector_tlate(size_t val), the
    following code
    \code
    ovector_int o1=2;
    \endcode
    still compiles, and is equivalent to 
    \code
    ovector_int o1(2);
    \endcode
    while the code
    \code
    ovector_int o1;
    o1=2;
    \endcode
    will not compile. As a matter of style, \c ovector_int \c o1(2);
    is preferable to \c ovector_int \c o1=2; to avoid confusion.

    \subsection matstruct_subsect Matrix structure
    
    The matrices from \ref omatrix_tlate
    are structured in exactly the same way as in GSL.
    For a matrix with 2 rows, 4 colums, and a "tda" or "trailing
    dimension" of 7, the memory for the matrix is structured in the
    following way:
    
    \verbatim
    00 01 02 03 XX XX XX
    10 11 12 13 XX XX XX
    \endverbatim
    
    where \c XX indicates portions of memory that are unreferenced. The tda
    can be accessed through, for example, the method
    omatrix_view_tlate::tda(). The \c get(size_t,size_t) methods
    always take the row index as the first argument and the column
    index as the second argument. The matrices from \ref umatrix_tlate
    have a trailing dimension which is always equal to the number of
    columns.

    \subsection rev_subsect Reversing the order of vectors

    You can get a reversed vector view from \ref
    ovector_reverse_tlate, or uvector_reverse_tlate. For these
    classes, \c operator[] and related methods are redefined to
    perform the reversal. If you want to make many calls to these
    indexing methods for a reversed vector, then simply copying the
    vector to a reversed version may be faster.
    
    \subsection constvec_subsect Const-correctness with vectors

    Vector objects, like \ref ovector, can be const in the usual way.
    However because vector views, like \ref ovector_view are like
    pointers to vectors, there are two ways in which they can be
    const. The view can point to a const vector, or the view can be a
    'const view' which is fixed to point only to one vector (whether
    or not the vector pointed to happens to be const). This is exactly
    analogous to the distinction between const double *p, double *
    const p, and const double * const p. The first is a non-const
    pointer to const data, the second is a const pointer to non-const
    data, and the third is a const pointer to const data. The
    equivalent expressions in \o2 are
    \code
    ovector_const_view v1;
    const ovector_view v2;
    const ovector_const_view v3;
    \endcode
    Explicitly,
    - <tt>ovector_const_view</tt> is a view of a const vector, the
    view may change, but the vector may not.
    - <tt>const ovector_const_view</tt> is a const view of a const
    vector, the view may not point to a different vector and the
    vector may not change.
    - <tt>const ovector_view</tt> is a const view of a normal vector,
    the view may not change, but the vector can.
    This same distinction is also present, for example, in ublas vectors
    views within boost. 
    
    A reference of type \ref ovector_base is often used as a function
    argument, and can hold either a \ref ovector or a \ref
    ovector_view. The important rule to remember with \ref ovector_base
    is that, a const reference, i.e.
    \code
    const ovector_base &v;
    \endcode
    is always a const reference to data which cannot be changed. A
    normal \ref ovector_base reference does not refer to const data. 

    \comment

    Vector objects, like \ref ovector, can be const in the usual
    way. However, in order to properly prevent the user from
    making a non-const view of a const vector, A separate 
    view class \ref ovector_const_view was created. To demonstrate,
    the following code is allowed
    \code
    ovector x(2);
    const ovector_view y=x;
    \endcode
    However, if the original vector is <tt>const</tt>, then the
    \ref ovector_const_view class should be used
    \code
    const ovector x(2);
    const ovector_const_view y=x;
    \endcode

    \endcomment

    \subsection vmout_subsect Vector and matrix output

    Both the \ref ovector_view_tlate and \ref omatrix_view_tlate
    classes have an &lt;&lt; operator defined for each class. For
    writing generic vectors to a stream, you can use \ref vector_out()
    which is defined in \ref vector.h . Pretty matrix output is
    performed by global template functions \ref matrix_out(), \ref
    matrix_cx_out_paren(), and \ref matrix_out_paren() which are
    defined in \ref columnify.h .

    \subsection tensor_subsect Tensors

    Some preliminary support is provided for tensors of arbitrary rank
    and size in the class \ref tensor. Classes \ref tensor1, \ref
    tensor2, \ref tensor3, and \ref tensor4 are rank-specific versions
    for 1-, 2-, 3- and 4-rank tensors. For n-dimsional data defined on
    a grid, \ref tensor_grid provides a space to define a hyper-cubic
    grid in addition to the the tensor data. This class \ref
    tensor_grid also provides n-dimensional interpolation of the data
    defined on the specified grid.

    \hline
    \section permute_section Permutations

    Permutations are implemented through the \ref permutation class.
    This class is fully compatible with gsl_permutation objects since
    it is inherited from gsl_permutation_struct. The class also
    contains no new data members, so upcasting and downcasting can
    always be performed. It is perfectly permissible to call
    GSL permutation functions from \ref permutation objects by 
    simply passing the address of the permutation, i.e.
    \code
    permutation p(4);
    p.init();
    gsl_permutation_swap(&p,2,3);
    \endcode

    The functions which apply a permutation to a
    user-specified vector are member template functions in the \ref
    permutation class (see \ref permutation::apply() ).

    Memory allocation/deallocation between the class and the
    gsl_struct is compatible in many cases, but mixing these forms is
    strongly discouraged, i.e. avoid using
    <tt>gsl_permutation_alloc()</tt> on a \ref permutation object, but
    rather use \ref permutation::allocate() instead.  The use of \ref
    permutation::free() is encouraged, but any remaining memory will 
    be deallocated in the object destructor.

    \hline
    \section linalg_section Linear algebra

    There is a small set of linear algebra routines. These are not
    intended to be a replacement for higher performance linear algebra
    libraries, but are designed for use by \o2 routines so that they
    work for generic matrix and vector types. For vector and matrix
    types using \c operator[], the BLAS and linear algebra routines
    are inside the \ref o2scl_cblas and \ref o2scl_linalg namespaces.
    For vector and matrix types using \c operator(), the BLAS and
    linear algebra routines routines are inside the \ref
    o2scl_cblas_paren and \ref o2scl_linalg_paren namespaces.

    The linear algebra classes and functions include:
    - Householder transformations (\ref householder.h)
    - Householder solver (\ref hh.h)
    - LU decomposition and solver (\ref lu.h)
    - QR decomposition and solver (\ref qr.h)
    - Solve tridiagonal systems (\ref tridiag.h)
    - Lanczos diagonalization is inside class \ref lanczos, which
    also can compute the eigenvalues of a tridiagonal matrix. 
    - Givens rotations (\ref givens.h)

    There is also a set of linear solvers for generic matrix and
    vector types which descend from \ref o2scl_linalg::linear_solver.
    These classes provide GSL-like solvers, but are generalized so
    that they are compatible with vector and matrix types which allow
    access through <tt>operator[]</tt>.

    For users who require high-performance linear algebra, the \ref
    ovector and \ref omatrix objects can be used to call LAPACK
    routines directly, just as can be done with GSL. For an example of
    how to do this, see
    http://sourceware.org/ml/gsl-discuss/2001/msg00326.html . For
    uBlas users, there is an example of how to use uBlas vectors with
    \o2 in \ref ex_ublas_sect. Finally, there are also a couple of
    examples, <tt>gesvd.cpp</tt> and <tt>zheev.cpp</tt> in the
    <tt>src/internal</tt> directory which show how to call LAPACK with
    \o2 objects which may be adaptable for your platform and
    configuration.

    \hline
    \section intp_section Interpolation
    
    The classes o2scl_interp and o2scl_interp_vec allow basic
    interpolation, lookup, differentiation, and integration of data
    given in two ovectors or ovector views. In contrast to the GSL
    routines, data which is presented with a decreasing independent
    variable is handled automatically. For interpolation with
    arrays rather than ovectors, use array_interp or array_interp_vec.

    For fast interpolation of arrays where the independent variable
    is strictly increasing and without error-checking, you can directly
    use the children of \ref base_interp. 

    \subsection twoint_subsect The two interpolation interfaces

    The difference between the two classes, o2scl_interp and
    o2scl_interp_vec, analogous to the difference between using \c
    gsl_interp_eval() and \c gsl_spline_eval() in \c GSL. You can
    create a o2scl_interp object and use it to interpolate among any
    pair of chosen vectors, i.e.
    \code
    ovector x(20), y(20);
    // fill x and y with data
    o2scl_interp oi;
    double y_half=oi.interp(0.5,20,x,y);
    \endcode
    Alternatively, you can create a o2scl_interp_vec object which can be
    optimized for a pair of vectors that you specify in advance
    \code
    ovector x(20), y(20);
    // fill x and y with data
    o2scl_interp_vec oi(20,x,y);
    double y_half=oi.interp(0.5);
    \endcode

    \subsection search_subsect Lookup and binary search

    The class search_vec contains a searching functions for objects of
    type \ref ovector which are monotonic. Note that if you want to
    find the index of an \ref ovector where a particular value is
    located without any assumptions with regard to the ordering, you
    can use ovector::lookup() which performs an exhaustive search.

    \subsection smintp_subsect "Smart" interpolation
    
    The classes smart_interp and smart_interp_vec allow interpolation,
    lookup, differentiation, and integration of data which is
    non-monotonic or multiply-valued outside the region of interest.
    As with o2scl_interp above, the corresponding array versions are
    given in sma_interp and sma_interp_vec.

    \subsection lowinterp_subsect Lower-level interpolation objects

    There are five lower-level interpolation objects which 
    specify the basic \o2 interpolation types. They are
    - \ref linear_interp
    - \ref cspline_interp
    - \ref cspline_peri_interp
    - \ref akima_interp
    - \ref akima_peri_interp

    \subsection interp_mgr_subsect Interpolation manager objects

    Many \o2 classes require the ability to create and destroy
    interpolation objects at will, given one of the lower-level
    interpolation types described above. For this purpose,
    interpolation managers have been created, which allow the user to
    provide a class with an interpolation manager associated with a
    low-level interpolation type (and thus free the user from having
    to worry about the associated memory management). The abstract
    base interpolation manager type is \ref base_interp_mgr, and the
    default interpolation manager object is \ref def_interp_mgr . The
    \ref def_interp_mgr class has two template arguments: the first is
    the vector type to be interpolated, and the second is the
    lower-lever interpolation type.
    
    For an example usage of the default interpolation manager, see 
    the \ref ex_contour_sect , which specifies an interpolation manager
    for the \ref contour class. The \ref table class also works with
    interpolation manager objects (see \ref table::set_interp() ).

    \subsection highdintp_subsect Two and higher dimensional interpolation

    Support for two-dimensional interpolation (by successive
    one-dimensional interpolations) is given in \ref twod_intp (see
    \ref tintp_section, and a simple version of n-dimensional
    interpolation in \ref tensor_grid.

    \hline
    \section const_section Physical constants
    
    The constants from GSL are reworked with the type \c const \c
    double and placed in namespaces called gsl_cgs, gsl_cgsm, gsl_mks,
    gsl_mksa, and gsl_num. Some additional constants are given in the
    namespace o2scl_const. Some of the numerical values have been
    updated from recently released data from NIST.

    \hline
    \section funct_section Function Objects

    Functions are passed to numerical routines using template-based
    function classes, sometimes called "functors". There are several
    basic kinds of function
    objects:
    - \ref funct : One function of one variable
    - \ref multi_funct : One function of several variables
    - \ref mm_funct : \c n functions of \c n variables
    - \ref fit_funct : One function of one variable with \c n 
    fitting parameters
    - \ref ode_funct : \c n derivatives as a function of \c n function
    values and the value of the independent variable
    - \ref jac_funct : Jacobian function for solver
    - \ref grad_funct : Gradient function for minimizers
    - \ref ool_hfunct : Hessian product for constrained minimization

    For each of these classes (except \ref funct), there is a version
    named \c _vfunct instead of \c _funct which is designed to be used
    with C-style arrays instead of vector classes.

    The class name suffixes denote children of a generic function type
    which are created using different kinds of inputs:
    - _fptr: function pointer for a static or global function
    - _gsl: GSL-like function pointer 
    - _mfptr: function pointer template for a class member function
    - _cmfptr: function pointer template for a class member function which
    is const
    - _strings: functions specified using strings, e.g. "x^2-2",
    which are in the separate \o2x package.
    - _noerr: (for \ref funct and multi_funct) a function which 
    directly returns the function value rather than returning
    an integer error value
    - _nopar: (for \ref funct) a function which has no parameters and
    directly returns the function value.

    See the \ref ex_fptr_sect and the \ref ex_mroot_sect which
    provide detailed examples of how functions can be specified 
    to classes through these function objects.

    There is a small overhead associated with the indirection: a "user
    class" accesses the function class which then calls function which
    was specified in the constructor of the function class. In many
    problems, the overhead associated with the indirection is small.
    Some of this overhead can always be avoided by inheriting directly
    from the function class and thus the user class will make a direct
    virtual function call. To eliminate the overhead entirely, one can
    specify a new type for the template parameter in the user class.

    Note that virtual functions can be specified through 
    this mechanism as well. For example, if \ref cern_mroot
    is used to solve a set of equations specified as
    \include fptr_doc.cpp
    Then the solver will solve the member function in the derived
    type, not the parent type. 

    Note also that providing a user access to a function object
    instantiatied with a protected or private member function is
    (basically) the same as providing them access to that function.

    \hline
    \section table_section Data tables
    
    The class \ref table is a container to hold and perform operations
    on related columns of data. It supports column operations,
    interpolation, column reference by either name or index, binary
    searching (in the case of ordered columns), sorting, and fitting
    two columns to a user-specified function. 

    \hline
    \section string_section String manipulation

    There are a couple classes and functions to help manipulate
    strings of text. Conversion routines for \c std::string 
    objects are given in \ref string_conv.h and include
    - \ref ptos() - pointer to string
    - \ref itos() - integer to string
    - \ref dtos() - double to string
    - \ref stoi() - string to integer
    - \ref stod() - string to double

    See also \ref size_of_exponent(), \ref format_float,
    and \ref double_to_ieee_string(). 

    A class called \ref columnify converts a set of
    strings into nicely formatted columns by padding with the
    necessary amount of spaces. This class operates on string objects
    of type \c std::string, and also works will for formatting columns
    of floating-point numbers.  This class is used to provide output
    for matrices in the functions \ref matrix_out(), \ref
    matrix_out_paren(), and \ref matrix_cx_out_paren(). For output
    of vectors, see \ref vector_out() in \ref array.h. 

    A related function, \ref screenify(), reformats a column of
    strings into many columns stored row-by-row in a new string
    array. It operates very similar to the way the classic Unix
    command \c ls organizes files and directories in multiple columns
    in order to save screen space.
    
    The function \ref count_words() counts the number of "words"
    in a string, which are delimited by whitespace.

    \hline
    \section diff_section Differentiation

    Differentiation is performed by descendants of \ref deriv and the
    classes are provided. These allow one to calculate either first,
    second, and third derivatives. The GSL approach is used in \ref
    gsl_deriv, and the CERNLIB routine is used in \ref
    cern_deriv. Both of these compute derivatives for a function
    specified using a descendant of \ref funct. For functions which
    are tabulated over equally-spaced abscissas, the class \ref
    eqi_deriv is provided which applies the formulas from Abramowitz
    and Stegun at a specified order.

    \b Warning: For gsl_deriv and cern_deriv, the second and third
    derivatives are calculated by naive repeated application of the
    code for the first derivative and can be particularly troublesome
    if the function is not sufficiently smooth. Error estimation is 
    also incorrect for second and third derivatives.

    \hline
    \section inte_section Integration

    Integration is performed by descendants of \ref inte and is
    provided in the library \c o2scl_inte.

    There are several routines for one-dimensional integration.
    - General integration over a finite interval: cern_adapt,
    cern_gauss, cern_gauss56, gsl_inte_qag, and gsl_inte_qng.
    - General integration from 0 to  \f$ \infty \f$ : gsl_inte_qagiu
    - General integration from  \f$ -\infty \f$ to 0: gsl_inte_qagil
    - General integration from  \f$ -\infty \f$ to \f$ \infty \f$ : 
    gsl_inte_qagi
    - General integration over a finite interval for a function with
    singularities: gsl_inte_qags and gsl_inte_qagp
    - Cauchy principal value integration over a finite interval:
    cern_cauchy and gsl_inte_qawc
    - Integration over a function weighted by \c cos(x) or \c sin(x):
    \ref gsl_inte_qawo_cos and \ref gsl_inte_qawo_sin
    - Fourier integrals: \ref gsl_inte_qawf_cos and \ref
    gsl_inte_qawf_sin
    - Integration over a weight function
    \f[
    W(x)=(x-a)^{\alpha}(b-x)^{\beta}\log^{\mu}(x-a)\log^{\nu}(b-x)
    \f]
    is performed by \ref gsl_inte_qaws. 
    
    For the GSL-based integration routines, the variables inte::tolx
    and inte::tolf have the same role as the quantities usually
    denoted in the GSL integration routines by \c epsabs and \c
    epsrel. In particular, the integration classes attempt to ensure
    that
    \f[ 
    |\mathrm{result}-I| \leq \mathrm{Max}(\mathrm{tolx},
    \mathrm{tolf}|I|)
    \f]
    and returns an error to attempt to ensure that
    \f[
    |\mathrm{result}-I| \leq \mathrm{abserr} \leq
    \mathrm{Max}(\mathrm{tolx},\mathrm{tolf}|I|)
    \f]
    where \c I is the integral to be evaluated. Even when the
    corresponding descendant of inte::integ() returns success, these
    inequalities may fail for sufficiently difficult functions. All 
    of the GSL integration routines except for gsl_inte_qng use
    a workspace given in gsl_inte_table which holds the results
    of the various subdivisions of the original interval. The 
    size of this workspace (and thus then number of subdivisions) can
    be controlled with gsl_inte_table::set_wkspace().

    The GSL routines were originally based on QUADPACK, which is
    available at http://www.netlib.org/quadpack . 

    \note The GSL integration routines can sometimes lose precision
    if the integrand is everywhere much smaller than unity. Some
    rescaling is required in these cases.
    
    Multi-dimensional hypercubic integration is performed by \ref
    composite_inte, the sole descendant of multi_inte. \ref
    composite_inte allows you to specify a set of one-dimensional
    integration routines (objects of type \ref inte) and apply them to
    a multi-dimensional problem.

    General multi-dimensional integration is performed by \ref
    comp_gen_inte, the sole descendant of \ref gen_inte.  The user is
    allowed to specify a upper and lower limits which are functions of
    the variables for integrations which have not yet been performed,
    i.e. the n-dimensional integral
    \f[ 
    \int_{x_0=a_0}^{x_0=b_0} f(x_0) \int_{x_1=a_1(x_0)}^{x_1=b_1(x_0)} 
    f(x_0, x_1) ...
    \int_{x_{\mathrm{n}-1}=a_{\mathrm{n}-1}(x_0,x_1,..,x_{\mathrm{n}-2})}^
    {x_{\mathrm{n}-1}=b_{\mathrm{n}-1}(x_0,x_1,..,x_{\mathrm{n}-2})} 
    f(x_0,x_1,...,x_{\mathrm{n-1}})~d x_{\mathrm{n}-1}~...~d x_1~d x_0
    \f]
    Again, one specifies a set of \ref inte objects to apply to
    each variable to be integrated over.

    Monte Carlo integration is also provided (see \ref mcarlo_section).

    \hline
    \section poly_section Roots of Polynomials

    Classes are provided for solving quadratic, cubic, and quartic
    equations as well as general polynomials. There is a standard
    nomenclature: classes which handle polynomials with real
    coefficients and real roots end with the suffix <tt>_real</tt>
    (quadratic_real, cubic_real and quartic_real), classes which
    handle real coefficients and complex roots end with the suffix
    <tt>_real_coeff</tt> (quadratic_real_coeff, cubic_real_coeff,
    quartic_real_coeff, and poly_real_coeff), and classes which handle
    complex polynomials with complex coefficients end with the suffix
    <tt>_complex</tt> (quadratic_complex, cubic_complex,
    quartic_complex, and poly_complex). As a reminder, complex roots
    may not occur in conjugate pairs if the coefficients are not
    real. Most of these routines will return an error if the leading
    coefficient is zero.

    In the public interfaces to the polynomial solvers, the
    complex type <tt>std::complex<double></tt> is used. These can
    be converted to and from the GSL complex type using the 
    \ref complex_to_gsl() and \ref gsl_to_complex() functions.

    At present, the polynomial routines work with complex numbers as
    objects of type \c std::complex<double> and are located in library
    \c o2scl_other. 

    For quadratics, \ref gsl_quadratic_real_coeff is the best if the
    coefficients are real, while if the coefficients are complex, use
    \ref quadratic_std_complex. For cubics with real coefficients,
    \ref cern_cubic_real_coeff is the best, while if the coefficients
    are complex, use \ref cubic_std_complex.
    
    For a quartic polynomial with real coefficients, \ref
    cern_quartic_real_coeff is the best, unless the coefficients of
    odd powers happen to be small, in which case, \ref
    gsl_quartic_real2 tends to work better. For quartics, generic
    polynomial solvers such as \ref gsl_poly_real_coeff can provide
    more accurate (but slower) results. If the coefficients are
    complex, then you can use \ref simple_quartic_complex.

    \hline
    \section solve_section Equation Solving

    \subsection onedsolve_subsect One-dimensional solvers

    Solution of one equation in one variable is accomplished by
    children of the class \ref root. 

    For one-dimensional solving, use cern_root or gsl_root_brent if
    you have the root bracketed, or gsl_root_stef if you have the
    derivative available. If you have neither a bracket or a
    derivative, you can use cern_mroot_root. 

    The \ref root base class provides the structure for three
    different solving methods:
    - \ref root::solve() which solves a function given an initial
    guess \c x
    - \ref root::solve_bkt() which solves a function given a solution
    bracketed between \c x1 and \c x2. The values of the function at
    \c x1 and \c x2 should have different signs.
    - \ref root::solve_de() which solves a function given an initial
    guess \c x and the derivative of the function \c df.

    If not all of these three functions are overloaded, then the
    source code in the \ref root base class is designed to try to
    automatically provide the solution using the remaining
    functions. Most of the one-dimensional solving routines, in their
    original form, are written in the second or third form above.  For
    example, gsl_root_brent is originally a bracketing routine of the
    form root::solve_bkt(), but calls to either root::solve() or
    root::solve_de() will attempt to automatically bracket the
    function given the initial guess that is provided. Of
    course, it is frequently most efficient to use the solver in the
    way it was intended.

    \subsection multisolve_subsect Multi-dimensional solvers

    Solution of more than one equation is accomplished by descendants
    of the class \ref mroot . The higher-level interface is provided
    by the function \ref mroot::msolve() .

    For multi-dimensional solving, you can use either cern_mroot or
    gsl_mroot_hybrids. While cern_mroot cannot utilize user-supplied
    derivatives, gsl_mroot_hybrids can use user-supplied derivative
    information (as in the GSL hybridsj method) using the function
    \ref gsl_mroot_hybrids::msolve_de() .

    \hline
    \section min_section Minimization

    One-dimensional minimization is performed by descendants of \ref
    minimize . There are two one-dimensional minimization algorithms,
    \ref cern_minimize and \ref gsl_min_brent, and they are both
    bracketing algorithms type where an interval and an initial guess
    must be provided. If only an initial guess and no bracket is
    given, these two classes will attempt to find a suitable bracket
    from the initial guess. While the \ref minimize base class is
    designed to allow future descendants to optionally use derivative
    information, this is not yet supported for any one-dimensional
    minimizers.

    Multi-dimensional minimization is performed by descendants of
    multi_min: \ref gsl_mmin_simp2, \ref gsl_mmin_conp, \ref
    gsl_mmin_conf, and \ref gsl_mmin_bfgs2. (The class \ref
    gsl_mmin_simp2 has been updated with the new "simplex2" method
    from GSL-1.12. The older "simplex" method is also available in
    \ref gsl_mmin_simp .) The classes \ref gsl_mmin_simp and \ref
    gsl_mmin_simp2 do not require any derivative information. The
    remaining minimization classes are intended for use when the
    gradient of the function is available, but they can also
    automaticallly compute the \gradient numerically. The standard way
    to provide the \gradient is to use a child of \ref grad_funct (or
    \ref grad_vfunct). Finally, the user may specify the automatic
    gradient object of type \ref gradient (or \ref gradient_array)
    which is used by the minimizer to compute the gradient numerically
    when a function is not specified.
    
    See an example for the usage of the multi-dimensional minimizers
    in \ref ex_mmin_sect .

    Simulated annealing methods are also provided for multi-dimensional
    minimization (see \ref anneal_section).

    It is important to note that not all of the minimization routines
    test the second derivative to ensure that it doesn't vanish to 
    ensure that we have indeed found a true minimum. 

    The class \ref multi_min_fix provides a convenient way of 
    fixing some of the parameters and minimizing over others,
    without requiring a the function interface to be rewritten. An
    example is given in \ref ex_mmin_fix_sect. 

    A naive way of implementing constraints is to add a function to
    the original which increases the value outside of the allowed
    region. This can be done with the functions \ref constraint() and
    \ref lower_bound(). There are two analogous functions, \ref
    cont_constraint() and \ref cont_lower_bound(), which continuous
    and differentiable versions. Where possible, it is better to use
    the constrained minimization routines described below.

    \hline
    \section conmin_section Constrained Minimization

    \o2 reimplements the Open Optimization Library (OOL) available at
    http://ool.sourceforge.net. The associated classes allow
    constrained minimization when the constraint can be expressed as a
    hyper-cubic constraint on all of the independent variables. The
    routines have been rewritten and reformatted for C++ in order to
    facilitate the use of member functions and user-defined vector
    types as arguments. The base class is ool_constr_mmin and there
    are two different constrained minimzation algorithms implemented
    in ool_mmin_pgrad, ool_mmin_spg. (The ool_mmin_gencan minimizer is
    not yet finished). The \o2 implementation should be essentially
    identical to the most recently released version of OOL.

    The constrained minimization classes operate in a similar way
    to the other multi-dimensional minimization classes (which are
    derived from multi_min). The constraints are specified with
    the function
    \code
    ool_constr_mmin::set_constraints(size_t nc, vec_t &lower, 
    vec_t &upper);
    \endcode
    and the minimization can be performed by calling either
    multi_min::mmin() or multi_min::mmin_de() (if the \gradient is
    provided by the user). The method in \ref ool_mmin_gencan requires
    a Hessian vector product and the user can specify this product for
    the minimization by using \ref ool_constr_mmin::mmin_hess(). The
    Hessian product function can be specified as an object of type
    \ref ool_hfunct or \ref ool_hvfunct in a similar way to the other
    function objects in \o2.

    There are five error codes defined in \ref ool_constr_mmin
    which are specific to the OOL classes. 

    \hline
    \section mcarlo_section Monte Carlo Integration

    Monte Carlo integration is performed by descendants of mcarlo_inte
    (\ref gsl_monte, \ref gsl_miser, and \ref gsl_vegas). These
    routines are generally superior to the direct methods for
    integrals over regions with large numbers of spatial dimensions.

    \hline
    \section anneal_section Simulated Annealing

    Minimization by simulated annealing is performed by descendants of
    sim_anneal (see \ref gsl_anneal). Because simulated annealing is
    particularly well-suited to parallelization, a multi-threaded
    minimizer analogous to \ref gsl_anneal is given in \ref anneal_mt.
    This header-only class uses the Boost libraries and requires it
    for use.

    \hline
    \section fit_section Non-linear Least-Squares Fitting

    Fitting is performed by descendants of \ref fit_base and fitting
    functions can be specifed using \ref fit_funct.  The GSL fitting
    routines (scaled and unscaled) are implemented in gsl_fit. A
    generic fitting routine using a minimizer object specified as a
    child of multi_min is implemented in min_fit. When the multi_min
    object is (for example) a sim_anneal object, min_fit can avoid
    local minima which can occur when fitting noisy data.

    \hline
    \section ode_section Solution of Ordinary Differential Equations

    Classes for non-adaptive integration are provided as descendents
    of \ref odestep and classes for adaptive integration are
    descendants of \ref adapt_step. To specify a set of functions
    to these classes, use a child of ode_funct for a generic 
    vector type or a child of ode_vfunct when using arrays.

    Solution of simple initial value problems is performed by \ref
    ode_iv_solve. This class contains a couple different methods,
    depending on whether the user needs only the final value or the
    solution on a fixed grid.

    The solution of boundary-value problems is based on the abstract
    base class \ref ode_bv_solve. At the moment, a simple shooting method 
    is the only implementation of this base class and is given 
    in \ref ode_bv_shoot . An experimental multishooting class is 
    given in \ref ode_bv_multishoot .

    An experimental application of linear solvers to solve the 
    finite-difference equations for a boundary value problem
    is given in \ref ode_it_solve .

    \hline
    \section rng_section Random Number Generation

    Random number generators are descendants of \ref rnga . While the
    base object \ref rnga is created to allow user-defined random
    number generators, the only random number generator presently
    included are from GSL. The GSL random number generator code is
    reimplemented in the class \ref gsl_rnga to avoid an additional
    performance penalty. This may not be a truly "object-oriented"
    interface in that it does not use virtual functions, but it avoids
    any possible performance penalty. Random number generators are
    implemented as templates in \ref sim_anneal and \ref mcarlo_inte.
    In these classes, the random number generator is a template type,
    rather than a member data pointer, in order to ensure fast
    execution.

    \hline
    \section tintp_section Two-dimensional Interpolation

    Successive use of \ref smart_interp is implemented in \ref
    twod_intp, which is useful for interpolating data presented
    on a two-dimensional grid (though the spacings between grid points
    need not be equal). Also, one can compute \contour lines from
    data represented on a grid using the \ref contour class. 

    If data is arranged without a grid, then \ref planar_intp can
    be used. At present, the only way to compute \contour lines on 
    data which is not defined on a grid is to use \ref planar_intp to
    recast the data on a grid and then use \ref contour afterwards. 

    Higher-dimensional interpolation is possible with \ref tensor_grid.

    \hline
    \section gslcheb_section Chebyshev Approximation

    A class implementing the Chebyshev approximations based on GSL is
    given in \ref gsl_chebapp. This class has its own copy
    constructor, so that Chebyshev approximations can be copied and
    passed as arguments to functions.

    \hline
    \section unitconv_section Unit Conversions

    There is a class which performs conversion between units specified
    with strings in \ref convert_units. The \ref convert_units_gnu
    class also uses a system call to the GNU units command (if it's
    installed in the path) to obtain the proper conversion factors.

    \hline
    \section other_section Other classes and functions

    The \o2 library contains several classes which are still under
    development and are to be considered experimental. While it is
    expected that the testing code associated with these classes will
    succeed on most any system, the interface, structure, and basic
    approaches used by these classes may change in future versions of
    \o2.

    \b Three-dimensional data tables - \ref table3d \n
    \b Two-dimensional Gaussian distribution - \ref gaussian_2d \n
    \b Series \b acceleration - \ref gsl_series \n
    \b Command-line interface - \ref cli \n
    \b Automatic bin sizing - \ref bin_size \n
    \b Fourier \b transforms - \ref gsl_fft \n
    \b Polylogarithms - \ref polylog \n

    \hline
    \section lset_section Library settings

    There are a couple library settings which are handled by 
    a global object \ref lib_settings of type \ref lib_settings_class.

    There are several data files that are used by various classes in
    the library. The installation procedure should ensure that these
    files are automatically found. However, if these data files are
    moved after installation, then a call to
    lib_settings_class::set_data_dir() can adjust the library to use
    the new directory. It is assumed that the directory structure
    within the data directory has not changed.

    \hline
    \section io_section Object I/O

    The I/O portion of the library is still experimental.
    
    Collections of objects can be stored in a \ref collection class,
    and these collections can be written to or read from text or
    binary files. User-defined classes may be added to the collections
    and may be read and written to files as long as a descendant of
    \ref io_base is provided.

    Every type has an associated I/O type which is a descendant of
    \ref io_base. In order to perform any sort of input/output on any
    type, an object of the corresponding I/O type must be instantiated
    by the user. This is not done automatically by the library. (Since
    it doesn't know which objects are going to be used ahead of time,
    the library would have to instantiate <em>all</em> of the I/O
    objects, which is needlessly slow.) This makes the I/O slightly
    less user-friendly, but much more efficient. For convenience, each
    subsection of the library has a class (named with an \c _ioc
    suffix) which will automatically allocate all I/O types for that
    subsection.
    
    <b>Level 1 functions:</b> Functions that input/output data from
    library-defined objects and internal types from files and combine
    these objects in collections. These are primarily member functions
    of the class \ref collection.

    <b>Level 2 functions:</b> Functions which are designed to allow
    the user to input or output data for user-generated objects. These
    are primarily member functions of classes \ref cinput and \ref
    coutput.
    
    <b>Level 3 functions:</b> Functions which allow low-level
    modifications on how input and output is performed. Usage of 
    level 3 functions is not immediately recommended for the casual
    user.
    
    <b>Level 1 usage</b>:

    For adding an object to a collection when you have a pointer
    to the I/O object for the associated type:
    \code
    int collection::add(std::string name, io_base *tio, void *vec, 
    int sz=0, int sz2=0, bool overwrt=true, bool owner=false);
    \endcode
    For adding an object to a collection otherwise:
    \code
    int collection::add(std::string name, std::string stype,
    void *vec, int sz=0, int sz2=0,
    bool overwrt=true, bool owner=false);
    \endcode

    To retrieve an object as a <pre>void *</pre> from a collection 
    use one of:
    \code
    int get(std::string tname, void *&vec);
    int get(std::string tname, void *&vec, int &sz);
    int get(std::string tname, void *&vec, int &sz, int &sz2);
    int get(std::string tname, std::string &stype, void *&vec);
    int get(std::string tname, std::string &stype, void *&vec, int &sz);
    int get(std::string tname, std::string &stype, void *&vec, int &sz,
    int &sz2);
    \endcode
    When retrieving a scalar object without error- and type-checking you
    can use the shorthand version:
    \code
    void *get(std::string name);
    \endcode

    To output one object to a file:
    \code
    int collection::out_one(out_file_format *outs, std::string stype, 
    std::string name, void *vp, int sz=0, int sz2=0);
    \endcode

    To input one object from a file with a given type and name:
    \code
    int collection::in_one_name(in_file_format *ins, std::string stype, 
    std::string name, void *&vp, int &sz, int &sz2);
    \endcode
    To input the first object of a given type from a file:
    \code
    int collection::in_one(in_file_format *ins, std::string stype, 
    std::string &name, void *&vp, int &sz, int &sz2);
    \endcode

    <b>Level 2 usage (string-based)</b>:

    If you don't have a pointer to the io_base child object corresponding
    to the type of subobject that you are manipulating, then you can use
    the following functions, which take the type name as a string.

    To input a sub-object in an io_base template for which memory
    has already been allocated use one of:
    \include io_doc1.cpp
    
    To automatically allocate memory and input a sub-object of a 
    io_base template use one of:
    \include io_doc2.cpp

    To output a subobject in an io_base template use:
    \code
    int collection::object_out(std::string type, out_file_format *outs,
    void *op, int sz=0, int sz2=0, std::string name="");
    \endcode
    
    <b>Level 2 usage (with io_base pointer)</b>:

    To input a sub-object in an io_base template for which memory
    has already been allocated use one of:
    \include io_doc4.cpp
    
    To automatically allocate memory and input a sub-object of a 
    io_base template use one of:
    \include io_doc5.cpp
    
    To output a subobject in an io_base template use:
    \include io_doc6.cpp

    To automatically allocate/deallocate memory for an object, use:
    \code
    virtual int mem_alloc(object *&op);
    virtual int mem_alloc_arr(object *&op, int sz);
    virtual int mem_alloc_2darr(object **&op, int sz, int sz2);
    virtual int mem_free(object *op);
    virtual int mem_free_arr(object *op);
    virtual int mem_free_2darr(object **op, int sz);
    \endcode
    
    \subsection iotlate_subsect Usage of io_tlate

    The functions io_tlate::input() and io_tlate::output() need to be
    implemented for every class has information for I/O. For
    subobjects of the class, cinput::object_in() and
    cinput::object_out() can be called to input or output the
    information associated with the subobject. For input,
    cinput::object_in_name(), cinput::object_in_mem(), and
    cinput::object_in_mem_name() allow the freedom to input an object
    with a name or with memory allocation. The function
    coutput::object_out_name() allows one to output an object with a
    name.  If the class contains a pointer to the subobject, then
    io_base::pointer_in() or io_base::pointer_out() can be used.

    \hline
    \section source_section Example source code

    \subsection exlist_subsect Example list

    - \ref ex_fptr_sect shows how member functions and external
    parameters are supplied to \o2 numerical routines. In this case, a
    member function representing an equation with one unknown 
    is solved using a one-dimesional solver.
    - \ref ex_mroot_sect demonstrates the multidimensional function
    solver and several different methods of specifying the function to
    be solved.
    - \ref ex_mmin_sect 
    - \ref ex_mmin_fix_sect 
    - \ref ex_deriv_sect 
    - \ref ex_inte_sect 
    - \ref ex_ode_sect 
    - \ref ex_anneal_sect demonstrates multidimensional
    minimization by simulated annealing
    - \ref ex_minte_sect demonstrates several ways to compute a 
    multidimensional integral including Monte Carlo and direct methods
    - \ref ex_contour_sect 
    - \ref ex_twod_intp_sect 
    - \ref ex_ublas_sect demonstrates the use of interpolation
    and solver classes with ublas vectors

    \comment
    \c ex_diff.cpp computes the derivative of a tabulated function

    \c ex_table.cpp demonstrates the use of the \ref table class

    \c ex_file.cpp demonstrates the file I/O
    \endcomment

    \subsection ex_fptr_sect Function and solver example

    \dontinclude ex_fptr.cpp
    \skip Example:
    \until End of example

    The image below shows how the solver progresses to the 
    solution of the example function.
    \image html ex_fptr.png "ex_fptr.png"
    \image latex ex_fptr.eps "ex_fptr.eps" width=9cm

    \subsection ex_mroot_sect Multidimensional solver example

    \dontinclude ex_mroot.cpp
    \skip Example:
    \until End of example

    \subsection ex_mmin_sect Multidimensional minimizer example

    \dontinclude ex_mmin.cpp
    \skip Example:
    \until End of example

    \subsection ex_mmin_fix_sect Minimizer fixing variables example

    \dontinclude ex_mmin_fix.cpp
    \skip Example:
    \until End of example

    \subsection ex_deriv_sect Numerical differentiation example

    \dontinclude ex_deriv.cpp
    \skip Example:
    \until End of example

    \subsection ex_inte_sect Numerical integration example

    \dontinclude ex_inte.cpp
    \skip Example:
    \until End of example

    \subsection ex_ode_sect Ordinary differential equations example

    \dontinclude ex_ode.cpp
    \skip Example:
    \until End of example

    \subsection ex_anneal_sect Simulated annealing example

    \dontinclude ex_anneal.cpp
    \skip Example:
    \until End of example

    \subsection ex_minte_sect Multidimensional integration example

    \dontinclude ex_minte.cpp
    \skip Example:
    \until End of example

    \subsection ex_contour_sect Contour lines example

    \dontinclude ex_contour.cpp
    \skip Example:
    \until End of example

    \subsection ex_twod_intp_sect Two-dimensional interpolation example

    \dontinclude ex_twod_intp.cpp
    \skip Example:
    \until End of example

    \subsection ex_ublas_sect ublas vector example

    \dontinclude ex_ublas.cpp
    \skip Example:
    \until End of example

    \hline
    \section design_section Design Considerations

    The design goal is to create an object-oriented computing library
    with classes that perform common numerical tasks. The most
    important principle is that the library should add functionality
    to the user while at the same time retaining as much freedom for
    the user as possible and allowing for ease of use and extensibility. 
    To that end, 
    - The classes which utilize user-specified functions
    should be able to operate on member functions without requiring
    a particular inheritance structure,
    - The interfaces ought to be generic so that the user
    can create new classes which perform related numerical
    tasks through inheritance,
    - The classes should not use static variables or functions
    - Const-correctness and type-safety should be used wherever possible, and
    - The design should be somewhat compatible with GSL.
    Also, the library provides higher-level routines
    for situations which do not require lower-level access. 

    <b>Header file dependencies</b>
    
    For reference, it's useful to know how the top-level header files
    depend on each other, since it can be difficult to trace
    everything down. In the \c base directory, the following are some
    of the most "top-level" header files and their associated
    dependencies within \o2 (there are other dependencies on GSL and
    the C standard library not listed here).
    \code
    err_hnd.h : (none)
    lib_settings.h : (none)
    array.h: err_hnd.h
    vector.h: err_hnd.h
    sring_conv.h : lib_settings.h
    misc.h : err_hnd.h lib_settings.h
    test_mgr.h : string_conv.h
    uvector_tlate.h: err_hnd.h string_conv.h array.h vector.h
    ovector_tlate.h: err_hnd.h string_conv.h 
                  uvector_tlate.h array.h vector.h 
    \endcode

    <B>The use of templates</b>
    
    Templates are used extensively, and this makes for longer
    compilation times so any code that can be removed conveniently
    from the header files should be put into source code files
    instead. 

    \subsection errdesign_subsect Error handling

    <b>Thread safety</b>
    Two approaches to thread-safe error handling which are worth
    comparing: the first is GSL which uses return codes and global
    function for an error handler, and the second is the Math/Special
    Functions section of Boost, which uses a separate policy type for
    each function. One issue is thread safety: the GSL approach is
    thread safe only in the sense that one can in principle use the
    return codes in different threads to track errors. What one cannot
    do in GSL is use different user-defined error handlers for
    different threads. The Special Functions library allows one to
    choose a different Policy for every special function call, and
    thus allows quite a bit more flexibility in designing
    multi-threaded error handling.

    \subsection vecdesign_subsect Vector design

    \o2 vector and matrix types are a hybrid approach: creating objects
    compatibile with GSL, while providing syntactic simplicity and 
    object-oriented features common to C++ vector classes. In terms
    of their object-oriented nature, they are not as elegant as 
    the ublas vector types from ublas, but for many applications they
    are also faster (and they are always at least as fast). 

    However, ensuring const-correctness makes the design a bit thorny,
    and this is still in progress. 

    \subsection typecase_subsect Type-casting in vector and matrix design

    \o2 uses a GSL-like approach where viewing const double *
    arrays is performed by explicitly casting aways const'ness
    internally and then preventing the user from changing the data. 

    In GSL, the preprocessor output for \c vector/view_source.c is:
    \include gsl_vector_doc.cpp
    Note the explicit cast from const double * to double *. This
    is similar to what is done in \c src/base/ovector.cpp.

    \subsection define_subsect Define constants and macros

    There are a couple define constants and macros that \o2
    understands, they are all in upper case and begin with the prefix
    <tt>O2SCL_</tt>. The two define constants O2SCL_PARTLIB and
    O2SCL_EOSLIB control which sublibraries are compiled together with
    the main library (see \ref install_section ). 

    Range-checking for arrays and matrices is turned on by default,
    but can be turned off by defining <tt>O2SCL_NO_RANGE_CHECK</tt>
    during the initial configuration of the library. To see how the
    library was configured at runtime, use the \ref lib_settings
    class.

    There are several macros for error handling defined in \ref
    err_hnd.h, and several for vector/matrix arithmetic in \ref
    vec_arith.h .

    Finally, I sometimes comment out sections of code with 
    \code
    #ifdef O2SCL_NEVER_DEFINED
    ...
    #endif
    \endcode
    This constant should not be defined by the user as it will cause
    compilation to fail.

    \subsection global_subsect Global objects

    There are three global objects that are created in
    libo2scl:
    - \ref def_err_hnd is the default error handler
    - \ref err_hnd is the pointer to the error handler (points to
    def_err_hnd by default)
    - \ref lib_settings to control a few library settings

    All other global objects are to be avoided.

    \subsection thread_subsect Thread safety

    Most of the classes are thread-safe, meaning that two instances of
    the same class will not clash if their methods are called
    concurrently since static variables are only used for compile-time
    constants.  However, two threads cannot, in general, safely access
    the same instance of a class. In this respect, \o2 is no
    different from GSL.
    
    \subsection docdesign_subsect Documentation design
    
    The commands \\comment and \\endcomment delineate comments about
    the documentation that are present in the header files but don't
    ever show up in the HTML or LaTeX documentation.

    \subsection crightfoo_subsect Copyright notices

    For files where it is appropriate to do so, I have followed the
    prescription suggested in
    http://lists.gnu.org/archive/html/help-gsl/2008-11/msg00017.html
    retaining the GSL copyright notices and putting the \o2 notices at
    the top. CERNLIB has no such standard, but their licensing information
    is outlined at
    http://cernlib.web.cern.ch/cernlib/conditions.html .

    \subsection futurework_subsect Design plans

    <b>Exception classes:</b> \n Eventually I'd like to create a sensible
    hierarchy of exception classes to be thrown by the handler
    if the user would prefer (this is already in progress).

    <b>Boost and linear algebra:</b> \n I would like to ensure this
    class is compatible with boost, and start integrating things
    accordingly. IMHO object-oriented linear algebra is in a rather
    sad state at the moment. uBlas and MTL are both promising,
    however, and I'd like to start implementing some sort of
    compatibility with uBlas vectors and matrices soon. The uBlas
    documentation is pretty sparse, but that's the pot calling the
    kettle a cheap piece of metal.

    <b>Other Improvements:</b> \n I'm particularly interested in
    improving the ODE and fitting classes, as well as updating the
    BFGS2 minimizer. Of course, more examples and better documentation
    are also a must.

    <b>Algorithms to include</b>
    - Method of lines for PDEs
    - Some of the MESA interpolation routines.
    - C++ translation of MINUIT (done already by ROOT, but quite difficult). 
    - Creating closed regions from contour lines (I have no idea how to
    do this at the moment, though I'm sure someone has solved this 
    problem already somewhere.)

    <b>Complex numbers</b> \n I'm not sure where to go with complex
    numbers. My guess is that <tt>std::complex</tt> is not
    significantly slower (or is faster) than <tt>gsl_complex</tt>, but
    it would be good to check this. Then there's the C99 standard,
    which is altogether different. Unfortunately the interfaces may be
    sufficiently different that it's impossible to make templated
    classes which operate on generic complex number types.

    <b>I/O</b> \n The I/O classes are (admittedly) not structured very
    well, and there are a lot of things which could be done there.
    There are a lot of alternative solutions out there, but most of
    them work on binary file formats (which I find difficult to use).
    - On a shorter time scale I'd like to replace the inner workings
    of the \ref collection class with a mechanism using
    <tt>boost::any</tt>, which will avoid the silly <tt>void *</tt>s
    all over the place. This is already in progress. 
    - On a longer time scale, I'd like to replace the file format I/O
    classes with something using Boost pipes.

    \hline
    \section license_section License Information
    
    \o2 (as well as CERNLIB and the Gnu Scientific Library (GSL)) is
    licensed under version 3 of the GPL as provided in the files \c
    COPYING and in \c doc/o2scl/extras/gpl_license.txt. After
    installation, it is included in the documentation in \c
    PREFIX/doc/extras/gpl_license.txt where the default \c PREFIX is
    \c /usr/local.
    
    \verbinclude gpl_license.txt

    This documentation is provided under the GNU Free Documentation
    License, as given below and provided in \c
    doc/o2scl/extras/fdl_license.txt. After installation, it is included
    in the documentation in \c PREFIX/doc/extras/fdl_license.txt where
    the default \c PREFIX is \c /usr/local.
    
    \verbinclude fdl_license.txt
    
    \hline
    \section ack_section Acknowledgements
    
    I would like to thank the creators of GSL and Doxygen for their
    excellent work! Thanks also to Julien Garaud for contributing the
    ODE multishooting class and for the <tt>ex_hydrogen</tt> example.

    \hline
    \section ref_section Bibliography

    Some of the references which contain links should direct you to
    the work referred to directly through dx.doi.org.

    \anchor Bader83 Bader83:
    G. Bader and P. Deuflhard, Numer. Math. \b 41 (1983) 373.

    \anchor Bus75 Bus75:
    J.C.P. Bus and T.J. Dekker, ACM
    Trans. Math. Software \b 1 (1975) 330.

    \anchor Cash90 Cash90:
    Cash, J.R., Karp, A.H., ACM Transactions of Mathematical
    Software, vol. 16 (1990) 201-222. 

    \anchor Fletcher87 Fletcher87:
    R. Fletcher, Practical methods of optimization (John Wiley & Sons,
    Chichester 1987) p. 39.

    \anchor Hairer00 Hairer00:
    Hairer, E., Norsett S.P., Wanner, G. Solving ordinary
    differential equations I, Nonstiff Problems, 2nd revised edition,
    Springer, 2000.

    \anchor Krabs83 Krabs83:
    W. Krabs, Einfuhrung in die lieare und nictlineare Optimierung
    fur Ingenieure (BSB B.G. Teubner, Leipzig 1983) p. 84.

    \anchor Lepage78 Lepage78:
    G. P. Lepage, originally described in J. Comp. Phys. 27 (1978)
    192.

    \anchor Lewin83 Lewin83:
    L. Lewin, Polylogarithms and Associated Functions (North-Holland,
    New York, 1983).

    \anchor Longman58 Longman58:
    I.M. Longman, MTAC (later renamed Math. Comp.) \b 12 (1958) 205.

    \anchor More79 More79:
    J.J. More' and M.Y. Cosnard, ACM Trans. Math. Software,
    \b 5 (1979) 64-85.

    \anchor More80 More80:
    J.J. More' and M.Y. Cosnard, Algoright 554 BRENTM,
    Collected Algorithms from CACM (1980).
    
    \anchor Nelder65 Nelder65:
    Nelder, J.A., Mead, R., Computer Journal 7 (1965) pp. 308-313.

    \anchor Press90 Press90:
    W.H. Press, G.R. Farrar, "Recursive Stratified Sampling for
    Multidimensional Monte Carlo Integration", Computers in Physics,
    v4 (1990), pp. 190-195.

    \anchor Prince81 Prince81:
    P.J. Prince and J.R. Dormand J. Comp. Appl. Math., 7 (1981) 67.

    \anchor Rutishauser63 Rutishauser63:
    H. Rutishauser, Ausdehnung des Rombergschen Prinzips
    (Extension of Romberg's Principle), Numer. Math. \b 5 (1963) 48-54.

*/
/** 
    \page cvn_page Notes on const vectors

    The code

    \code
    const ovector x;
    //ovector_const_base &cb=x;
    \endcode
    
    doesn't complile, which is a bit weird (you have to use a
    const ovector_const_base reference instead). 
    
    Also, ovector_base has a function like
    
    \code
    double &operator[](size_t i)
    \endcode
    
    which in principle could be actually const, i.e.
    
    \code
    double &operator[](size_t i) const
    \endcode
    
    but isn't. Also annoying is that "const ovector_base &x" is not a
    perfectly generic base, since it's not a base for
    "ovector_const_base &y" or "const ovector_const_base &y2" (last one
    I'm not sure about). I had thought about making ovector_const_base a
    child of ovector_base instead, but that means that the interpolation
    routines which have ovector_const_base as a default template
    parameter will have to be changed. (Is this a problem?) Even worse,
    ovector_const_base classes could be upcasted easily to ovector_base
    removing const correctness. 
    
    Finally, the meaning of "const ovector_view" vs. "ovector_view" is
    kind of confusing, though maybe we just have to explain to the
    user that "const" means const where-ever it appears. 
    
    Happily, this new branch removes the need for "ovector_view_assign", 
    in the trunk which was kind of frustrating. 
    
    Probably the best new solution is to rework the hierarchy altogether
    and admit that ovector and ovector_view aren't really related
    objects. There is no such thing as a generic base for both ovector
    and ovector_view. This of course might require a redesign of the
    solver classes, for example, as they treat these two classes as
    necessarily related (alloc_vec_t is supposed to be related to
    vec_t). Maybe not though, I need to think about this more. 

*/
/** 
    \page dl_page Download O2scl

    The current version is 0.902. The source distribution can 
    be obtained from \n
    - \c http://sourceforge.net/project/showfiles.php?group_id=206918

    You may also download \o2 from version from the Subversion
    repository. To obtain the bleeding-edge developer version, use
    something like
    \verbatim
    svn co https://o2scl.svn.sourceforge.net/svnroot/o2scl/trunk o2scl
    \endverbatim
    The checkout process is a bit more time consuming than directly
    downloading the source distribution.

    Version 0.902 corresponds with version 37 in the svn respository.

    Version 0.901 corresponds with version 31 in the svn respository.

    Version 0.9 corresponds with version 26 in the svn respository.

    Version 0.806 was a quick release to fix some issues with
    compilations in Mac OS X and does not correspond exactly 
    with a version in the repository. 

    Version 0.805 can be obtained directly from svn, it 
    corresponds to version 16 in the repository.
    
*/
/*
  \htmlonly
  <table border=0>
  <tr><td>Release version number</td><td>SVN repository number</td></tr>
  <tr><td>0.901</td><td>31</td></tr>
  <tr><td>0.9</td><td>26</td></tr>
  <tr><td>0.806</td><td>none</td></tr>
  <tr><td>0.805</td><td>16</td></tr>
  </table>
  \endhtmlonly
  
*/
/**
   \brief An empty class to add some items to the todo and bug lists
   
   \todo
   - The o2scl-test and o2scl-examples targets require grep, awk, 
   tail, cat, and wc. It would be good to reduce this list
   to ensure better compatibility. 
   - More examples and benchmarks
   - There are a couple classes which Doxygen doesn't yet parse
   properly for the class hierarchy: gsl_root_stef, 
   and ool_mmin_gencan. Also should double check
   smart_interp, etc.

   \future
   - Fix the PNG images so that they're smaller and repair the
   transparency issue (probably can be done just using different
   arguments to 'convert' in the pngfix target)
   - Make sure we have a uvector_cx_alloc, 
   ovector_cx_const_reverse, ovector_cx_const_subvector_reverse, 
   uvector_reverse, uvector_const_reverse, uvector_subvector_reverse,
   uvector_const_subvector_reverse, omatrix_cx_diag, blah_const_diag, 
   umatrix_diag, and umatrix_cx_diag
   - ovector_cx_view::operator=(uvector_cx_view &) is missing
   - ovector_cx::operator=(uvector_cx_view &) is missing
   - uvector_c_view::operator+=(complex) is missing
   - uvector_c_view::operator-=(complex) is missing
   - uvector_c_view::operator*=(complex) is missing

   \todo At present, the testing code assumes that shared libraries are
   installed. Can this be improved? (12/3/08 - I'm not sure what the
   status is on this...this should be checked.)

   \todo Make sure default template parameters are documented
   in each class, and make sure default template parameters exist
   where they should.
   
   \future Consider breaking documentation up into sections? 
   
   \bug 
   - BLAS libraries not named \c libblas or \c libgslblas are not
   properly detected in ./configure and will have to be added
   manually. 
   - The -lm flag may not be added properly by ./configure 
   (1/4/09 - I'm not sure I remember why this is a problem, 
   but it probably has to do with the detection of libraries
   which is done in configure.ac). 

*/
class other_todos_and_bugs {
}

/** \brief The main namespace
    \nothing

    Most of the classes reside in this namespace, but it is removed
    from the documentation for clarity. 
    \htmlonly
    See <a href="annotated.html">Data Structures</a> for a full list.
    \endhtmlonly

*/
namespace o2scl {
}

/*
    \hline
    \section motiv_section Motivation

    I wanted a library for numerical programming in C++. This
    immediately suggests the following question: Why C++? Part of the
    answer to this question is practical; I am much more familiar with
    C/C++ so learning a different language (e.g. Fortran) never seemed
    like it made much sense. A related question might be: Why not an
    interpreted language, like python? Answer: faster
    execution. Finally, why C++ and not C? Answer: The object-oriented
    approach arguably allows for simpler and faster development while
    not appreciably removing flexibility. The object-orientation also
    encourages library development.  Furthermore, I find the syntactic
    simplicity provided by C++ to be very convient at times.

    In regards to the GNU Scientific Library (GSL), GSL is not simple
    to use for C++ development because it makes it very difficult to
    utilize member functions in those routines which take functions as
    inputs.

    I have also allowed the classes to hide their memory allocation
    structure, even though the user is frequently allowed to specify
    when it is allocated or deallocated. In light of the philosophy of
    'encapsulation' this makes sense, as the memory requirements for
    the classes are not generic. Users who do not know about the
    details of the algorithm need not know about the details of the
    memory requirements, and users who need to know about the details
    of the memory requirements most often also want to know the
    details about the algorithm.

*/
