/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/collection.h>
#include <o2scl/test_mgr.h>
#include <o2scl/text_file.h>
#include <o2scl/binary_file.h>
#include <o2scl/base_ioc.h>
#include <o2scl/table.h>

using namespace std;
using namespace o2scl;

namespace o2scl {

  class mypair {
  public:
    mypair() {
      arr=new double[2];
    }
    ~mypair() {
      delete[] arr;
    }
    void set(double sx, double sy) { 
      x=sx; y=sy; 
      arr[0]=sx; arr[1]=sy;
    };
    void get(double &sx, double &sy) { 
      sx=x; sy=y; 
    };
    mypair *p;
    const char *type() { return "mypair"; }
  protected:
    friend class io_tlate<class mypair>;
    double x, y, *arr;
  };

  template <> int io_tlate<mypair>::input(cinput *co, in_file_format *ins,
					  mypair *pp) {
    ins->double_in(pp->x);
    ins->double_in(pp->y);
    std::string tmp="";
    co->object_in("double",ins,pp->arr,2,tmp);
    string stype;
    void *vp=(void *)(&pp->p);
    void **vpp=(void **)vp;
    pointer_in(co,ins,vpp,stype);
    return 0;
  }

  template <> int io_tlate<mypair>::output(coutput *co, out_file_format *outs,
					   mypair *pp) {
    outs->double_out(pp->x);
    outs->double_out(pp->y);
    co->object_out("double",outs,pp->arr,2);
    pointer_out(co,outs,pp->p,"mypair");
    return 0;
  }
  template <> const char *io_tlate<mypair>::type() { return "mypair"; }

  io_tlate<mypair> mypair_io;

}

int main(void) {
  double dpa, *dpap;
  string spa, wpa, sname, *spap, *wpap;
  int ipa, sz, sz2, i, itemp, *ipap;
  char cpa, *cpap;
  bool bpa, *bpap;
  void *vtemp;
  ofstream fout;
  o2scl::test_mgr t;
  def_err_hnd.set_mode(2);
  t.set_output_level(2);

  io_type_info it;
  it.type_summary(&cout,true);

  base_ioc bio;
  it.type_summary(&cout,true);
  base_ioc biod;

  /*
    t.test_gen(it.is_type("adapt_step")==0,"istype(adapt_step)");
    t.test_gen(it.is_type("am_funct_strings")==0,"istype(am_funct_strings)");
    t.test_gen(it.is_type("apr_eos")==0,"istype(apr_eos)");
    t.test_gen(it.is_type("bool")==0,"istype(bool)");
    t.test_gen(it.is_type("boson")==0,"istype(boson)");
    //  t.test_gen(it.is_type("cern_cubic_real_coeff")==0,
    //	     "istype(cern_cubic_real_coeff)");
    t.test_gen(it.is_type("cern_adapt")==0,"istype(cern_adapt)");
    t.test_gen(it.is_type("cern_cauchy")==0,"istype(cern_cauchy)");
    t.test_gen(it.is_type("cern_deriv")==0,"istype(cern_deriv)");
    t.test_gen(it.is_type("cern_gauss")==0,"istype(cern_gauss)");
    t.test_gen(it.is_type("cern_gauss56")==0,"istype(cern_gauss56)");
    t.test_gen(it.is_type("cern_mroot")==0,"istype(cern_mroot)");
    //t.test_gen(it.is_type("cern_quartic_real_coeff")==0,
    //"istype(cern_quartic_real_coeff)");
    t.test_gen(it.is_type("cern_root")==0,"istype(cern_root)");
    t.test_gen(it.is_type("char")==0,"istype(char)");
    t.test_gen(it.is_type("classical")==0,"istype(classical)");
    t.test_gen(it.is_type("comp_gen_inte")==0,"istype(comp_gen_inte)");
    t.test_gen(it.is_type("composite_inte")==0,"istype(composite_inte)");
    //t.test_gen(it.is_type("cubic_complex")==0,"istype(cubic_complex)");
    //  t.test_gen(it.is_type("cubic_real_coeff")==0,
    // "istype(cubic_real_coeff)");
    //  t.test_gen(it.is_type("cubic_real")==0,"istype(cubic_real)");
    //t.test_gen(it.is_type("cubic_std_complex")==0,
    // "istype(cubic_std_complex)");
    t.test_gen(it.is_type("deriv")==0,"istype(deriv)");
    t.test_gen(it.is_type("double")==0,"istype(double)");
    t.test_gen(it.is_type("eff_boson")==0,"istype(eff_boson)");
    t.test_gen(it.is_type("eff_fermion")==0,"istype(eff_fermion)");
    t.test_gen(it.is_type("eff_quark")==0,"istype(eff_quark)");
    t.test_gen(it.is_type("eos")==0,"istype(eos)");
    t.test_gen(it.is_type("fermi_integral")==0,"istype(fermi_integral)");
    t.test_gen(it.is_type("fermion")==0,"istype(fermion)");
    t.test_gen(it.is_type("fit_base")==0,"istype(fit_base)");
    t.test_gen(it.is_type("fit_function_strings")==0,
    "istype(fit_function_strings)");
    t.test_gen(it.is_type("funct_string")==0,"istype(funct_string)");
    t.test_gen(it.is_type("gen_inte")==0,"istype(gen_inte)");
    t.test_gen(it.is_type("gsl_anneal")==0,"istype(gsl_anneal)");
    t.test_gen(it.is_type("gsl_astep")==0,"istype(gsl_astep)");
    //t.test_gen(it.is_type("gsl_cubic_real_coeff")==0,
    //"istype(gsl_cubic_real_coeff)");
    t.test_gen(it.is_type("gsl_deriv")==0,"istype(gsl_deriv)");
    t.test_gen(it.is_type("gsl_fermi")==0,"istype(gsl_fermi)");
    t.test_gen(it.is_type("gsl_fit")==0,"istype(gsl_fit)");
    t.test_gen(it.is_type("gsl_inte")==0,"istype(gsl_inte)");
    t.test_gen(it.is_type("gsl_inte_qag")==0,"istype(gsl_inte_qag)");
    t.test_gen(it.is_type("gsl_inte_qng")==0,"istype(gsl_inte_qng)");
    t.test_gen(it.is_type("gsl_inte_qagi")==0,"istype(gsl_inte_qagi)");
    t.test_gen(it.is_type("gsl_inte_qagil")==0,"istype(gsl_inte_qagil)");
    t.test_gen(it.is_type("gsl_inte_qagiu")==0,"istype(gsl_inte_qagiu)");
    t.test_gen(it.is_type("gsl_inte_qags")==0,"istype(gsl_inte_qags)");
    t.test_gen(it.is_type("gsl_inte_qagp")==0,"istype(gsl_inte_qagp)");
    t.test_gen(it.is_type("gsl_inte_qawc")==0,"istype(gsl_inte_qawc)");
    t.test_gen(it.is_type("gsl_inte_qaws")==0,"istype(gsl_inte_qaws)");
    t.test_gen(it.is_type("gsl_inte_qawo")==0,"istype(gsl_inte_qawo)");
    t.test_gen(it.is_type("gsl_inte_qawf")==0,"istype(gsl_inte_qawf)");
    t.test_gen(it.is_type("gsl_miser")==0,"istype(gsl_miser)");
    t.test_gen(it.is_type("gsl_monte")==0,"istype(gsl_monte)");
    //t.test_gen(it.is_type("gsl_poly_real_coeff")==0,
    //"istype(gsl_poly_real_coeff)");
    //t.test_gen(it.is_type("gsl_quadratic_real_coeff")==0,
    //"istype(gsl_quadratic_real_coeff)");
    //t.test_gen(it.is_type("gsl_quartic_real")==0,"istype(gsl_quartic_real)");
    //t.test_gen(it.is_type("gsl_quartic_real2")==0,
    //"istype(gsl_quartic_real2)");
    t.test_gen(it.is_type("gsl_rnga")==0,"istype(gsl_rnga)");
    t.test_gen(it.is_type("gsl_series")==0,"istype(gsl_series)");
    t.test_gen(it.is_type("gsl_vegas")==0,"istype(gsl_vegas)");
    t.test_gen(it.is_type("hadronic_eos")==0,"istype(hadronic_eos)");
    t.test_gen(it.is_type("int")==0,"istype(int)");
    t.test_gen(it.is_type("inte")==0,"istype(inte)");
    t.test_gen(it.is_type("mcarlo_inte")==0,"istype(mcarlo_inte)");
    t.test_gen(it.is_type("minimize")==0,"istype(minimize)");
    t.test_gen(it.is_type("min_fit")==0,"istype(min_fit)");
    t.test_gen(it.is_type("mm_funct_strings")==0,"istype(mm_funct_strings)");
    t.test_gen(it.is_type("mroot")==0,"istype(mroot)");
    t.test_gen(it.is_type("multi_funct_strings")==0,
    "istype(multi_funct_strings)");
    t.test_gen(it.is_type("multi_inte")==0,"istype(multi_inte)");
    t.test_gen(it.is_type("multi_min")==0,"istype(multi_min)");
    //t.test_gen(it.is_type("naive_quartic_complex")==0,
    //"istype(naive_quartic_complex)");
    //t.test_gen(it.is_type("naive_quartic_real")==0,
    //"istype(naive_quartic_real)");
    t.test_gen(it.is_type("nambujl_eos")==0,"istype(nambujl_eos)");
    t.test_gen(it.is_type("nonrel_fermion")==0,"istype(nonrel_fermion)");
    t.test_gen(it.is_type("ode_funct_strings")==0,"istype(ode_funct_strings)");
    t.test_gen(it.is_type("odestep")==0,"istype(odestep)");
    t.test_gen(it.is_type("part")==0,"istype(part)");
    //t.test_gen(it.is_type("poly_complex")==0,"istype(poly_complex)");
    //t.test_gen(it.is_type("poly_real_coeff")==0,"istype(poly_real_coeff)");
    //t.test_gen(it.is_type("quadratic_complex")==0,
    //"istype(quadratic_complex)");
    //t.test_gen(it.is_type("quadratic_real_coeff")==0,
    //"istype(quadratic_real_coeff)");
    //t.test_gen(it.is_type("quadratic_real")==0,"istype(quadratic_real)");
    //t.test_gen(it.is_type("quadratic_std_complex")==0,
    //"istype(quadratic_std_complex)");
    t.test_gen(it.is_type("quark_eos")==0,"istype(quark_eos)");
    t.test_gen(it.is_type("quark")==0,"istype(quark)");
    //t.test_gen(it.is_type("quartic_complex")==0,"istype(quartic_complex)");
    //t.test_gen(it.is_type("quartic_real_coeff")==0,
    //"istype(quartic_real_coeff)");
    //t.test_gen(it.is_type("quartic_real")==0,"istype(quartic_real)");
    t.test_gen(it.is_type("rel_boson")==0,"istype(rel_boson)");
    t.test_gen(it.is_type("rel_fermion")==0,"istype(rel_fermion)");
    t.test_gen(it.is_type("rmf_eos")==0,"istype(rmf_eos)");
    t.test_gen(it.is_type("rnga")==0,"istype(rnga)");
    t.test_gen(it.is_type("root")==0,"istype(root)");
    //t.test_gen(it.is_type("schematic_eos")==0,"istype(schematic_eos)");
    t.test_gen(it.is_type("sim_anneal")==0,"istype(sim_anneal)");
    t.test_gen(it.is_type("skyrme_eos")==0,"istype(skyrme_eos)");
    t.test_gen(it.is_type("string")==0,"istype(string)");
    t.test_gen(it.is_type("table")==0,"istype(table)");
    t.test_gen(it.is_type("thermo")==0,"istype(thermo)");
    t.test_gen(it.is_type("tov_solve")==0,"istype(tov_solve)");
    t.test_gen(it.is_type("tptr_geoseries")==0,"istype(tptr_geoseries)");
    t.test_gen(it.is_type("tptr_schedule")==0,"istype(tptr_schedule)");
    t.test_gen(it.is_type("word")==0,"istype(word)");
  */
  
  it.type_summary(&cout,true);

  ofstream tsout;
  tsout.open("collection.1.dat");
  it.type_summary(&tsout,true);
  tsout.close();

  collection co;

  // Set up test data 
  dpa=10.0;
  cpa='c';
  ipa=45;
  bpa=1;
  spa="test of a string";
  wpa="testword";

  // Set up array test data
  bool *bapa=new bool[4];
  bapa[0]=true; bapa[1]=false; bapa[2]=false; bapa[3]=true;
  char *capa=new char[4];
  capa[0]='t'; capa[1]='e'; capa[2]='s'; capa[3]='t';
  double *dapa=new double[4];
  dapa[0]=1.0; dapa[1]=2.0; dapa[2]=5.0; dapa[3]=6.0;
  int *iapa=new int[4];
  iapa[0]=6; iapa[1]=2; iapa[2]=5; iapa[3]=1;
  string *sapa=new string[4];
  sapa[0]="these strings"; sapa[1]="form another"; 
  sapa[2]="test of"; sapa[3]="the collection";
  string *wapa=new string[4];
  wapa[0]="this"; wapa[1]="is"; wapa[2]="a"; wapa[3]="test";

  // Set up matrix test data
  bool **bmpa=new bool *[3];
  bmpa[0]=new bool[2]; bmpa[1]=new bool[2]; bmpa[2]=new bool[2]; 
  bmpa[0][0]=1; bmpa[0][1]=0; bmpa[1][0]=1; bmpa[1][1]=0; 
  bmpa[2][0]=1; bmpa[2][1]=0;
  char **cmpa=new char *[3];
  cmpa[0]=new char[2]; cmpa[1]=new char[2]; cmpa[2]=new char[2]; 
  cmpa[0][0]='a'; cmpa[0][1]='b'; cmpa[1][0]='c'; cmpa[1][1]='d';
  cmpa[2][0]='e'; cmpa[2][1]='f';
  double **dmpa=new double *[3];
  dmpa[0]=new double[2]; dmpa[1]=new double[2]; dmpa[2]=new double[2];
  dmpa[0][0]=4.0; dmpa[0][1]=5.0; dmpa[1][0]=1.0; dmpa[1][1]=-1.0;
  dmpa[2][0]=3.0; dmpa[2][1]=2.0;
  int **impa=new int *[3];
  impa[0]=new int[2]; impa[1]=new int[2]; impa[2]=new int[2]; 
  impa[0][0]=9; impa[0][1]=8; impa[1][0]=7; impa[1][1]=6;
  impa[2][0]=5; impa[2][1]=4;
  string **smpa=new string *[3];
  smpa[0]=new string[2]; smpa[1]=new string[2]; smpa[2]=new string[2]; 
  smpa[0][0]="test 1"; smpa[0][1]="test 2"; smpa[1][0]="test 3"; 
  smpa[1][1]="test 4"; smpa[2][0]="test 5"; smpa[2][1]="test 6 7";
  string **wmpa=new string *[3];
  wmpa[0]=new string[2]; wmpa[1]=new string[2]; wmpa[2]=new string[2]; 
  wmpa[0][0]="test1"; wmpa[0][1]="tst2"; wmpa[1][0]="t3"; wmpa[1][1]="t4";
  wmpa[2][0]="test5"; wmpa[2][1]="t6";

  // Test add() and get() for basic types
  sname="btest";
  t.test_gen(bio.bool_io->add(co,sname,&bpa)==0,"add bool");
  bpa=co.getb(sname);
  t.test_gen(bpa==1,"bool");

  sname="ctest";
  t.test_gen(bio.char_io->add(co,sname,&cpa)==0,"add char");
  cpa=co.getc(sname);
  t.test_gen(cpa=='c',"char");
  
  sname="dtest";
  t.test_gen(bio.double_io->add(co,sname,&dpa)==0,"add double");
  dpa=co.getd(sname);
  t.test_rel(dpa,10.0,1.0e-14,"double");
  
  sname="itest";
  t.test_gen(bio.int_io->add(co,sname,&ipa)==0,"add int");
  ipa=co.geti(sname);
  t.test_gen(ipa==45,"int");

  sname="stest";
  t.test_gen(bio.string_io->add(co,sname,&spa)==0,"add string");
  spa=co.gets(sname);
  t.test_str(spa,"test of a string","string");

  sname="wtest";
  t.test_gen(bio.word_io->add(co,sname,&wpa)==0,"add word");
  wpa=co.getw(sname);
  t.test_str(wpa,"testword","string");

  size_t szz, szz2;

  // Test add() and get() for arrays
  sname="batest";
  t.test_gen(bio.bool_io->add(co,sname,bapa,4)==0,"add bool[]");
  t.test_gen(co.getb(sname,bapa,szz)==0,"get bool[]");
  t.test_gen((bapa[0])==1,"bool[]");
  t.test_gen((bapa[1])==0,"bool[]");
  t.test_gen((bapa[2])==0,"bool[]");
  t.test_gen((bapa[3])==1,"bool[]");

  sname="catest";
  t.test_gen(bio.char_io->add(co,sname,capa,4)==0,"add char[]");
  t.test_gen(co.getc(sname,capa,szz)==0,"get char[]");
  t.test_gen((capa[0])=='t',"char[]");
  t.test_gen((capa[1])=='e',"char[]");
  t.test_gen((capa[2])=='s',"char[]");
  t.test_gen((capa[3])=='t',"char[]");
  
  sname="datest";
  t.test_gen(bio.double_io->add(co,sname,dapa,4)==0,"add double[]");
  t.test_gen(co.getd(sname,dapa,szz)==0,"get double[]");
  t.test_rel(dapa[0],1.0,1.0e-14,"double[]");
  t.test_rel(dapa[1],2.0,1.0e-14,"double[]");
  t.test_rel(dapa[2],5.0,1.0e-14,"double[]");
  t.test_rel(dapa[3],6.0,1.0e-14,"double[]");
  
  sname="iatest";
  t.test_gen(bio.int_io->add(co,sname,iapa,4)==0,"add int[]");
  t.test_gen(co.geti(sname,iapa,szz)==0,"get int[]");
  t.test_gen(iapa[0]==6,"int[]");
  t.test_gen(iapa[1]==2,"int[]");
  t.test_gen(iapa[2]==5,"int[]");
  t.test_gen(iapa[3]==1,"int[]");

  sname="satest";
  t.test_gen(bio.string_io->add(co,sname,sapa,4)==0,"add string[]");
  t.test_gen(co.gets(sname,sapa,szz)==0,"get string[]");
  t.test_str(sapa[0],"these strings","string[]");
  t.test_str(sapa[1],"form another","string[]");
  t.test_str(sapa[2],"test of","string[]");
  t.test_str(sapa[3],"the collection","string[]");
  
  sname="watest";
  t.test_gen(bio.word_io->add(co,sname,wapa,4)==0,"add word[]");
  t.test_gen(co.getw(sname,wapa,szz)==0,"get word[]");
  t.test_str(wapa[0],"this","word[]");
  t.test_str(wapa[1],"is","word[]");
  t.test_str(wapa[2],"a","word[]");
  t.test_str(wapa[3],"test","word[]");
  
  // Test add() and get() for matrices
  sname="bmtest";
  t.test_gen(bio.bool_io->add_2darray(co,sname,bmpa,3,2)==0,"add bool[][]");
  t.test_gen(co.getb(sname,bmpa,szz,szz2)==0,"get bool[][]");
  t.test_gen((bmpa[0][0])==1,"bool[][]");
  t.test_gen((bmpa[0][1])==0,"bool[][]");
  t.test_gen((bmpa[1][0])==1,"bool[][]");
  t.test_gen((bmpa[1][1])==0,"bool[][]");
  t.test_gen((bmpa[2][0])==1,"bool[][]");
  t.test_gen((bmpa[2][1])==0,"bool[][]");
    
  sname="cmtest";
  t.test_gen(bio.char_io->add_2darray(co,sname,cmpa,3,2)==0,"add char[][]");
  t.test_gen(co.getc(sname,cmpa,szz,szz2)==0,"get char[][]");
  t.test_gen((cmpa[0][0])=='a',"char[][]");
  t.test_gen((cmpa[0][1])=='b',"char[][]");
  t.test_gen((cmpa[1][0])=='c',"char[][]");
  t.test_gen((cmpa[1][1])=='d',"char[][]");
  t.test_gen((cmpa[2][0])=='e',"char[][]");
  t.test_gen((cmpa[2][1])=='f',"char[][]");

  sname="dmtest";
  t.test_gen(bio.double_io->add_2darray(co,sname,dmpa,3,2)==0,
	     "add double[][]");
  t.test_gen(co.getd(sname,dmpa,szz,szz2)==0,"get double[][]");
  t.test_rel(dmpa[0][0],4.0,1.0e-14,"double[][]");
  t.test_rel(dmpa[0][1],5.0,1.0e-14,"double[][]");
  t.test_rel(dmpa[1][0],1.0,1.0e-14,"double[][]");
  t.test_rel(dmpa[1][1],-1.0,1.0e-14,"double[][]");
  t.test_rel(dmpa[2][0],3.0,1.0e-14,"double[][]");
  t.test_rel(dmpa[2][1],2.0,1.0e-14,"double[][]");
    
  sname="imtest";
  t.test_gen(bio.int_io->add_2darray(co,sname,impa,3,2)==0,"add int[][]");
  t.test_gen(co.geti(sname,impa,szz,szz2)==0,"get int[][]");
  t.test_gen(impa[0][0]==9,"int[][]");
  t.test_gen(impa[0][1]==8,"int[][]");
  t.test_gen(impa[1][0]==7,"int[][]");
  t.test_gen(impa[1][1]==6,"int[][]");
  t.test_gen(impa[2][0]==5,"int[][]");
  t.test_gen(impa[2][1]==4,"int[][]");

  sname="smtest";
  t.test_gen(bio.string_io->add_2darray(co,sname,smpa,3,2)==0,
	     "add string[][]");
  t.test_gen(co.gets(sname,smpa,szz,szz2)==0,"get string[][]");
  t.test_str(smpa[0][0],"test 1","string[][]");
  t.test_str(smpa[0][1],"test 2","string[][]");
  t.test_str(smpa[1][0],"test 3","string[][]");
  t.test_str(smpa[1][1],"test 4","string[][]");
  t.test_str(smpa[2][0],"test 5","string[][]");
  t.test_str(smpa[2][1],"test 6 7","string[][]");
    
  sname="wmtest";
  t.test_gen(bio.word_io->add_2darray(co,sname,wmpa,3,2)==0,"add word[][]");
  t.test_gen(co.getw(sname,wmpa,szz,szz2)==0,"get word[][]");
  t.test_str(wmpa[0][0],"test1","word[][]");
  t.test_str(wmpa[0][1],"tst2","word[][]");
  t.test_str(wmpa[1][0],"t3","word[][]");
  t.test_str(wmpa[1][1],"t4","word[][]");
  t.test_str(wmpa[2][0],"test5","word[][]");
  t.test_str(wmpa[2][1],"t6","word[][]");

  cout << "\nPrint:" << endl;
  text_out_file tf(&cout);
  itemp=co.fout(&tf);
  cout << endl;

  t.test_gen(itemp==0,"print1");
  
  fout.open("collection.4.dat", ios::out);
  fout << "# comment 1 - bool, string, word" << endl;
  fout << "bool btest 1" << endl;
  fout << "string stest\nohmigosh" << endl;
  fout << "word wtest ohmigosh" << endl;
  fout << "# comment 2 - bool[], string[], word[]" << endl;
  fout << "bool[] batest2 4 0 1 0 1" << endl;
  fout << "string[] satest2 4" << endl;
  fout << "this\nis\na\ntest" << endl;
  fout << "word[] watest2 4 this is a test" << endl;
  fout << "# comment 5 - word[][]" << endl;
  fout.close();
  
  //t.test_gen(co.rewrite("collection.4.dat","collection.6.dat")==0,
  //"rewrite()");

  delete[] bapa;
  delete[] capa;
  delete[] dapa;
  delete[] iapa;
  delete[] sapa;
  delete[] wapa;
  delete[] bmpa;
  delete[] cmpa;
  delete[] dmpa;
  delete[] impa;
  delete[] smpa;
  delete[] wmpa;

  fout.open("collection.5.dat", ios::out);

  fout << "# Comment - scalars" << endl;
  fout << "bool btest 1" << endl;
  fout << "char ctest z" << endl;
  fout << "double dtest 1.1" << endl;
  fout << "int itest 234" << endl;
  fout << "string stest\nohmigosh" << endl;
  fout << "word wtest ohmigosh" << endl;
  
  fout << "# Comment - vectors" << endl;
  fout << "bool[] batest 4 0 1 0 1" << endl;
  fout << "char[] catest 4 t s t 2" << endl;
  fout << "double[] datest 4 4.0 3.0 7.0 1.0" << endl;
  fout << "int[] iatest 4 7 2 4 5" << endl;
  fout << "string[] satest 4" << endl;
  fout << "this is\na longer\ntest for\nstring array" << endl;
  fout << "word[] watest 4 this is a test" << endl;

  fout << "# Comment - matrices" << endl;
  fout << "bool[][] bmtest 3 2" << endl;
  fout << "0 1\n1 0\n0 1" << endl;
  fout << "char[][] cmtest 3 2" << endl;
  fout << "a b\nd c\ne f" << endl;
  fout << "double[][] dmtest 3 2" << endl;
  fout << "1.5 -6.2\n1.3e3 4.2\n1.3e2 -5.0" << endl;
  fout << "int[][] imtest 3 2" << endl;
  fout << "1 2\n4 3\n5 6" << endl;
  fout << "string[][] smtest 3 2" << endl;
  fout << " t s t1\nt s t2 \n t s t3 \nt s t6\nt s t5\nt s t4" << endl;
  fout << "word[][] wmtest 3 2" << endl;
  fout << "this is\nthe last\ntest yeah!" << endl;
  
  fout.close();
  
  co.clear();
  text_in_file tif("collection.5.dat");
  t.test_gen(co.fin(&tif,0,10)==0,"fin()");
  t.test_gen(co.size()==18,"size()");

  cout << endl;
  co.fout(&tf);
  cout << endl;

  bpa=co.getb("btest");
  t.test_gen(bpa==1,"bool");

  cpa=co.getc("ctest");
  t.test_gen(cpa=='z',"char");

  dpa=co.getd("dtest");
  t.test_rel(dpa,1.1,1.0e-5,"double");

  ipa=co.geti("itest");
  t.test_gen(ipa==234,"int");

  spa=co.gets("stest");
  t.test_gen(spa=="ohmigosh","string");

  wpa=co.getw("wtest");
  t.test_gen(wpa=="ohmigosh","word");
  
  t.test_gen(co.getb("batest",bapa,szz)==0,"get()");
  t.test_gen(bapa[0]==0,"bool[] (1)");
  t.test_gen(bapa[1]==1,"bool[] (2)");
  t.test_gen(bapa[2]==0,"bool[] (3)");
  t.test_gen(bapa[3]==1,"bool[] (4)");
  
  t.test_gen(co.getc("catest",capa,szz)==0,"get()");
  t.test_gen(capa[0]=='t',"char[] (1)");
  t.test_gen(capa[1]=='s',"char[] (2)");
  t.test_gen(capa[2]=='t',"char[] (3)");
  t.test_gen(capa[3]=='2',"char[] (4)");
  
  t.test_gen(co.getd("datest",dapa,szz)==0,"get()");
  t.test_rel(dapa[0],4.0,1.0e-14,"double[] (1)");
  t.test_rel(dapa[1],3.0,1.0e-14,"double[] (2)");
  t.test_rel(dapa[2],7.0,1.0e-14,"double[] (3)");
  t.test_rel(dapa[3],1.0,1.0e-14,"double[] (4)");
  
  t.test_gen(co.geti("iatest",iapa,szz)==0,"get()");
  t.test_gen(iapa[0]==7,"int[] (1)");
  t.test_gen(iapa[1]==2,"int[] (2)");
  t.test_gen(iapa[2]==4,"int[] (3)");
  t.test_gen(iapa[3]==5,"int[] (4)");
  
  t.test_gen(co.gets("satest",sapa,szz)==0,"get()");
  t.test_str(sapa[0],"this is","string[] (1)");
  t.test_str(sapa[1],"a longer","string[] (2)");
  t.test_str(sapa[2],"test for","string[] (3)");
  t.test_str(sapa[3],"string array","string[] (4)");
  
  t.test_gen(co.getw("watest",wapa,szz)==0,"get()");
  t.test_str(wapa[0],"this","word[] (1)");
  t.test_str(wapa[1],"is","word[] (2)");
  t.test_str(wapa[2],"a","word[] (3)");
  t.test_str(wapa[3],"test","word[] (4)");

  t.test_gen(co.getb("bmtest",bmpa,szz,szz2)==0,
	     "get() for bool[][]");
  t.test_gen(bmpa[0][0]==0,"bool[][] (1)");
  t.test_gen(bmpa[0][1]==1,"bool[][] (2)");
  t.test_gen(bmpa[1][0]==1,"bool[][] (3)");
  t.test_gen(bmpa[1][1]==0,"bool[][] (4)");
  t.test_gen(bmpa[2][0]==0,"bool[][] (5)");
  t.test_gen(bmpa[2][1]==1,"bool[][] (6)");

  t.test_gen(co.getc("cmtest",cmpa,szz,szz2)==0,
	     "get() for char[][]");
  t.test_gen(cmpa[0][0]=='a',"char[][] (1)");
  t.test_gen(cmpa[0][1]=='b',"char[][] (2)");
  t.test_gen(cmpa[1][0]=='d',"char[][] (3)");
  t.test_gen(cmpa[1][1]=='c',"char[][] (4)");
  t.test_gen(cmpa[2][0]=='e',"char[][] (5)");
  t.test_gen(cmpa[2][1]=='f',"char[][] (6)");

  t.test_gen(co.getd("dmtest",dmpa,szz,szz2)==0,
	     "get() for double[][]");
  t.test_rel(dmpa[0][0],1.5,1.0e-14,"double[][] (1)");
  t.test_rel(dmpa[0][1],-6.2,1.0e-14,"double[][] (2)");
  t.test_rel(dmpa[1][0],1.3e3,1.0e-14,"double[][] (3)");
  t.test_rel(dmpa[1][1],4.2,1.0e-14,"double[][] (4)");
  t.test_rel(dmpa[2][0],1.3e2,1.0e-14,"double[][] (5)");
  t.test_rel(dmpa[2][1],-5.0,1.0e-14,"double[][] (6)");

  t.test_gen(co.geti("imtest",impa,szz,szz2)==0,"get() for int[][]");
  t.test_gen(impa[0][0]==1,"int[][] (1)");
  t.test_gen(impa[0][1]==2,"int[][] (2)");
  t.test_gen(impa[1][0]==4,"int[][] (3)");
  t.test_gen(impa[1][1]==3,"int[][] (4)");
  t.test_gen(impa[2][0]==5,"int[][] (5)");
  t.test_gen(impa[2][1]==6,"int[][] (6)");

  t.test_gen(co.gets("smtest",smpa,szz,szz2)==0,
	     "get() for string[][]");
  t.test_str(smpa[0][0]," t s t1","string[][] (1)");
  t.test_str(smpa[0][1],"t s t2 ","string[][] (2)");
  t.test_str(smpa[1][0]," t s t3 ","string[][] (3)");
  t.test_str(smpa[1][1],"t s t6","string[][] (4)");
  t.test_str(smpa[2][0],"t s t5","string[][] (5)");
  t.test_str(smpa[2][1],"t s t4","string[][] (6)");

  t.test_gen(co.getw("wmtest",wmpa,szz,szz2)==0,
	     "get() for word[][]");
  t.test_str(wmpa[0][0],"this","word[][] (1)");
  t.test_str(wmpa[0][1],"is","word[][] (2)");
  t.test_str(wmpa[1][0],"the","word[][] (3)");
  t.test_str(wmpa[1][1],"last","word[][] (4)");
  t.test_str(wmpa[2][0],"test","word[][] (5)");
  t.test_str(wmpa[2][1],"yeah!","word[][] (6)");
  
  int tcnt=0;
  for(collection::iterator itx=co.begin();itx!=co.end();itx++) {
    std::cout << itx->iop->type() << " " << itx.name() << " "
	      << itx->size << std::endl;
    tcnt++;
  }
  t.test_gen(tcnt==18,"iterator");

  tcnt=0;
  for(collection::type_iterator it2=co.begin("bool");
      it2!=co.end("bool");it2++) {
    tcnt++;
  }
  t.test_gen(tcnt==3,"type_iterator");
  
  co.clear();

  mypair myp, my3, *my2;
  myp.set(2.0,3.0);
  my3.set(4.0,5.0);
  myp.p=&my3;
  my3.p=NULL;
  mypair_io.add(co,"myp",&myp);
  text_out_file tofm("collection.2.dat");
  co.fout(&tofm);
  
  co.clear();
  text_in_file tifm("collection.2.dat");
  co.fin(&tifm);
  co.fout(&tf);
  co.get("myp",my2);
  double myx, myy;
  my2->get(myx,myy);
  t.test_rel(myx,2.0,1.0e-6,"user object 1");
  t.test_rel(myy,3.0,1.0e-6,"user object 2");

  // -------------------------------------------
  // Test text_file
  // -------------------------------------------

  co.clear();

  table tab(2);
  tab.line_of_names("a b c");
  tab.set("a",0,3.0);
  tab.set("b",0,1.0);
  tab.set("c",0,4.0);
  tab.set("a",1,1.0);
  tab.set("b",1,5.0);
  tab.set("c",1,9.0);
  
  table_io_type table_io;
  table_io.add(co,"t",&tab);

  mypair_io.add(co,"myp",&myp);
  
  bio.bool_io->add(co,"b",&bpa);
  bio.char_io->add(co,"c",&cpa);
  bio.double_io->add(co,"d",&dpa);
  bio.int_io->add(co,"i",&ipa);
  bio.string_io->add(co,"s",&spa);
  bio.word_io->add(co,"w",&wpa);
  
  text_out_file *tofmn=new text_out_file("collection.3.dat");
  co.fout(tofmn);
  delete tofmn;

  cout << "----------------------------------------------------" << endl;

  collection con;
  text_in_file *tifmn=new text_in_file("collection.3.dat");
  con.fin(tifmn);
  delete tifmn;
  
  tofmn=new text_out_file(&cout);
  con.fout(tofmn);
  delete tofmn;
  cout << "----------------------------------------------------" << endl;
  
  text_out_file tofl(&cout);
  co.get_type(tofl,"double","d");
  co.get_type(tofl,"string","s");
  co.set("d","12.0");
  co.get_type(tofl,"double","d");

  t.report();
  
  return 0;
}
