/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/mm_funct.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int func(size_t nv, const ovector_base &x, ovector_base &y, double &pa) {
  y[0]=pa+x[0]-x[1];
  y[1]=pa+x[0]+x[1];
  return 0;
}

int funca(size_t nv, const double x[2], double y[2], double &pa) {
  y[0]=pa+x[0]-x[1];
  y[1]=pa+x[0]+x[1];
  return 0;
}

class ac {
public:
  int mfunc(size_t nv, const ovector_base &x, ovector_base &y, double &pa) {
    y[0]=pa+x[0]-x[1];
    y[1]=pa+x[0]+x[1];
    return 0;
  }
  int mfunca(size_t nv, const double x[2], double y[2], double &pa) {
    y[0]=pa+x[0]-x[1];
    y[1]=pa+x[0]+x[1];
    return 0;
  }

};

int main(void) {
  test_mgr t;
  def_err_hnd.set_mode(2);
  t.set_output_level(2);

  mm_funct<double,ovector_base> *basep;
  mm_funct_fptr<double,ovector_base> f1(func);
  //string fs[2]={"x+a-x2","x+a+x2"};
  //mm_funct_strings<double,ovector_base> f2(2,fs,"x,x2",1,"a");
  double a=2.0;
  
  ovector ap(1);
  ap[0]=2.0;
  ovector x(2), y(2);
  x[0]=1.2;
  x[1]=3.5;

  // ---------------------------------------------------------
  // ovector section
  // ---------------------------------------------------------
  
  // Normal function pointer
  y[0]=0.0; y[1]=0.0;
  f1(2,x,y,a);
  t.test_rel(y[0],-0.3,1.0e-6,"fptr");
  t.test_rel(y[1],6.7,1.0e-6,"fptr");

  // Check that a base class pointer also works
  y[0]=0.0; y[1]=0.0;
  basep=&f1;
  (*basep)(2,x,y,a);
  t.test_rel(y[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(y[1],6.7,1.0e-6,"mfptr");

  // Strings
  /*
  y[0]=0.0; y[1]=0.0;
  f2.set_parms(ap);
  f2(2,x,y,a);
  t.test_rel(y[0],-0.3,1.0e-6,"string");
  t.test_rel(y[1],6.7,1.0e-6,"string");
  */
  
  // Check that a base class pointer also works
  /*
  y[0]=0.0; y[1]=0.0;
  basep=&f2;
  (*basep)(2,x,y,a);
  t.test_rel(y[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(y[1],6.7,1.0e-6,"mfptr");
  */

  // Member function pointer
  y[0]=0.0; y[1]=0.0;
  ac c1;
  mm_funct_mfptr<ac,double,ovector_base> f3(&c1,&ac::mfunc);
  f3(2,x,y,a);
  t.test_rel(y[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(y[1],6.7,1.0e-6,"mfptr");

  // Check that a base class pointer also works
  y[0]=0.0; y[1]=0.0;
  basep=&f3;
  (*basep)(2,x,y,a);
  t.test_rel(y[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(y[1],6.7,1.0e-6,"mfptr");
  
  // The following line will not compile under gcc:
  //  mm_funct_mfptr<ac,double,double[2]> f3a(&c1,&ac::mfunca);
  // This is the reason we need to have a separate set of classes
  // for double[nv].

  // ---------------------------------------------------------
  // double[] section
  // ---------------------------------------------------------
  
  mm_vfunct<double,2> *basepa;
  mm_vfunct_fptr<double,2> f1a(funca);
  //mm_vfunct_strings<double,2> f2a(2,fs,"x,x2",1,"a");

  double xa[2]={1.2,3.5}, ya[2], apa[1]={2.0};

  // Normal function pointer
  ya[0]=0.0; ya[1]=0.0;
  f1a(2,xa,ya,a);
  t.test_rel(ya[0],-0.3,1.0e-6,"fptr");
  t.test_rel(ya[1],6.7,1.0e-6,"fptr");

  // Check that a base class pointer also works
  ya[0]=0.0; ya[1]=0.0;
  basepa=&f1a;
  (*basepa)(2,xa,ya,a);
  t.test_rel(ya[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(ya[1],6.7,1.0e-6,"mfptr");

  // Strings
  /*
  ya[0]=0.0; ya[1]=0.0;
  f2a.set_parms(apa);
  f2a(2,xa,ya,a);
  t.test_rel(ya[0],-0.3,1.0e-6,"string");
  t.test_rel(ya[1],6.7,1.0e-6,"string");
  */
  
  // Check that a base class pointer also works
  /*
  ya[0]=0.0; ya[1]=0.0;
  basepa=&f2a;
  (*basepa)(2,xa,ya,a);
  t.test_rel(ya[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(ya[1],6.7,1.0e-6,"mfptr");
  */

  // Member function pointer
  ya[0]=0.0; ya[1]=0.0;
  mm_vfunct_mfptr<ac,double,2> f3a(&c1,&ac::mfunca);
  f3a(2,xa,ya,a);
  t.test_rel(ya[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(ya[1],6.7,1.0e-6,"mfptr");

  // Check that a base class pointer also works
  ya[0]=0.0; ya[1]=0.0;
  basepa=&f3a;
  (*basepa)(2,xa,ya,a);
  t.test_rel(ya[0],-0.3,1.0e-6,"mfptr");
  t.test_rel(ya[1],6.7,1.0e-6,"mfptr");
  
  t.report();
  return 0;
}

