/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/multi_funct.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int func(size_t nv, const ovector_base &x, double &y, double &pa);
double func2(size_t nv, const ovector_base &x, double &pa);

int func(size_t nv, const ovector_base &x, double &y, double &pa) {
  y=pa+x[0]-x[1];
  return 0;
}

double func2(size_t nv, const ovector_base &x, double &pa) {
  return pa+x[0]-x[1];
}

class ac {
public:
  int mfunc(size_t nv, const ovector_base &x, double &y, double &pa) {
    y=pa+x[0]-x[1];
    return 0;
  }
  double mfunc2(size_t nv, const ovector_base &x, double &pa) {
    return pa+x[0]-x[1];
  }
};


int vfunc(size_t nv, const double x[2], double &y, double &pa);
double vfunc2(size_t nv, const double x[2], double &pa);

int vfunc(size_t nv, const double x[2], double &y, double &pa) {
  y=pa+x[0]-x[1];
  return 0;
}

double vfunc2(size_t nv, const double x[2], double &pa) {
  return pa+x[0]-x[1];
}

class vac {
public:
  int mvfunc(size_t nv, const double x[2], double &y, double &pa) {
    y=pa+x[0]-x[1];
    return 0;
  }
  double mvfunc2(size_t nv, const double x[2], double &pa) {
    return pa+x[0]-x[1];
  }
};

int main(void) {
  test_mgr t;
  def_err_hnd.set_mode(2);
  t.set_output_level(2);

  // --------------------------------------------------------------------
  // test ovector versions
  // --------------------------------------------------------------------

  multi_funct_fptr<double,ovector_base> f1(func);
  multi_funct_fptr_noerr<double,ovector_base> f4(func2);
  //multi_funct_strings<double,ovector_base> f2("x+a-x2",2,"x,x2",1,"a");
  double a=2.0, y;
  ovector ap(1);
  ap[0]=2.0;
  ovector x(2);
  x[0]=1.2;
  x[1]=3.5;
  void *vp=0;
  
  y=0.0;
  f1(2,x,y,a);
  t.test_rel(y,-0.3,1.0e-6,"fptr");

  f4(2,x,y,a);
  t.test_rel(y,-0.3,1.0e-6,"fptr_noerr");
  
  /*
  y=0.0;
  cout << "Setting parms: " << endl;
  f2.set_parms(ap);
  cout << "Done." << endl;
  cout << "Evaluating function: " << endl;
  f2(2,x,y,a);
  cout << "Done." << endl;
  t.test_rel(y,-0.3,1.0e-6,"string");
  */
  
  y=0.0;
  ac c1;
  multi_funct_mfptr<ac,double,ovector_base> f3(&c1,&ac::mfunc);
  f3(2,x,y,a);
  t.test_rel(y,-0.3,1.0e-6,"mfptr");

  y=0.0;
  multi_funct_mfptr_noerr<ac,double,ovector_base> f5(&c1,&ac::mfunc2);
  f5(2,x,y,a);
  t.test_rel(y,-0.3,1.0e-6,"mfptr");

  // --------------------------------------------------------------------
  // test array versions
  // --------------------------------------------------------------------

  multi_vfunct_fptr<double,2> f12(vfunc);
  multi_vfunct_fptr_noerr<double,2> f42(vfunc2);
  //multi_vfunct_strings<double,2> f22("x+a-x2",2,"x,x2",1,"a");
  double ap2[1];
  ap2[0]=2.0;
  double x2[2];
  x2[0]=1.2;
  x2[1]=3.5;
  
  y=0.0;
  f12(2,x2,y,a);
  t.test_rel(y,-0.3,1.0e-6,"fptr");

  f42(2,x2,y,a);
  t.test_rel(y,-0.3,1.0e-6,"fptr_noerr");
  
  /*
  y=0.0;
  f22.set_parms(ap2);
  f22(2,x2,y,a);
  t.test_rel(y,-0.3,1.0e-6,"string");
  */
  
  y=0.0;
  vac c12;
  multi_vfunct_mfptr<vac,double,2> f32(&c12,&vac::mvfunc);
  f32(2,x2,y,a);
  t.test_rel(y,-0.3,1.0e-6,"mfptr");

  y=0.0;
  multi_vfunct_mfptr_noerr<vac,double,2> f52(&c12,&vac::mvfunc2);
  f52(2,x2,y,a);
  t.test_rel(y,-0.3,1.0e-6,"mfptr");

  t.report();
  return 0;
}
