/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/search_vec.h>

#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  def_err_hnd.set_mode(2);
  t.set_output_level(2);
  size_t i;
  search_vec<ovector> ib;

  ovector x(10);

  for(i=0;i<10;i++) {
    x[i]=((double)(i));
  }

  t.test_gen(ib.ordered_lookup(-5.5,10,x)==0,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(15.5,10,x)==9,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(5.0,10,x)==5,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(5.49,10,x)==5,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(5.51,10,x)==6,"ordered_lookup (inc)");

  t.test_gen(ib.find_interval_uncons(-5.5,10,x)==0,
	     "find_interval_uncons (inc)");
  t.test_gen(ib.find_interval_uncons(15.5,10,x)==9,
	     "find_interval_uncons (inc)");
  t.test_gen(ib.find_interval_uncons(5.0,10,x)==5,
	     "find_interval_uncons (inc)");
  t.test_gen(ib.find_interval_uncons(5.5,10,x)==5,
	     "find_interval_uncons (inc)");
  t.test_gen(ib.find_interval_uncons(5.51,10,x)==5,
	     "find_interval_uncons (inc)");

  for(i=0;i<10;i++) {
    x[i]=9.0-((double)(i));
  }

  t.test_gen(ib.ordered_lookup(-5.5,10,x)==9,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(15.5,10,x)==0,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(5.0,10,x)==4,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(5.5,10,x)==3,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(5.51,10,x)==3,"ordered_lookup (dec)");

  t.test_gen(ib.find_interval_uncons(-5.5,10,x)==9,
	     "find_interval_uncons (dec)");
  t.test_gen(ib.find_interval_uncons(15.5,10,x)==0,
	     "find_interval_uncons (dec)");
  t.test_gen(ib.find_interval_uncons(5.0,10,x)==4,
	     "find_interval_uncons (dec)");
  t.test_gen(ib.find_interval_uncons(5.5,10,x)==3,
	     "find_interval_uncons (dec)");
  t.test_gen(ib.find_interval_uncons(5.51,10,x)==3,
	     "find_interval_uncons (dec)");
  
  cout.setf(ios::scientific);
  cout.setf(ios::showpos);
  cout << "Data: " << endl;
  for(i=0;i<10;i++) {
    x[i]=((double)(i));
    cout << i << " " << x[i] << endl;
  }
  cout << endl;

  cout << "find_interval(), ordered_lookup(), "
       << "find_interval_uncons(), bsearch_inc()."  << endl;
  cout << -1.0 << " " << ib.find_interval(-1.0,10,x) << " " 
       << ib.ordered_lookup(-1.0,10,x) << " " 
       << ib.find_interval_uncons(-1.0,10,x) << " " 
       << ib.bsearch_inc(-1.0,x,0,9) << endl;
  cout << 0.0 << " " << ib.find_interval(0.0,10,x) << " "
       << ib.ordered_lookup(0.0,10,x) << " "
       << ib.find_interval_uncons(0.0,10,x) << " "
       << ib.bsearch_inc(0.0,x,0,9) << endl; 
  cout << 0.9999 << " " << ib.find_interval(0.9999,10,x) << " "
       << ib.ordered_lookup(0.9999,10,x) << " "
       << ib.find_interval_uncons(0.9999,10,x) << " "
       << ib.bsearch_inc(0.9999,x,0,9) << endl; 
  cout << 1.0 << " " << ib.find_interval(1.0,10,x) << " "
       << ib.ordered_lookup(1.0,10,x) << " "
       << ib.find_interval_uncons(1.0,10,x) << " "
       << ib.bsearch_inc(1.0,x,0,9) << endl; 
  cout << 1.5 << " " << ib.find_interval(1.5,10,x) << " "
       << ib.ordered_lookup(1.5,10,x) << " "
       << ib.find_interval_uncons(1.5,10,x) << " "
       << ib.bsearch_inc(1.5,x,0,9) << endl; 
  cout << 5.0 << " " << ib.find_interval(5.0,10,x) << " "
       << ib.ordered_lookup(5.0,10,x) << " "
       << ib.find_interval_uncons(5.0,10,x) << " "
       << ib.bsearch_inc(5.0,x,0,9) << endl; 
  cout << 5.5 << " " << ib.find_interval(5.5,10,x) << " "
       << ib.ordered_lookup(5.5,10,x) << " "
       << ib.find_interval_uncons(5.5,10,x) << " "
       << ib.bsearch_inc(5.5,x,0,9) << endl; 
  cout << 8.0 << " " << ib.find_interval(8.0,10,x) << " "
       << ib.ordered_lookup(8.0,10,x) << " "
       << ib.find_interval_uncons(8.0,10,x) << " "
       << ib.bsearch_inc(8.0,x,0,9) << endl; 
  cout << 8.5 << " " << ib.find_interval(8.5,10,x) << " "
       << ib.ordered_lookup(8.5,10,x) << " "
       << ib.find_interval_uncons(8.5,10,x) << " "
       << ib.bsearch_inc(8.5,x,0,9) << endl; 
  cout << 9.0 << " " << ib.find_interval(9.0,10,x) << " "
       << ib.ordered_lookup(9.0,10,x) << " "
       << ib.find_interval_uncons(9.0,10,x) << " "
       << ib.bsearch_inc(9.0,x,0,9) << endl; 
  cout << 9.5 << " " << ib.find_interval(9.5,10,x) << " "
       << ib.ordered_lookup(9.5,10,x) << " "
       << ib.find_interval_uncons(9.5,10,x) << " "
       << ib.bsearch_inc(9.5,x,0,9) << endl; 
  cout << endl;

  cout << "Data: " << endl;
  for(i=0;i<10;i++) {
    x[i]=9.0-((double)(i));
    cout << i << " " << x[i] << endl;
  }
  cout << endl;
  
  cout << "find_interval(), ordered_lookup(), "
       << "find_interval_uncons(), bsearch_dec()." 
       << endl;
  cout << -1.0 << " " << ib.find_interval_dec(-1.0,10,x) << " " 
       << ib.ordered_lookup(-1.0,10,x) << " " 
       << ib.find_interval_uncons(-1.0,10,x) << " " 
       << ib.bsearch_dec(-1.0,x,0,9) << endl; 
  cout << 0.0 << " " << ib.find_interval_dec(0.0,10,x) << " "
       << ib.ordered_lookup(0.0,10,x) << " "
       << ib.find_interval_uncons(0.0,10,x) << " "
       << ib.bsearch_dec(0.0,x,0,9) << endl; 
  cout << 0.9999 << " " << ib.find_interval_dec(0.9999,10,x) << " "
       << ib.ordered_lookup(0.9999,10,x) << " "
       << ib.find_interval_uncons(0.9999,10,x) << " "
       << ib.bsearch_dec(0.9999,x,0,9) << endl; 
  cout << 1.0 << " " << ib.find_interval_dec(1.0,10,x) << " " 
       << ib.ordered_lookup(1.0,10,x) << " " 
       << ib.find_interval_uncons(1.0,10,x) << " " 
       << ib.bsearch_dec(1.0,x,0,9) << endl; 
  cout << 1.5 << " " << ib.find_interval_dec(1.5,10,x) << " " 
       << ib.ordered_lookup(1.5,10,x) << " " 
       << ib.find_interval_uncons(1.5,10,x) << " " 
       << ib.bsearch_dec(1.5,x,0,9) << endl; 
  cout << 5.0 << " " << ib.find_interval_dec(5.0,10,x) << " " 
       << ib.ordered_lookup(5.0,10,x) << " " 
       << ib.find_interval_uncons(5.0,10,x) << " " 
       << ib.bsearch_dec(5.0,x,0,9) << endl; 
  cout << 5.5 << " " << ib.find_interval_dec(5.5,10,x) << " " 
       << ib.ordered_lookup(5.5,10,x) << " " 
       << ib.find_interval_uncons(5.5,10,x) << " " 
       << ib.bsearch_dec(5.5,x,0,9) << endl; 
  cout << 8.0 << " " << ib.find_interval_dec(8.0,10,x) << " " 
       << ib.ordered_lookup(8.0,10,x) << " " 
       << ib.find_interval_uncons(8.0,10,x) << " " 
       << ib.bsearch_dec(8.0,x,0,9) << endl; 
  cout << 8.5 << " " << ib.find_interval_dec(8.5,10,x) << " " 
       << ib.ordered_lookup(8.5,10,x) << " " 
       << ib.find_interval_uncons(8.5,10,x) << " " 
       << ib.bsearch_dec(8.5,x,0,9) << endl; 
  cout << 9.0 << " " << ib.find_interval_dec(9.0,10,x) << " " 
       << ib.ordered_lookup(9.0,10,x) << " " 
       << ib.find_interval_uncons(9.0,10,x) << " " 
       << ib.bsearch_dec(9.0,x,0,9) << endl; 
  cout << 9.5 << " " << ib.find_interval_dec(9.5,10,x) << " " 
       << ib.ordered_lookup(9.5,10,x) << " " 
       << ib.find_interval_uncons(9.5,10,x) << " " 
       << ib.bsearch_dec(9.5,x,0,9) << endl; 
  cout << endl;

  t.report();
  return 0;
}

