/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_inte_qag.h>

using namespace std;
using namespace o2scl;

// This function oscillates quite rapidly near x=0
double testfun(double x, int &pa) {
  return -sin(1.0/(x+0.01))*pow(x+0.01,-2.0);
}

// A GSL-version of the above function
double gslfun(double x, void *pa) {
  return -sin(1.0/(x+0.01))*pow(x+0.01,-2.0);
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  def_err_hnd.set_mode(2);
  gsl_inte_qag<int ,funct_fptr_noerr<int > > it1;

  double ans, exact;
  int vp=0;

  cout.setf(ios::scientific);
  cout.precision(10);

  funct_fptr_noerr<int > tf2(testfun);

  ans=it1.integ(tf2,0.0,1.0,vp);
  exact=cos(100.0)-cos(1/1.01);
  cout << ans << " " << it1.get_error() << " " << exact << " "
       << fabs(ans-exact) << endl;
  t.test_rel(ans,exact,1.0e-8,"qag test");

  {
    double result, error;
    gsl_integration_workspace *w=gsl_integration_workspace_alloc(1000);
    gsl_function F;
    F.function=&gslfun;
    gsl_integration_qag(&F,0,1,it1.tolx,it1.tolf,1000,
			GSL_INTEG_GAUSS15,w,&result,&error); 
    gsl_integration_workspace_free(w);
    cout << ans << " " << result << endl;
    cout << it1.get_error() << " " << error << endl;
    std::string s1=double_to_ieee_string(&ans);
    std::string s2=double_to_ieee_string(&result);
    std::string s3=double_to_ieee_string(&exact);
    cout << s1 << "\n" << s2 << "\n" << s3 << endl;
  }

  cout << "Test using different keys for qag:" << endl;
  exact=cos(100.0)-cos(1/1.01);
  for(int h=1;h<=6;h++) {
    it1.set_key(h);
    if (h==6) it1.verbose=1;
    cout.precision(6);
    ans=it1.integ(tf2,0.0,1.0,vp);
    cout.precision(10);
    cout << h << " " << ans << " " << it1.get_error() << " " 
	 << exact << " " << fabs(ans-exact) << endl;
  }
  cout << endl;
  
  t.report();
  return 0;
}
