/*
  -------------------------------------------------------------------

  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner

  This file is part of O2scl.

  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <gsl/gsl_blas.h>
#include <o2scl/test_mgr.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/omatrix_tlate.h>
#include <o2scl/cblas.h>
#include <o2scl/columnify.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  def_err_hnd.set_mode(2);

  ovector v1(5), v2(5), v3(5);
  ovector g1(5), g2(5), g3(5);
  ovector h1(5), h2(5), h3(5);
  omatrix m1(5,5), gm1(5,5), hm1(5,5);
  omatrix m2(5,5), gm2(5,5), hm2(5,5);
  omatrix m3(5,5), gm3(5,5), hm3(5,5);
  double a1[5], a2[5], a3[5];
  double dm1[5][5];
  double dm2[5][5];
  double dm3[5][5];
  double res1, res2, res3, res4;

  // Test with operator[]
  {

    for(size_t i=0;i<5;i++) {
      a1[i]=sin(i+1);
      a2[i]=cos(i+1);
      a3[i]=tan(i+1);
      v1[i]=sin(i+1);
      v2[i]=cos(i+1);
      v3[i]=tan(i+1);
      g1[i]=sin(i+1);
      g2[i]=cos(i+1);
      g3[i]=tan(i+1);
      h1[i]=sin(i+1);
      h2[i]=cos(i+1);
      h3[i]=tan(i+1);
      for(size_t j=0;j<5;j++) {
	dm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm3[i][j]=0.0;
	m3[i][j]=0.0;
	gm3[i][j]=0.0;
	hm3[i][j]=0.0;
      }
    }
  
    // daxpy
    gsl_blas_daxpy(0.5,&g1,&g2);
    cblas_daxpy(5,0.5,h1.data,1,h2.data,1);
    o2scl_cblas::daxpy(5,0.5,v1,v2);
    o2scl_cblas::daxpy(5,0.5,a1,a2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"daxpy1");
    t.test_rel_arr(5,v2,h2,1.0e-12,"daxpy2");
    t.test_rel_arr(5,a2,h2,1.0e-12,"daxpy2");

    // ddot
    gsl_blas_ddot(&g1,&g2,&res1);
    res2=cblas_ddot(5,h1.data,1,h2.data,1);
    res3=o2scl_cblas::ddot(5,v1,v2);
    res4=o2scl_cblas::ddot(5,a1,a2);
    t.test_rel(res1,res2,1.0e-12,"ddot1");
    t.test_rel(res1,res3,1.0e-12,"ddot2");
    t.test_rel(res1,res4,1.0e-12,"ddot2");
  
    // dnrm2
    res1=gsl_blas_dnrm2(&g1);
    res2=cblas_dnrm2(5,h1.data,1);
    res3=o2scl_cblas::dnrm2(5,v1);
    res4=o2scl_cblas::dnrm2(5,a1);
    t.test_rel(res1,res2,1.0e-12,"dnrm21");
    t.test_rel(res1,res3,1.0e-12,"dnrm22");
    t.test_rel(res1,res4,1.0e-12,"dnrm22");
  
    // dscal
    gsl_blas_dscal(0.5,&g3);
    cblas_dscal(5,0.5,h3.data,1);
    o2scl_cblas::dscal(5,0.5,v3);
    o2scl_cblas::dscal(5,0.5,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dscal1");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dscal2");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dscal2");
  
    // dgemv, NoTrans
    gsl_blas_dgemv(CblasNoTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasNoTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas::dgemv(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,5,5,0.1,m1,v1,0.2,v2);
    o2scl_cblas::dgemv(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,0.1,dm1,a1,0.2,a2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2");
    t.test_rel_arr(5,a2,h2,1.0e-12,"dgemv2");

    // dgemv, Trans (Note that ConjTrans was not originally supported)
    gsl_blas_dgemv(CblasTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas::dgemv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasTrans,
		       5,5,0.1,m1,v1,0.2,v2);
    o2scl_cblas::dgemv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasTrans,
		       5,5,0.1,dm1,a1,0.2,a2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1t");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2t");
    t.test_rel_arr(5,a2,h2,1.0e-12,"dgemv2t");

    // dtrsv, Upper, NoTrans, NonUnit
    gsl_blas_dtrsv(CblasUpper,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasUpper,
		       o2scl_cblas::O2cblasNoTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasUpper,
		       o2scl_cblas::O2cblasNoTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1unn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2unn");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2unn");

    // dtrsv, Lower, NoTrans, NonUnit
    gsl_blas_dtrsv(CblasLower,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasLower,
		       o2scl_cblas::O2cblasNoTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasLower,
		       o2scl_cblas::O2cblasNoTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1lnn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2lnn");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2lnn");

    // dtrsv, Upper, Trans, NonUnit
    gsl_blas_dtrsv(CblasUpper,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasUpper,
		       o2scl_cblas::O2cblasTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasUpper,
		       o2scl_cblas::O2cblasTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1utn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2utn");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2utn");

    // dtrsv, Lower, Trans, NonUnit
    gsl_blas_dtrsv(CblasLower,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasLower,
		       o2scl_cblas::O2cblasTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,m1,v3);
    o2scl_cblas::dtrsv(o2scl_cblas::O2cblasRowMajor,o2scl_cblas::O2cblasLower,
		       o2scl_cblas::O2cblasTrans,o2scl_cblas::O2cblasNonUnit,
		       5,5,dm1,a3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1ltn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2ltn");
    t.test_rel_arr(5,a3,h3,1.0e-12,"dtrsv2ltn");

    // dgemm, NoTrans, NoTrans
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1ntnt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2ntnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3ntnt");
    
    // dgemm, Trans, NoTrans
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1tnt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2tnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3tnt");

    // dgemm, NoTrans, Trans
    gsl_blas_dgemm(CblasNoTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1ntt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2ntt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3ntt");

    // dgemm, Trans, Trans
    gsl_blas_dgemm(CblasTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"dgemm1tt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemm2tt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemm3tt");

    // dgemm, ColMajor, NoTrans, NoTrans
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2ntnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3ntnt");
    
    // dgemm, ColMajor, Trans, NoTrans
    cblas_dgemm(CblasColMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2tnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3tnt");

    // dgemm, ColMajor, NoTrans, Trans
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2ntt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3ntt");

    // dgemm, ColMajor, Trans, Trans
    cblas_dgemm(CblasColMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"dgemmc2tt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"dgemmc3tt");
  }

  // Test with operator()
  {

    for(size_t i=0;i<5;i++) {
      a1[i]=sin(i+1);
      a2[i]=cos(i+1);
      a3[i]=tan(i+1);
      v1[i]=sin(i+1);
      v2[i]=cos(i+1);
      v3[i]=tan(i+1);
      g1[i]=sin(i+1);
      g2[i]=cos(i+1);
      g3[i]=tan(i+1);
      h1[i]=sin(i+1);
      h2[i]=cos(i+1);
      h3[i]=tan(i+1);
      for(size_t j=0;j<5;j++) {
	dm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm1[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	m2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	gm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	hm2[i][j]=sqrt(((double)(i+1.0)))*((double)(j+1.0));
	dm3[i][j]=0.0;
	m3[i][j]=0.0;
	gm3[i][j]=0.0;
	hm3[i][j]=0.0;
      }
    }
  
    // daxpy
    gsl_blas_daxpy(0.5,&g1,&g2);
    cblas_daxpy(5,0.5,h1.data,1,h2.data,1);
    o2scl_cblas_paren::daxpy(5,0.5,v1,v2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"daxpy1");
    t.test_rel_arr(5,v2,h2,1.0e-12,"daxpy2");

    // ddot
    gsl_blas_ddot(&g1,&g2,&res1);
    res2=cblas_ddot(5,h1.data,1,h2.data,1);
    res3=o2scl_cblas_paren::ddot(5,v1,v2);
    t.test_rel(res1,res2,1.0e-12,"ddot1");
    t.test_rel(res1,res3,1.0e-12,"ddot2");
  
    // dnrm2
    res1=gsl_blas_dnrm2(&g1);
    res2=cblas_dnrm2(5,h1.data,1);
    res3=o2scl_cblas_paren::dnrm2(5,v1);
    t.test_rel(res1,res2,1.0e-12,"dnrm21");
    t.test_rel(res1,res3,1.0e-12,"dnrm22");
  
    // dscal
    gsl_blas_dscal(0.5,&g3);
    cblas_dscal(5,0.5,h3.data,1);
    o2scl_cblas_paren::dscal(5,0.5,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dscal1");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dscal2");
  
    // dgemv, NoTrans
    gsl_blas_dgemv(CblasNoTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasNoTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas_paren::dgemv(o2scl_cblas_paren::O2cblasRowMajor,
			     o2scl_cblas_paren::O2cblasNoTrans,
			     5,5,0.1,m1,v1,0.2,v2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2");

    // dgemv, Trans (Note that ConjTrans was not originally supported)
    gsl_blas_dgemv(CblasTrans,0.1,&gm1,&g1,0.2,&g2);
    cblas_dgemv(CblasRowMajor,CblasTrans,5,5,0.1,hm1.data,5,
		h1.data,1,0.2,h2.data,1);
    o2scl_cblas_paren::dgemv(o2scl_cblas_paren::O2cblasRowMajor,
			     o2scl_cblas_paren::O2cblasTrans,
			     5,5,0.1,m1,v1,0.2,v2);
    t.test_rel_arr(5,g2,h2,1.0e-12,"dgemv1t");
    t.test_rel_arr(5,v2,h2,1.0e-12,"dgemv2t");

    // dtrsv, Upper, NoTrans, NonUnit
    gsl_blas_dtrsv(CblasUpper,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::O2cblasRowMajor,
			     o2scl_cblas_paren::O2cblasUpper,
			     o2scl_cblas_paren::O2cblasNoTrans,
			     o2scl_cblas_paren::O2cblasNonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1unn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2unn");

    // dtrsv, Lower, NoTrans, NonUnit
    gsl_blas_dtrsv(CblasLower,CblasNoTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasNoTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::O2cblasRowMajor,
			     o2scl_cblas_paren::O2cblasLower,
			     o2scl_cblas_paren::O2cblasNoTrans,
			     o2scl_cblas_paren::O2cblasNonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1lnn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2lnn");

    // dtrsv, Upper, Trans, NonUnit
    gsl_blas_dtrsv(CblasUpper,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasUpper,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::O2cblasRowMajor,
			     o2scl_cblas_paren::O2cblasUpper,
			     o2scl_cblas_paren::O2cblasTrans,
			     o2scl_cblas_paren::O2cblasNonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1utn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2utn");

    // dtrsv, Lower, Trans, NonUnit
    gsl_blas_dtrsv(CblasLower,CblasTrans,CblasNonUnit,&gm1,&g3);
    cblas_dtrsv(CblasRowMajor,CblasLower,CblasTrans,CblasNonUnit,
		5,hm1.data,5,h3.data,1);
    o2scl_cblas_paren::dtrsv(o2scl_cblas_paren::O2cblasRowMajor,
			     o2scl_cblas_paren::O2cblasLower,
			     o2scl_cblas_paren::O2cblasTrans,
			     o2scl_cblas_paren::O2cblasNonUnit,
			     5,5,m1,v3);
    t.test_rel_arr(5,g3,h3,1.0e-12,"dtrsv1ltn");
    t.test_rel_arr(5,v3,h3,1.0e-12,"dtrsv2ltn");

    // dgemm, NoTrans, NoTrans
    gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1ntnt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2ntnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemm3ntnt");
    
    // dgemm, Trans, NoTrans
    gsl_blas_dgemm(CblasTrans,CblasNoTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1tnt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2tnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemm3tnt");

    // dgemm, NoTrans, Trans
    gsl_blas_dgemm(CblasNoTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1ntt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2ntt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemm3ntt");

    // dgemm, Trans, Trans
    gsl_blas_dgemm(CblasTrans,CblasTrans,0.1,&gm1,&gm2,0.2,&gm3);
    cblas_dgemm(CblasRowMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasRowMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,gm3,hm3,1.0e-12,"paren_dgemm1tt");
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemm2tt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemm3tt");

    // dgemm, ColMajor, NoTrans, NoTrans
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2ntnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemmc3ntnt");
    
    // dgemm, ColMajor, Trans, NoTrans
    cblas_dgemm(CblasColMajor,CblasTrans,CblasNoTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasNoTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2tnt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemmc3tnt");

    // dgemm, ColMajor, NoTrans, Trans
    cblas_dgemm(CblasColMajor,CblasNoTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasNoTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2ntt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemmc3ntt");

    // dgemm, ColMajor, Trans, Trans
    cblas_dgemm(CblasColMajor,CblasTrans,CblasTrans,5,5,5,0.1,hm1.data,5,
		hm2.data,5,0.2,hm3.data,5);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,m1,m2,0.2,m3);
    o2scl_cblas::dgemm(o2scl_cblas::O2cblasColMajor,
		       o2scl_cblas::O2cblasTrans,
		       o2scl_cblas::O2cblasTrans,
		       5,5,5,0.1,dm1,dm2,0.2,dm3);
    t.test_rel_mat(5,5,m3,hm3,1.0e-12,"paren_dgemmc2tt");
    t.test_rel_mat(5,5,dm3,hm3,1.0e-12,"paren_dgemmc3tt");
  }

  t.report();
  return 0;
}

