/*
  -------------------------------------------------------------------

  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner

  This file is part of O2scl.

  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.

  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <iostream>

#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>

#include <o2scl/lu.h>
#include <o2scl/test_mgr.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/omatrix_tlate.h>
#include <o2scl/columnify.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  def_err_hnd.set_mode(2);

  cout.setf(ios::scientific);
  
  if (lib_settings.range_check()) {
    cout << "O2scl range checking on." << endl;
  } else {
    cout << "O2scl range checking off." << endl;
  }

  {
    using namespace o2scl_cblas;
    using namespace o2scl_linalg;

    for(size_t ki=0;ki<10;ki++) {

      omatrix gm1(5,5), om1(5,5), gm2(5,5), om2(5,5);
      ovector gv1(5), ov1(5), gv2(5), ov2(5), gv3(5), ov3(5);
      permutation gp1(5), op1(5), mp1(5);
      double mm1[5][5], d, mv1[5], mv2[5], mm2[5][5], mv3[5];
      int sig, ret;
      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(i);
	ov1[i]=cos(i);
	mv1[i]=cos(i);
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  om1[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  mm1[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  gm2[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  om2[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  mm2[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	}
      }

      // Test LU decomposition

      gsl_linalg_LU_decomp(&gm1,&gp1,&sig);

      {
	omatrix tl(5,5), tu(5,5), ta(5,5);
	for(size_t i=0;i<5;i++) {
	  for(size_t j=0;j<5;j++) {
	    ta[i][j]=gm2[gp1[i]][j];
	    tu[i][j]=0.0;
	    if (i==j) tl[i][j]=1.0;
	    else tl[i][j]=0.0;
	    if (i>j) tl[i][j]=gm1[i][j];
	    else tu[i][j]=gm1[i][j];
	  }
	}
	omatrix lu(5,5);
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,&tl,&tu,0.0,&lu);
	
	t.test_rel_mat(5,5,lu,ta,1.0e-12,"LU decomp gsl");
      }

      LU_decomp(5,om1,op1,sig);

      {
	omatrix tl(5,5), tu(5,5), ta(5,5);
	for(size_t i=0;i<5;i++) {
	  for(size_t j=0;j<5;j++) {
	    ta[i][j]=om2[op1[i]][j];
	    tu[i][j]=0.0;
	    if (i==j) tl[i][j]=1.0;
	    else tl[i][j]=0.0;
	    if (i>j) tl[i][j]=om1[i][j];
	    else tu[i][j]=om1[i][j];
	  }
	}
	omatrix lu(5,5);
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,&tl,&tu,0.0,&lu);
      
	t.test_rel_mat(5,5,lu,ta,1.0e-12,"LU decomp o2scl1");
      
      }

      LU_decomp(5,mm1,mp1,sig);

      {
	omatrix tl(5,5), tu(5,5), ta(5,5);
	for(size_t i=0;i<5;i++) {
	  for(size_t j=0;j<5;j++) {
	    ta[i][j]=mm2[mp1[i]][j];
	    tu[i][j]=0.0;
	    if (i==j) tl[i][j]=1.0;
	    else tl[i][j]=0.0;
	    if (i>j) tl[i][j]=mm1[i][j];
	    else tu[i][j]=mm1[i][j];
	  }
	}
	omatrix lu(5,5);
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,&tl,&tu,0.0,&lu);
      
	t.test_rel_mat(5,5,lu,ta,1.0e-12,"LU decomp o2scl2");
      
      }

      // Test solve and refine

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(i+ki);
	ov1[i]=cos(i+ki);
	mv1[i]=cos(i+ki);
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=1.0/(i+j+1);
	  om1[i][j]=1.0/(i+j+1);
	  mm1[i][j]=1.0/(i+j+1);
	  gm2[i][j]=1.0/(i+j+1);
	  om2[i][j]=1.0/(i+j+1);
	  mm2[i][j]=1.0/(i+j+1);
	}
      }

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,gm1,gv1,0.0,gv2);
      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,om1,ov1,0.0,ov2);
      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,mm1,mv1,0.0,mv2);
    
      for(size_t i=0;i<5;i++) {
	gv1[i]=0.0;
	ov1[i]=0.0;
	mv1[i]=0.0;
      }

      // -------------------------------------------------

      gsl_linalg_LU_decomp(&gm1,&gp1,&sig);
      gsl_linalg_LU_solve(&gm1,&gp1,&gv2,&gv1);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,gm2,gv1,0.0,gv3);
      t.test_rel_arr(5,gv2,gv3,1.0e-10,"solve 1");
    
      for(size_t i=0;i<5;i++) gv1[i]*=1.2;

      gsl_linalg_LU_refine(&gm2,&gm1,&gp1,&gv2,&gv1,&gv3);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,gm2,gv1,0.0,gv3);
      t.test_rel_arr(5,gv2,gv3,1.0e-10,"refine 1");
    
      // -------------------------------------------------

      LU_decomp(5,om1,op1,sig);
      LU_solve(5,om1,op1,ov1,ov2);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,om2,ov1,0.0,ov3);
      t.test_rel_arr(5,ov2,ov3,1.0e-10,"solve 2");

      for(size_t i=0;i<5;i++) ov1[i]*=1.2;

      LU_refine(5,om2,om1,op1,ov2,ov1,ov3);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,om2,ov1,0.0,ov3);
      t.test_rel_arr(5,ov2,ov3,1.0e-10,"refine 1");

      // -------------------------------------------------

      LU_decomp(5,mm1,mp1,sig);
      LU_solve(5,mm1,mp1,mv1,mv2);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,mm2,mv1,0.0,mv3);
      t.test_rel_arr(5,mv2,mv3,1.0e-10,"solve 2");

      for(size_t i=0;i<5;i++) gv1[i]*=1.2;

      LU_refine(5,mm2,mm1,mp1,mv2,mv1,mv3);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,mm2,mv1,0.0,mv3);
      t.test_rel_arr(5,mv2,mv3,1.0e-10,"refine 1");
    
      // -------------------------------------------------

      // Test invert

      for(size_t i=0;i<5;i++) {
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=ki/(1.0+i+j);
	  om1[i][j]=ki/(1.0+i+j);
	  mm1[i][j]=ki/(1.0+i+j);
	  gm2[i][j]=ki/(1.0+i+j);
	  om2[i][j]=ki/(1.0+i+j);
	  mm2[i][j]=ki/(1.0+i+j);
	}
      }

      gsl_linalg_LU_decomp(&gm1,&gp1,&sig);
      gsl_linalg_LU_invert(&gm1,&gp1,&gm2);

      LU_decomp(5,om1,op1,sig);
      LU_invert<omatrix,omatrix_col>(5,om1,op1,om2);
    
      LU_decomp(5,mm1,mp1,sig);
      LU_invert<double[5][5],array_2d_col<5,5> >(5,mm1,mp1,mm2);

      t.test_rel_mat(5,5,gm2,om2,1.0e-12,"LU invert 1");
      t.test_rel_mat(5,5,gm2,mm2,1.0e-12,"LU invert 2");

    }
  }

  {
    using namespace o2scl_cblas_paren;
    using namespace o2scl_linalg_paren;

    for(size_t ki=0;ki<10;ki++) {

      omatrix gm1(5,5), om1(5,5), gm2(5,5), om2(5,5);
      ovector gv1(5), ov1(5), gv2(5), ov2(5), gv3(5), ov3(5);
      permutation gp1(5), op1(5), mp1(5);
      double mm1[5][5], d, mv1[5], mv2[5], mm2[5][5], mv3[5];
      int sig, ret;
      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(i);
	ov1[i]=cos(i);
	mv1[i]=cos(i);
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  om1[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  mm1[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  gm2[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  om2[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	  mm2[i][j]=1.0+sin(((double)ki))+sin(i)+tan(j);
	}
      }

      // Test LU decomposition

      gsl_linalg_LU_decomp(&gm1,&gp1,&sig);

      {
	omatrix tl(5,5), tu(5,5), ta(5,5);
	for(size_t i=0;i<5;i++) {
	  for(size_t j=0;j<5;j++) {
	    ta[i][j]=gm2[gp1[i]][j];
	    tu[i][j]=0.0;
	    if (i==j) tl[i][j]=1.0;
	    else tl[i][j]=0.0;
	    if (i>j) tl[i][j]=gm1[i][j];
	    else tu[i][j]=gm1[i][j];
	  }
	}
	omatrix lu(5,5);
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,&tl,&tu,0.0,&lu);

	t.test_rel_mat(5,5,lu,ta,1.0e-12,"LU decomp gsl");
      }

      LU_decomp(5,om1,op1,sig);

      {
	omatrix tl(5,5), tu(5,5), ta(5,5);
	for(size_t i=0;i<5;i++) {
	  for(size_t j=0;j<5;j++) {
	    ta[i][j]=om2[op1[i]][j];
	    tu[i][j]=0.0;
	    if (i==j) tl[i][j]=1.0;
	    else tl[i][j]=0.0;
	    if (i>j) tl[i][j]=om1[i][j];
	    else tu[i][j]=om1[i][j];
	  }
	}
	omatrix lu(5,5);
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1.0,&tl,&tu,0.0,&lu);
      
	t.test_rel_mat(5,5,lu,ta,1.0e-12,"LU decomp o2scl1");
      
      }

      // Test solve and refine

      for(size_t i=0;i<5;i++) {
	gv1[i]=cos(i+ki);
	ov1[i]=cos(i+ki);
	mv1[i]=cos(i+ki);
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=1.0/(i+j+1);
	  om1[i][j]=1.0/(i+j+1);
	  mm1[i][j]=1.0/(i+j+1);
	  gm2[i][j]=1.0/(i+j+1);
	  om2[i][j]=1.0/(i+j+1);
	  mm2[i][j]=1.0/(i+j+1);
	}
      }

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,gm1,gv1,0.0,gv2);
      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,om1,ov1,0.0,ov2);
    
      for(size_t i=0;i<5;i++) {
	gv1[i]=0.0;
	ov1[i]=0.0;
      }

      // -------------------------------------------------

      gsl_linalg_LU_decomp(&gm1,&gp1,&sig);
      gsl_linalg_LU_solve(&gm1,&gp1,&gv2,&gv1);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,gm2,gv1,0.0,gv3);
      t.test_rel_arr(5,gv2,gv3,1.0e-10,"solve 1");
    
      for(size_t i=0;i<5;i++) gv1[i]*=1.2;

      gsl_linalg_LU_refine(&gm2,&gm1,&gp1,&gv2,&gv1,&gv3);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,gm2,gv1,0.0,gv3);
      t.test_rel_arr(5,gv2,gv3,1.0e-10,"refine 1");
    
      // -------------------------------------------------

      LU_decomp(5,om1,op1,sig);
      LU_solve(5,om1,op1,ov1,ov2);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,om2,ov1,0.0,ov3);
      t.test_rel_arr(5,ov2,ov3,1.0e-10,"solve 2");

      for(size_t i=0;i<5;i++) ov1[i]*=1.2;

      LU_refine(5,om2,om1,op1,ov2,ov1,ov3);

      dgemv(O2cblasRowMajor,O2cblasNoTrans,5,5,1.0,om2,ov1,0.0,ov3);
      t.test_rel_arr(5,ov2,ov3,1.0e-10,"refine 1");

      // -------------------------------------------------

      // Test invert

      for(size_t i=0;i<5;i++) {
	for(size_t j=0;j<5;j++) {
	  gm1[i][j]=ki/(1.0+i+j);
	  om1[i][j]=ki/(1.0+i+j);
	  mm1[i][j]=ki/(1.0+i+j);
	  gm2[i][j]=ki/(1.0+i+j);
	  om2[i][j]=ki/(1.0+i+j);
	  mm2[i][j]=ki/(1.0+i+j);
	}
      }

      gsl_linalg_LU_decomp(&gm1,&gp1,&sig);
      gsl_linalg_LU_invert(&gm1,&gp1,&gm2);

      LU_decomp(5,om1,op1,sig);
      LU_invert<omatrix,omatrix_col>(5,om1,op1,om2);
    
      t.test_rel_mat(5,5,gm2,om2,1.0e-12,"LU invert 1");

    }
  }

  t.report();
  return 0;
}

