/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/multi_funct.h>
#include <o2scl/gsl_mmin_simp2.h>
#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

double minfun(size_t n, const ovector_base &x, int &pa) {
  double ret;
  ret=x[0]*x[0]+(x[1]-2.0)*(x[1]-2.0)+3.0;
  return ret;
}

double minfuna(size_t n, const double x[], int &pa) {
  double ret;
  ret=x[0]*x[0]+(x[1]-2.0)*(x[1]-2.0)+3.0;
  return ret;
}

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  def_err_hnd.set_mode(2);

  double min=0.0, min2;
  ovector x(2), x2(2);
  double xa[2];
  int vp=0;
  gsl_mmin_simp2<int ,multi_funct<int > > g;
  gsl_mmin_simp2<int ,multi_vfunct<int ,2>,double[2],double[2],
    array_alloc<double[2]> > ga;
  
  cout.setf(ios::scientific);

  multi_funct_fptr_noerr<int > mf(minfun);
  multi_vfunct_fptr_noerr<int ,2> mfa(minfuna);
  
  int ret;

  // Standard function

  x[0]=1.1;
  x[1]=0.9;
  ret=g.mmin(2,x,min,vp,mf);
  cout << min << " " << x[0] << " " << x[1] << endl;
  t.test_abs(x[0],0.0,1.0e-4,"gsl_mmin_nmsimplex 1");
  t.test_rel(x[1],2.0,2.0e-4,"gsl_mmin_nmsimplex 2");
  
  // Standard function with arrays

  xa[0]=1.1;
  xa[1]=0.9;
  ret=ga.mmin(2,xa,min,vp,mfa);
  cout << min << " " << xa[0] << " " << xa[1] << endl;
  t.test_abs(xa[0],0.0,1.0e-4,"gsl_mmin_nmsimplex 1");
  t.test_rel(xa[1],2.0,2.0e-4,"gsl_mmin_nmsimplex 2");
  
  // GSL-like interface

  x[0]=1.0;
  x[1]=1.0;
  ovector step_size(2);
  step_size[0]=0.1;
  step_size[1]=0.1;
  g.set(mf,vp,2,x,step_size);
  for(size_t it=0;it<100 && g.size>g.tolx;it++) {
    g.iterate();
  }
  x[0]=g.x[0];
  x[1]=g.x[1];
  min=g.fval;
  cout << min << " " << x[0] << " " << x[1] << endl;
  t.test_abs(x[0],0.0,1.0e-4,"gsl_mmin_nmsimplex 1");
  t.test_rel(x[1],2.0,2.0e-4,"gsl_mmin_nmsimplex 2");

  // Specify full simplex

  omatrix simp(3,2);
  simp[0][0]=1.0;
  simp[0][1]=1.0;
  simp[1][0]=1.1;
  simp[1][1]=1.1;
  simp[2][0]=2.0;
  simp[2][1]=1.0;
  ret=g.mmin_simplex(2,simp,min,vp,mf);
  cout << min << " " << x[0] << " " << x[1] << endl;
  t.test_abs(simp[0][0],0.0,1.0e-4,"gsl_mmin_nmsimplex 1");
  t.test_rel(simp[0][1],2.0,2.0e-4,"gsl_mmin_nmsimplex 2");

  t.report();
  return 0;
}
 
