/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/nuclear_mass.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_fm;

nuclear_mass_info::nuclear_mass_info() {
  
  std::string tlist[119]=
    {"n","H","He","Li","Be","B","C","N","O","F","Ne","Na","Mg","Al","Si",
     "P","S","Cl","Ar","K","Ca","Sc","Ti","V","Cr","Mn","Fe","Co","Ni",
     "Cu","Zn","Ga","Ge","As","Se","Br","Kr","Rb","Sr","Y","Zr","Nb","Mo",
     "Tc","Ru","Rh","Pd","Ag","Cd","In","Sn","Sb","Te","I","Xe","Cs","Ba",
     "La","Ce","Pr","Nd","Pm","Sm","Eu","Gd","Tb","Dy","Ho","Er","Tm",
     "Yb","Lu","Hf","Ta","W","Re","Os","Ir","Pt","Au","Hg","Tl","Pb","Bi",
     "Po","At","Rn","Fr","Ra","Ac","Th","Pa","U","Np","Pu","Am","Cm","Bk",
     "Cf","Es","Fm","Md","No","Lr","Rf","Db","Sg","Bh","Hs","Mt","Ds","Rg",
     "Uub","Uut","Uuq","Uup","Uuh","Uus","Uuo"};
  
  for(int i=0;i<nelements;i++) {
    element_list[i]=tlist[i];
    element_table.insert(make_pair(element_list[i],i));
  }

}

nuclear_mass::nuclear_mass() {
}

ame_mass::ame_mass(std::string version) {

  void *vp;
  int sz, sz2;

  string file;
  collection co;

  base_ioc bio;
  ame_entry95_io_type ame_entry95_io;
  ame_entry03_io_type ame_entry03_io;

  file_detect fd;
  if (version=="95exp") {
    file=lib_settings.get_data_dir()+"nucmass/ame95exp.o2";
  } else if (version=="95rmd") {
    file=lib_settings.get_data_dir()+"nucmass/ame95rmd.o2";
  } else if (version=="03round") {
    file=lib_settings.get_data_dir()+"nucmass/ame03round.o2";
  } else {
    file=lib_settings.get_data_dir()+"nucmass/ame03.o2";
  }
  in_file_format *iff=fd.open(file.c_str());

  if (co.fin(iff)==0) {

    size_t sz3;
    reference=co.gets("reference");
    co.gets("short_names",short_names,sz3);
    co.gets("col_names",col_names,sz3);
    
    string ame_name;
    if (version=="95exp" || version=="95rmd") {
      collection::type_iterator it=co.begin("ame_entry95");
      ame_name=it.name();
      sz=it->size;
      sz2=it->size2;
      vp=it->data;
    } else {
      collection::type_iterator it=co.begin("ame_entry03");
      ame_name=it.name();
      sz=it->size;
      sz2=it->size2;
      vp=it->data;
    }
    
    n=sz;
    mass=(ame_entry *)vp;
    
    co.disown("short_names");
    co.disown("col_names");
    co.disown(ame_name);
    co.clear();
    loaded=true;
    
  } else {
    
    std::string err="Failed to load file "+file+" in ame_mass::ame_mass().";
    O2SCL_ERR(err.c_str(),gsl_efilenotfound);
    
    loaded=false;
    
  }

  fd.close();
  
  
  return;
}

bool ame_mass::is_included(int l_Z, int l_N) {
  int i;
  ame_entry ret;
  ret.Z=0;
  ret.A=0;
  ret.N=0;
  for(i=0;i<n;i++) {
    if (mass[i].Z==l_Z && mass[i].N==l_N) {
      return true;
    }
  }
  return false;
}

ame_entry ame_mass::get_ZN(int l_Z, int l_N) {
  int i;
  ame_entry ret;
  ret.Z=0;
  ret.A=0;
  ret.N=0;
  for(i=0;i<n;i++) {
    if (mass[i].Z==l_Z && mass[i].N==l_N) {
      ret=mass[i];
    }
  }
  return ret;
}

ame_entry ame_mass::get_ZA(int l_Z, int l_A) {
  int i;
  ame_entry ret;
  ret.Z=0;
  ret.A=0;
  ret.N=0;
  for(i=0;i<n;i++) {
    if (mass[i].Z==l_Z && mass[i].A==l_A) {
      ret=mass[i];
    }
  }
  return ret;
}

ame_entry ame_mass::get_elA(string l_el, int l_A) {
  int i;
  ame_entry ret;
  ret.Z=0;
  ret.A=0;
  ret.N=0;
  for(i=0;i<n;i++) {
    if (mass[i].el==l_el && mass[i].A==l_A) {
      ret=mass[i];
    }
  }
  return ret;
}

ame_entry ame_mass::get(string nucleus) {
  if (isalpha(nucleus[1])) {
    string el=nucleus.substr(0,2);
    int A=stoi(nucleus.substr(2,nucleus.length()-2));
    return get_elA(el,A);
  } 
  string el=nucleus.substr(0,1);
  int A=stoi(nucleus.substr(1,nucleus.length()-1));
  return get_elA(el,A);
}

int ame_entry95_io_type::input(cinput *co, in_file_format *ins, ame_entry *t) {
  ins->int_in(t->NMZ);
  ins->int_in(t->N);
  ins->int_in(t->Z);
  ins->int_in(t->A);
  ins->word_in(t->el);
  ins->word_in(t->orig);
  ins->double_in(t->mass);
  ins->double_in(t->dmass);
  ins->double_in(t->be);
  ins->double_in(t->dbe);
  t->beoa=t->be/((double)t->A);
  t->dbeoa=t->dbe/((double)t->A);
  ins->word_in(t->bdmode);
  ins->double_in(t->bde);
  ins->double_in(t->dbde);
  ins->int_in(t->A2);
  ins->double_in(t->amass);
  ins->double_in(t->damass);
  return 0;
}

int ame_entry95_io_type::output(coutput *co, out_file_format *outs, 
				ame_entry *t) {
  outs->int_out(t->NMZ);
  outs->int_out(t->N);
  outs->int_out(t->Z);
  outs->int_out(t->A);
  outs->word_out(t->el);
  outs->word_out(t->orig);
  outs->double_out(t->mass);
  outs->double_out(t->dmass);
  outs->double_out(t->be);
  outs->double_out(t->dbe);
  outs->word_out(t->bdmode);
  outs->double_out(t->bde);
  outs->double_out(t->dbde);
  outs->int_out(t->A2);
  outs->double_out(t->amass);
  outs->double_out(t->damass);
  return 0;
}

const char *ame_entry95_io_type::type() { return "ame_entry95"; }
  
int ame_entry03_io_type::input(cinput *co, in_file_format *ins, ame_entry *t) {
  ins->int_in(t->NMZ);
  ins->int_in(t->N);
  ins->int_in(t->Z);
  ins->int_in(t->A);
  ins->word_in(t->el);
  ins->word_in(t->orig);
  ins->double_in(t->mass);
  ins->double_in(t->dmass);
  ins->double_in(t->beoa);
  ins->double_in(t->dbeoa);
  t->be=t->beoa*((double)t->A);
  t->dbe=t->dbeoa*((double)t->A);
  ins->word_in(t->bdmode);
  ins->double_in(t->bde);
  ins->double_in(t->dbde);
  ins->int_in(t->A2);
  ins->double_in(t->amass);
  ins->double_in(t->damass);
  return 0;
}

int ame_entry03_io_type::output(coutput *co, out_file_format *outs, 
				ame_entry *t) {
  outs->int_out(t->NMZ);
  outs->int_out(t->N);
  outs->int_out(t->Z);
  outs->int_out(t->A);
  outs->word_out(t->el);
  outs->word_out(t->orig);
  outs->double_out(t->mass);
  outs->double_out(t->dmass);
  outs->double_out(t->beoa);
  outs->double_out(t->dbeoa);
  outs->word_out(t->bdmode);
  outs->double_out(t->bde);
  outs->double_out(t->dbde);
  outs->int_out(t->A2);
  outs->double_out(t->amass);
  outs->double_out(t->damass);
  return 0;
}

const char *ame_entry03_io_type::type() { return "ame_entry03"; }

template<> int io_tlate<mnmsk_mass_entry>::input
(cinput *co, in_file_format *ins, mnmsk_mass_entry *t) {
  ins->int_in(t->N);
  ins->int_in(t->A);
  t->Z=t->A-t->N;
  ins->double_in(t->eps2);
  ins->double_in(t->eps3);
  ins->double_in(t->eps4);
  ins->double_in(t->eps6);
  ins->double_in(t->eps6sym);
  ins->double_in(t->beta2);
  ins->double_in(t->beta3);
  ins->double_in(t->beta4);
  ins->double_in(t->beta6);
  ins->double_in(t->Emic);
  ins->double_in(t->Mth);
  ins->double_in(t->Mexp);
  ins->double_in(t->sigmaexp);
  ins->double_in(t->EmicFL);
  ins->double_in(t->MthFL);
  int N2, A2;
  ins->int_in(N2);
  ins->int_in(A2);
  if (N2!=t->N || A2!=t->A) {
    O2SCL_ERR("Sync problem in mnmsk::input().",gsl_esanity);
  }
  ins->word_in(t->spinp);
  ins->word_in(t->spinn);
  ins->double_in(t->gapp);
  ins->double_in(t->gapn);
  ins->double_in(t->be);
  ins->double_in(t->S1n);
  ins->double_in(t->S2n);
  ins->double_in(t->PA);
  ins->double_in(t->PAm1);
  ins->double_in(t->PAm2);
  ins->double_in(t->Qbeta);
  ins->double_in(t->Tbeta);
  ins->double_in(t->S1p);
  ins->double_in(t->S2p);
  ins->double_in(t->Qalpha);
  ins->double_in(t->Talpha);
  return 0;
}

template<> int io_tlate<mnmsk_mass_entry>::output
(coutput *co, out_file_format *outs, mnmsk_mass_entry *t) {
  outs->int_out(t->N);
  outs->int_out(t->A);
  outs->double_out(t->eps2);
  outs->double_out(t->eps3);
  outs->double_out(t->eps4);
  outs->double_out(t->eps6);
  outs->double_out(t->eps6sym);
  outs->double_out(t->beta2);
  outs->double_out(t->beta3);
  outs->double_out(t->beta4);
  outs->double_out(t->beta6);
  outs->double_out(t->Emic);
  outs->double_out(t->Mth);
  outs->double_out(t->Mexp);
  outs->double_out(t->sigmaexp);
  outs->double_out(t->EmicFL);
  outs->double_out(t->MthFL);
  outs->int_out(t->N);
  outs->int_out(t->A);
  outs->word_out(t->spinp);
  outs->word_out(t->spinn);
  outs->double_out(t->gapp);
  outs->double_out(t->gapn);
  outs->double_out(t->be);
  outs->double_out(t->S1n);
  outs->double_out(t->S2n);
  outs->double_out(t->PA);
  outs->double_out(t->PAm1);
  outs->double_out(t->PAm2);
  outs->double_out(t->Qbeta);
  outs->double_out(t->Tbeta);
  outs->double_out(t->S1p);
  outs->double_out(t->S2p);
  outs->double_out(t->Qalpha);
  outs->double_out(t->Talpha);
  return 0;
}

template<> const char *io_tlate<mnmsk_mass_entry>::type()
{ return "mnmsk_mass_entry"; }
  
mnmsk_mass::mnmsk_mass() {

  void *vp;
  int sz, sz2;

  string file;
  collection co;
  
  base_ioc bio;
  mnmsk_mass_entry_io_type mio;

  file_detect fd;
  file=lib_settings.get_data_dir()+"nucmass/mnmsk.o2";
  
  in_file_format *iff=fd.open(file.c_str());
  
  if (co.fin(iff)==0) {
    
    string mm_name;
    /// Need to check error condition for this part
    collection::type_iterator it=co.begin("mnmsk_mass_entry");
    mm_name=it.name();
    sz=it->size;
    sz2=it->size2;
    vp=it->data;
    
    n=sz;
    mass=(mnmsk_mass_entry *)vp;
    
    co.disown(mm_name);
    co.clear();
    
    loaded=true;
    
  } else {

    std::string err="Failed to load file "+file+
      " in mnmsk_mass::mnmsk_mass().";
    O2SCL_ERR(err.c_str(),gsl_efilenotfound);
    loaded=false;
    
  }
  
  fd.close();
  
  last=n/2;

  return;
}

bool mnmsk_mass::is_included(int l_Z, int l_N) {
  int lo=0, hi=0, mid=last;

  // binary search for the correct Z first
  if (mass[mid].Z!=l_Z) {
    if (mass[mid].Z>l_Z) {
      lo=0;
      hi=mid;
    } else {
      lo=mid;
      hi=n-1;
    }
    while (hi>lo+1) {
      int mp=(lo+hi)/2;
      if (mass[mp].Z<l_Z) {
	lo=mp;
      } else {
	hi=mp;
      }
    }
    mid=lo;
    if (mass[mid].Z!=l_Z) mid=hi;
    if (mass[mid].Z!=l_Z) {
      return false;
    }
  }

  // The cached point was the right one, so we're done
  if (mass[mid].N==l_N) {
    return true;
  }


  // Now look for the right N among all the Z's
  while (mass[mid].Z==l_Z) {
    if (mass[mid].N==l_N) {
      return true;
    } else if (mass[mid].N>l_N) {
      if (mid==0) return false;
      mid--;
    } else {
      if (mid==n-1) return false;
      mid++;
    }
  }
  
  return false;
}

bool mnmsk_mass_exp::is_included(int l_Z, int l_N) {
  int lo=0, hi=0, mid=last;

  // binary search for the correct Z first
  if (mass[mid].Z!=l_Z) {
    if (mass[mid].Z>l_Z) {
      lo=0;
      hi=mid;
    } else {
      lo=mid;
      hi=n-1;
    }
    while (hi>lo+1) {
      int mp=(lo+hi)/2;
      if (mass[mp].Z<l_Z) {
	lo=mp;
      } else {
	hi=mp;
      }
    }
    mid=lo;
    if (mass[mid].Z!=l_Z) mid=hi;
    if (mass[mid].Z!=l_Z) {
      return false;
    }
  }

  // The cached point was the right one, so we're done
  if (mass[mid].N==l_N) {
    if (fabs(mass[mid].Mexp)>1.0e-20 &&
	fabs(mass[mid].Mexp)<1.0e90) {
      return true;
    } else {
      return false;
    }
  }


  // Now look for the right N among all the Z's
  while (mass[mid].Z==l_Z) {
    if (mass[mid].N==l_N) {
      if (fabs(mass[mid].Mexp)>1.0e-20 &&
	  fabs(mass[mid].Mexp)<1.0e90) {
	return true;
      } else {
	return false;
      }
    } else if (mass[mid].N>l_N) {
      if (mid==0) return false;
      mid--;
    } else {
      if (mid==n-1) return false;
      mid++;
    }
  }
  
  return false;
}

mnmsk_mass_entry mnmsk_mass::get_ZN(int l_Z, int l_N) {
  int lo=0, hi=0, mid=last;
  
  mnmsk_mass_entry ret;
  ret.Z=0;
  ret.A=0;
  ret.N=0;
  
  // binary search for the correct Z first
  if (mass[mid].Z!=l_Z) {
    if (mass[mid].Z>l_Z) {
      lo=0;
      hi=mid;
    } else {
      lo=mid;
      hi=n-1;
    }
    while (hi>lo+1) {
      int mp=(lo+hi)/2;
      if (mass[mp].Z<l_Z) {
	lo=mp;
      } else {
	hi=mp;
      }
    }
    mid=lo;
    if (mass[mid].Z!=l_Z) mid=hi;
    if (mass[mid].Z!=l_Z) {
      O2SCL_ERR((((string)"Nuclei with Z=")+itos(l_Z) 
		 +" not found in mnmsk_mass::get_ZN().").c_str(),
		gsl_enotfound);
    }
  }

  // The cached point was the right one, so we're done
  if (mass[mid].N==l_N) {
    ret=mass[mid];
    last=mid;
    return ret;
  }

  // Now look for the right N among all the Z's
  while (mass[mid].Z==l_Z) {
    if (mass[mid].N==l_N) {
      ret=mass[mid];
      last=mid;
      return ret;
    } else if (mass[mid].N>l_N) {
      mid--;
    } else {
      mid++;
    }
  }
  
  O2SCL_ERR((((string)"Nucleus with Z=")+itos(l_Z)+" and N="+itos(l_N)
	     +" not found in mnmsk_mass::get_ZN().").c_str(),gsl_enotfound);
  return ret;
}

hfb_mass::hfb_mass(size_t model) {

  table *t;
  string file, name;
  base_ioc bio;

  if (model==2) {
    file=lib_settings.get_data_dir()+"nucmass/hfb2.o2";
  } else if (model==8) {
    file=lib_settings.get_data_dir()+"nucmass/hfb8.o2";
  } else if (model==14) {
    file=lib_settings.get_data_dir()+"nucmass/hfb14.o2";
  } else {
    O2SCL_ERR("Invalid model number in hfb_mass::hfb_mass().",
	      gsl_einval);
  }
  int ret=o2scl_input_text(file,t,name);
  
  if (ret==0) {

    n=t->get_nlines();
    mass=new hfb_mass_entry[n];
    for(int i=0;i<n;i++) {
      mass[i].Z=((int)(t->get("Z",i)+0.01));
      mass[i].A=((int)(t->get("A",i)+0.01));
      mass[i].N=mass[i].A-mass[i].Z;
      mass[i].bet2=t->get("bet2",i);
      mass[i].bet4=t->get("bet4",i);
      mass[i].Rch=t->get("Rch",i);
      mass[i].def_wig=t->get("Edef",i);
      mass[i].Sn=t->get("Sn",i);
      mass[i].Sp=t->get("Sp",i);
      mass[i].Qbet=t->get("Qbet",i);
      mass[i].Mcal=t->get("Mcal",i);
      mass[i].Err=t->get("Err",i);
    }
    
    loaded=true;
    
  } else {

    std::string err="Failed to load file "+file+
      " in hfb_mass::hfb_mass().";
    O2SCL_ERR(err.c_str(),gsl_efilenotfound);
    loaded=false;
    
  }
  
  last=n/2;

  return;
}

bool hfb_mass::is_included(int l_Z, int l_N) {
  int lo=0, hi=0, mid=last;

  // binary search for the correct Z first
  if (mass[mid].Z!=l_Z) {
    if (mass[mid].Z>l_Z) {
      lo=0;
      hi=mid;
    } else {
      lo=mid;
      hi=n-1;
    }
    while (hi>lo+1) {
      int mp=(lo+hi)/2;
      if (mass[mp].Z<l_Z) {
	lo=mp;
      } else {
	hi=mp;
      }
    }
    mid=lo;
    if (mass[mid].Z!=l_Z) mid=hi;
    if (mass[mid].Z!=l_Z) {
      return false;
    }
  }

  // The cached point was the right one, so we're done
  if (mass[mid].N==l_N) {
    return true;
  }


  // Now look for the right N among all the Z's
  while (mass[mid].Z==l_Z) {
    if (mass[mid].N==l_N) {
      return true;
    } else if (mass[mid].N>l_N) {
      if (mid==0) return false;
      mid--;
    } else {
      if (mid==n-1) return false;
      mid++;
    }
  }
  
  return false;
}

hfb_mass_entry hfb_mass::get_ZN(int l_Z, int l_N) {
  int lo=0, hi=0, mid=last;
  
  hfb_mass_entry ret;
  ret.Z=0;
  ret.A=0;
  ret.N=0;
  
  // binary search for the correct Z first
  if (mass[mid].Z!=l_Z) {
    if (mass[mid].Z>l_Z) {
      lo=0;
      hi=mid;
    } else {
      lo=mid;
      hi=n-1;
    }
    while (hi>lo+1) {
      int mp=(lo+hi)/2;
      if (mass[mp].Z<l_Z) {
	lo=mp;
      } else {
	hi=mp;
      }
    }
    mid=lo;
    if (mass[mid].Z!=l_Z) mid=hi;
    if (mass[mid].Z!=l_Z) {
      O2SCL_ERR((((string)"Nuclei with Z=")+itos(l_Z) 
		 +" not found in hfb_mass::get_ZN().").c_str(),
		gsl_enotfound);
    }
  }

  // The cached point was the right one, so we're done
  if (mass[mid].N==l_N) {
    ret=mass[mid];
    last=mid;
    return ret;
  }

  // Now look for the right N among all the Z's
  while (mass[mid].Z==l_Z) {
    if (mass[mid].N==l_N) {
      ret=mass[mid];
      last=mid;
      return ret;
    } else if (mass[mid].N>l_N) {
      mid--;
    } else {
      mid++;
    }
  }
  
  O2SCL_ERR((((string)"Nucleus with Z=")+itos(l_Z)+" and N="+itos(l_N)
	     +" not found in hfb_mass::get_ZN().").c_str(),gsl_enotfound);
  return ret;
}

