/*
  -------------------------------------------------------------------
  
  Copyright (C) 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_CONVERT_UNITS_GNU_H
#define O2SCL_CONVERT_UNITS_GNU_H

#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <o2scl/err_hnd.h>
#include <o2scl/file_format.h>
#include <o2scl/misc.h>
#include <o2scl/constants.h>
#include <o2scl/convert_units.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Convert units using a system call to GNU units

      Experimental.
  */
  class convert_units_gnu : public convert_units {

  public:

    /** \brief Return the value \c val after converting using units \c
	from and \c to
    */
    virtual double convert(std::string from, std::string to, double val) {

      // Look in cache for conversion
      std::string both=from+","+to;
      miter m3=mcache.find(both);
      if (m3!=mcache.end()) {
	return val*m3->second.c;
      }

      // Look in cache for reverse conversion
      std::string both2=to+","+from;
      m3=mcache.find(both2);
      if (m3!=mcache.end()) {
	return val/m3->second.c;
      }

      // Look for combined conversions
      for(miter m=mcache.begin();m!=mcache.end();m++) {
	if (m->second.f==from) {
	  std::string b=m->second.t+","+to;
	  miter m2=mcache.find(b);
	  if (m2!=mcache.end()) {
	    if (verbose>0) {
	      std::cout << "Using conversions: " << m->second.f << " , "
			<< m->second.t << std::endl;
	      std::cout << " (1)          and: " << m2->second.f << " , "
			<< m2->second.t << std::endl;
	    }
	    return m->second.c*m2->second.c;
	  }
	  std::string b2=to+","+m->second.t;
	  miter m4=mcache.find(b2);
	  if (m4!=mcache.end()) {
	    if (verbose>0) {
	      std::cout << "Using conversions: " << m->second.f << " , "
			<< m->second.t << std::endl;
	      std::cout << " (2)          and: " << m4->second.f << " , "
			<< m4->second.t << std::endl;
	    }
	    return m->second.c/m4->second.c;
	  }
	} else if (m->second.t==from) {
	  std::string b=m->second.f+","+to;
	  miter m2=mcache.find(b);
	  if (m2!=mcache.end()) {
	    if (verbose>0) {
	      std::cout << "Using conversions: " << m->second.f << " , "
			<< m->second.t << std::endl;
	      std::cout << " (3)          and: " << m2->second.f << " , "
			<< m2->second.t << std::endl;
	    }
	    return 1.0/m->second.c*m2->second.c;
	  }
	} else if (m->second.f==to) {
	  std::string b=m->second.t+","+from;
	  miter m2=mcache.find(b);
	  if (m2!=mcache.end()) {
	    if (verbose>0) {
	      std::cout << "Using conversions: " << m->second.f << " , "
			<< m->second.t << std::endl;
	      std::cout << " (4)          and: " << m2->second.f << " , "
			<< m2->second.t << std::endl;
	    }
	    return 1.0/m->second.c/m2->second.c;
	  }
	  std::string b2=from+","+m->second.t;
	  miter m4=mcache.find(b2);
	  if (m4!=mcache.end()) {
	    if (verbose>0) {
	      std::cout << "Using conversions: " << m->second.f << " , "
			<< m->second.t << std::endl;
	      std::cout << " (5)          and: " << m4->second.f << " , "
			<< m4->second.t << std::endl;
	    }
	    return 1.0/m->second.c*m4->second.c;
	  }
	} else if (m->second.t==to) {
	  std::string b=m->second.f+","+from;
	  miter m2=mcache.find(b);
	  if (m2!=mcache.end()) {
	    if (verbose>0) {
	      std::cout << "Using conversions: " << m->second.f << " , "
			<< m->second.t << std::endl;
	      std::cout << " (6)          and: " << m2->second.f << " , "
			<< m2->second.t << std::endl;
	    }
	    return m->second.c/m2->second.c;
	  }
	}
      }
      
      // Neither found, run the 'units' command
      std::string cmd="units '"+from+"' '"+to+"'";
      if (verbose>0) std::cout << "cmd: " << cmd << std::endl;
      FILE *ps_pipe=popen(cmd.c_str(),"r");
      
      if (!ps_pipe) {
	O2SCL_ERR("Pipe could not be opened in convert_units::convert().",
		  gsl_efailed);
	return 0.0;
      }

      char line1[80];
      int size=80;

      fgets(line1,80,ps_pipe);
      if (verbose>0) std::cout << "output: " << line1 << std::endl;

      // Read the output from the 'units' command and compute the 
      // conversion factor
      std::string s=line1, t1, t2;
      std::istringstream *ins=new std::istringstream(s);
      (*ins) >> t1 >> t2;
      delete ins;
      double conv=stod(t2);
      
      // Cleanup
      if (pclose(ps_pipe)!=0) {
	O2SCL_ERR("Pipe could not be closed in convert_units::convert().",
		  gsl_efailed);
	return 0.0;
      }
      
      // Add the newly computed conversion to the table
      unit_t ut;
      ut.f=from;
      ut.t=to;
      ut.c=conv;
      mcache.insert(make_pair(both,ut));
      
      return conv*val;
    }

  };
  
#ifndef DOXYGENP
}
#endif

#endif
