/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/gsl_series.h>

using namespace std;
using namespace o2scl;

gsl_series::gsl_series(int usize) {
  w=gsl_sum_levin_u_alloc(usize);
  wt=gsl_sum_levin_utrunc_alloc(usize);
  size=usize;
}

int gsl_series::set_size(int new_size) {
  gsl_sum_levin_u_free(w);
  gsl_sum_levin_utrunc_free(wt);
  size=new_size;
  w=gsl_sum_levin_u_alloc(size);
  wt=gsl_sum_levin_utrunc_alloc(size);
  return 0;
}

gsl_series::~gsl_series() {
  gsl_sum_levin_u_free(w);
  gsl_sum_levin_utrunc_free(wt);
}

double gsl_series::series_accel(double *x, double &err) {
  double sum;
  gsl_sum_levin_utrunc_accel(x,size,wt,&sum,&err);
  return sum;
}

double gsl_series::series_accel_err(double *x, double &err) {
  double sum;
  gsl_sum_levin_u_accel(x,size,w,&sum,&err);
  return sum;
}

template<> int io_tlate<gsl_series>::input
(cinput *co, o2scl::in_file_format *ins,  o2scl::gsl_series *gs) {
  int size;
  ins->int_in(size);
  gs->set_size(size);
  return 0;
}
template<> int io_tlate<gsl_series>::output
(coutput *co, out_file_format *outs, o2scl::gsl_series *gs) {
  outs->int_out(gs->size);
  return 0;
}
template<> const char *io_tlate<gsl_series>::type()
{ return "gsl_series"; }

