/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_PART_IOC_H
#define O2SCL_PART_IOC_H
#include <string>
#include <o2scl/base_ioc.h>
#include <o2scl/root_ioc.h>
#include <o2scl/inte_ioc.h>
#include <o2scl/other_ioc.h>
#include <o2scl/rel_boson.h>
#include <o2scl/rel_fermion.h>
#include <o2scl/nonrel_fermion.h>
#include <o2scl/eff_boson.h>
#include <o2scl/classical.h>
#include <o2scl/eff_fermion.h>
#include <o2scl/eff_quark.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** \brief Setup I/O for particle classes
      
      This class is experimental.

      \comment
      removed boson and quark types since they're now abstract
      \endcomment
  */
  class part_ioc {

  public:

    part_ioc() {
      part_io=new part_io_type
	("part_io");
      thermo_io=new thermo_io_type
	("thermo_io");
      //quark_io=new quark_io_type
      //("quark_io");
      rel_boson_io=new rel_boson_io_type
	("rel_boson_io");
      rel_fermion_io=new rel_fermion_io_type
	("rel_fermion_io");
      //boson_io=new boson_io_type
      //("boson_io");
      classical_io=new classical_io_type
	("classical_io");
      eff_boson_io=new eff_boson_io_type
	("eff_boson_io");
      eff_fermion_io=new eff_fermion_io_type
	("eff_fermion_io");
      eff_quark_io=new eff_quark_io_type
	("eff_quark_io");
      fermion_io=new fermion_io_type
	("fermion_io");
      nonrel_fermion_io=new nonrel_fermion_io_type
	("nonrel_fermion_io");
    }

    ~part_ioc() {
      delete part_io;
      delete thermo_io;
      //delete quark_io;
      delete rel_boson_io;
      delete rel_fermion_io;
      //delete boson_io;
      delete classical_io;
      delete eff_boson_io;
      delete eff_fermion_io;
      delete eff_quark_io;
      delete fermion_io;
      delete nonrel_fermion_io;
    }
 
    part_io_type *part_io;
    thermo_io_type *thermo_io;
    //quark_io_type *quark_io;
    rel_boson_io_type *rel_boson_io;
    rel_fermion_io_type *rel_fermion_io;
    //boson_io_type *boson_io;
    classical_io_type *classical_io;
    eff_boson_io_type *eff_boson_io;
    eff_fermion_io_type *eff_fermion_io;
    eff_quark_io_type *eff_quark_io;
    fermion_io_type *fermion_io;
    nonrel_fermion_io_type *nonrel_fermion_io;
    
  protected:

#ifndef DOXYGENP

    root_ioc r;
    inte_ioc i;
    other_ioc o;

#endif
  
  };

#ifndef DOXYGENP
}
#endif

#endif
