/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_RNGA_H
#define O2SCL_RNGA_H

#include <iostream>
#include <ctime>
#include <o2scl/collection.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Random number generator base

      Using virtual functions is not recommended for random
      number generators, as speed is often an important issue. In
      order to facilitate the use of templates for routines which
      require random number generators, all descendants ought to
      provides the following functions:
      - double random() - Provide a random number in [0.0,1.0)
      - unsigned long int random_int(unsigned long int n) - 
      Provide a random integer in [0,n-1]
      - unsigned long int get_max() - Return the maximum integer
      for the random number generator. The argument to random_int()
      should be less than the value returned by get_max().

      \future Consider some analog of the GSL function
      <tt>gsl_rng_uniform_pos()</tt>, i.e. as used in the GSL Monte
      Carlo classes.
      
  */
  class rnga {
  public:

    rnga() {};

    /** \brief Initialize the seed with a value taken from the computer 
	clock
	
	This is a naive seed generator which uses <tt>seed=time(0)</tt>
	to generate a seed.

	\future Figure out a better way of computing a random
	seed in a platform-independent way.
    */
    void clock_seed();

    /// Get the seed
    unsigned long int get_seed() { return seed; }

    /// Set the seed
    void set_seed(unsigned long int s) { seed=s; }

  protected:

#ifndef DOXYGEN_INTERNAL

    /// The seed
    unsigned long int seed;  

    rnga(const rnga &);
    rnga& operator=(const rnga&);

#endif
  
  };

#ifndef DOXYGENP
}
#endif

#endif
