/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/cern_root.h>
#include <o2scl/funct.h>

int gfn(double x, double &y, int &pa) {
  y=sin(x-0.2);
  return 0;
}

class cl {
public:
  
  virtual ~cl() {}

  virtual int mfn(double x, double &y, int &pa) {
    y=sin(x-0.2);
    return 0;
  }

};

using namespace std;
using namespace o2scl;

int main(void) {
  cl acl;
  double a, b;
  int i;
  int vp=0;
  size_t tmp;
  test_mgr t;

  t.set_output_level(1);

  // 1 - Non-templated access through a funct object 
  funct_mfptr<cl,int > fmf(&acl,&cl::mfn);
  cern_root<int ,funct_mfptr<cl,int > > cr1;
  tmp=clock();
  a=1.0e-5;
  b=1.0;
  cr1.solve_bkt(a,b,vp,fmf);
  t.test_rel(a,0.2,1.0e-6,"1");
    
  // 4 - Templated access through a global function pointer
  typedef int (*gfnt)(double, double &, int &);
  cern_root<int ,gfnt> cr4b;
  gfnt gfnv=&gfn;
  tmp=clock();
  a=1.0e-5;
  b=1.0;
  cr4b.solve_bkt(a,b,vp,gfnv);
  t.test_rel(a,0.2,1.0e-6,"4");

  /*
  root_io_type riot;
  cern_root<int ,funct_mfptr<cl,int > > crio;
  crio.tolx=2.0;
  collection co;
  co.test_type(t,"root",&crio,vp,1);
  root<int ,funct<int > > *rp=(root<int ,funct<int > > *)vp;
  t.test_rel(rp->tolx,crio.tolx,1.0e-6,"input(output())");
  */
  
  t.report();
  return 0;
}

