/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, 2008, 2009, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#ifndef O2SCL_GSL_MROOT_HYBRIDS_B_H
#define O2SCL_GSL_MROOT_HYBRIDS_B_H

#include <cmath>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Base functions for gsl_mroot_hybrids

      This is a trivial recasting of the functions that were in file
      scope in the GSL version of the hybrids solver. 
      
      \todoc Document the individual functions for this class, 
      and possibly rename it.
  */
  class hybrids_base {
  protected:

    /// Compute the norm of \c f
    double enorm(const gsl_vector *f);

    /// Compute the norm of \f$ \vec{f} \cdot \vec{d} \f$
    double scaled_enorm(const gsl_vector *d, const gsl_vector *f);

    /// Compute the norm of \f$ \vec{a} + \vec{b} \f$.
    double enorm_sum(const gsl_vector *a, const gsl_vector *b);
  
    /// Desc.
    double compute_actual_reduction(double fnorm, double fnorm1);

    /// Desc.
    double compute_predicted_reduction(double fnorm, double fnorm1);

    /// Compute \f$ Q^{T} f \f$
    void compute_qtf(const gsl_matrix *q, const gsl_vector *f, 
		     gsl_vector *qtf);

    /// Desc.
    int newton_direction(const gsl_matrix *r, const gsl_vector *qtf, 
			 gsl_vector *p);

    /// Desc.
    void gradient_direction(const gsl_matrix *r, const gsl_vector *qtf,
			    const gsl_vector *diag, gsl_vector *g);

    /// Desc.
    void minimum_step(double gnorm, const gsl_vector *diag, gsl_vector *g);


  };
  
#ifndef DOXYGENP
}
#endif

#endif
